@@ -18,8 +18,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
    */
 
 #ifndef __NSArray_h_GNUSTEP_BASE_INCLUDE
@@ -123,6 +122,9 @@ GS_EXPORT_CLASS
 - (GS_GENERIC_TYPE(ElementT)) objectAtIndex: (NSUInteger)index;
 
 #if OS_API_VERSION(MAC_OS_X_VERSION_10_4, GS_API_LATEST)
+/**
+ * Returns array of objects at the indexes contained in NSIndexSet.
+ */
 - (GS_GENERIC_CLASS(NSArray, ElementT) *) objectsAtIndexes:
   (NSIndexSet *)indexes;
 #endif
