@@ -25,8 +25,7 @@
    
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 */ 
 
 #ifndef __NSLock_h_GNUSTEP_BASE_INCLUDE
@@ -362,9 +361,5 @@ GS_EXPORT NSLock_error_handler  *_NSLock_error_handler;
 }
 #endif
 
-#if     !NO_GNUSTEP && !defined(GNUSTEP_BASE_INTERNAL)
-#import <GNUstepBase/NSLock+GNUstepBase.h>
-#endif
-
 #endif /* __NSLock_h_GNUSTEP_BASE_INCLUDE */
 
