@@ -16,8 +16,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 
    */
 
@@ -66,6 +65,7 @@ typedef NSUInteger NSStreamEvent;
 /**
  * NSStream is an abstract class for objects representing streams. 
  */
+GS_EXPORT_CLASS
 @interface NSStream : NSObject
 
 /**
@@ -147,6 +147,7 @@ typedef NSUInteger NSStreamEvent;
  * NSInputStream is a subclass of NSStream that provides read-only
  * stream functionality.
  */
+GS_EXPORT_CLASS
 @interface NSInputStream : NSStream
 
 /**
@@ -208,6 +209,7 @@ typedef NSUInteger NSStreamEvent;
  * NSOutputStream is a subclass of NSStream that provides
  * write-only stream functionality.
  */
+GS_EXPORT_CLASS
 @interface NSOutputStream : NSStream
 
 /**
