@@ -1,4 +1,4 @@
-/* Interface for NSXMLDTD for GNUStep
+/**Interface for NSXMLDTD for GNUStep
    Copyright (C) 2008 Free Software Foundation, Inc.
 
    Written by:  Richard Frith-Macdonald <rfm@gnu.org>
@@ -14,12 +14,11 @@
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02111 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 */
 
 #ifndef __NSXMLDTD_h_GNUSTEP_BASE_INCLUDE
@@ -39,11 +38,12 @@ extern "C" {
 /**
  * Encapsulates document type definition data.
  */
+GS_EXPORT_CLASS
 @interface NSXMLDTD : NSXMLNode
 {
 #if     GS_NONFRAGILE
 #  if	defined(GS_NSXMLDTD_IVARS)
-@public GS_NSXMLDTD_IVARS
+@public GS_NSXMLDTD_IVARS;
 #  endif
 #endif
   /* The pointer to private additional data used to avoid breaking ABI
@@ -118,7 +118,7 @@ extern "C" {
 /**
  * Replaces the child at index with another child.
  */
-- (void) replaceChildAtIndex: (NSUInteger)index withNode: (NSXMLNode*)node;
+- (void) replaceChildAtIndex: (NSUInteger)index withNode: (NSXMLNode*)theNode;
 
 /**
  * Replaces all existing children with the child nodes in the array.
