@@ -18,8 +18,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02111 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 
 */
 #import "common.h"
@@ -279,7 +278,6 @@
 	    withString: (NSString*)by
 {
   NSRange       range;
-  unsigned int  count = 0;
   unsigned int	newEnd;
   NSRange	searchRange;
 
@@ -303,7 +301,6 @@
 
       do
         {
-          count++;
           [self replaceCharactersInRange: range
                               withString: by];
 
