@@ -24,8 +24,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02111 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 */
 
 
@@ -39,7 +38,6 @@
 #import "Foundation/NSPathUtilities.h"
 #endif
 
-#import "GNUstepBase/GSLock.h"
 #import "GNUstepBase/GSMime.h"
 #import "GNUstepBase/Unicode.h"
 
@@ -747,13 +745,13 @@ GSUnicode(const unichar *chars, unsigned length,
   if (isLatin1) *isLatin1 = YES;
   while (i < length)
     {
-      if ((c = chars[i++]) > 127)
+      if (chars[i++] > 127)
         {
 	  if (isASCII) *isASCII = NO;
 	  i--;
 	  while (i < length)
 	    {
-	      if ((c = chars[i++]) > 255)
+	      if (chars[i++] > 255)
 		{
 		  if (isLatin1) *isLatin1 = NO;
 		  i--;
@@ -3100,7 +3098,7 @@ GSPrivateNativeCStringEncoding()
       char      *old;
 
       /* Take it from the system locale information.  */
-      [gnustep_global_lock lock];
+      [GSPrivateGlobalLock() lock];
       /* Initialise locale system by setting current locale from
        * environment and then resetting it.  Must be done before
        * any call to nl_langinfo()
@@ -3111,7 +3109,7 @@ GSPrivateNativeCStringEncoding()
         }
       strncpy(encbuf, nl_langinfo(CODESET), sizeof(encbuf)-1);
       encbuf[sizeof(encbuf)-1] = '\0';
-      [gnustep_global_lock unlock];
+      [GSPrivateGlobalLock() unlock];
 #else
       encbuf[0] = '\0';
 #endif
