@@ -19,8 +19,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 */
 
 
@@ -80,6 +79,15 @@
 #  define GSTemplateValue	GSSizeValue
 #  define TYPE_METHOD	sizeValue
 #  define TYPE_NAME	NSSize
+#elif TYPE_ORDER == 6
+@interface GSEdgeInsetsValue : NSValue
+{
+  NSEdgeInsets data;
+}
+@end
+#  define GSTemplateValue	GSEdgeInsetsValue
+#  define TYPE_METHOD	edgeInsetsValue
+#  define TYPE_NAME	NSEdgeInsets
 #endif
 
 @implementation GSTemplateValue
@@ -165,6 +173,11 @@
 	return YES;
       else
 	return NO;
+#elif TYPE_ORDER == 6
+      if (data.top == val.top && data.left == val.left && data.bottom == val.bottom && data.right == val.right)
+	return YES;
+      else
+	return NO;
 #endif
     }
   return NO;
@@ -214,6 +227,18 @@
   for (i = 0; i < sizeof(double); i++)
     hash += val.c[i];
   return hash;
+#elif TYPE_ORDER == 6
+  union {
+    double d;
+    unsigned char c[sizeof(double)];
+  } val;
+  NSUInteger	hash = 0;
+  unsigned int	i;
+
+  val.d = data.top + data.left + data.bottom + data.right;
+  for (i = 0; i < sizeof(double); i++)
+    hash += val.c[i];
+  return hash;
 #endif
 }
 
@@ -242,6 +267,9 @@
   return NSStringFromRect(data);
 #elif TYPE_ORDER == 5
   return NSStringFromSize(data);
+#elif TYPE_ORDER == 6
+  return [NSString stringWithFormat:@"{top = %.2f, left = %.2f, bottom = %.2f, right = %.2f}",
+		   data.top, data.left, data.bottom, data.right];
 #endif
 }
 
