@@ -18,8 +18,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
    */
 
 
@@ -235,6 +234,10 @@ static SEL	objSel;
   NSUInteger	c = [other count];
 
   GSIMapInitWithZoneAndCapacity(&map, z, c);
+  if (nil == other || other == self)
+    {
+      return self;
+    }
   if (c > 0)
     {
       NSEnumerator	*e = [other keyEnumerator];
@@ -251,21 +254,14 @@ static SEL	objSel;
 
 	  if (isProxy == YES)
 	    {
-	      k = [e nextObject];
+	      if (nil == (k = [e nextObject])) break;
 	      o = [other objectForKey: k];
 	    }
 	  else
 	    {
-	      k = (*nxtObj)(e, nxtSel);
+	      if (nil == (k = (*nxtObj)(e, nxtSel))) break;
 	      o = (*otherObj)(other, objSel, k);
 	    }
-	  k = [k copyWithZone: z];
-	  if (k == nil)
-	    {
-	      DESTROY(self);
-	      [NSException raise: NSInvalidArgumentException
-			  format: @"Tried to init dictionary with nil key"];
-	    }
 	  if (shouldCopy)
 	    {
 	      o = [o copyWithZone: z];
@@ -274,12 +270,6 @@ static SEL	objSel;
 	    {
 	      o = RETAIN(o);
 	    }
-	  if (o == nil)
-	    {
-	      DESTROY(self);
-	      [NSException raise: NSInvalidArgumentException
-			  format: @"Tried to init dictionary with nil value"];
-	    }
 
 	  node = GSIMapNodeForKey(&map, (GSIMapKey)k);
 	  if (node)
@@ -289,7 +279,8 @@ static SEL	objSel;
 	    }
 	  else
 	    {
-	      GSIMapAddPairNoRetain(&map, (GSIMapKey)k, (GSIMapVal)o);
+	      GSIMapAddPair(&map, (GSIMapKey)k, (GSIMapVal)o);
+	      RELEASE(o);
 	    }
 	}
     }
