@@ -19,8 +19,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 */
 
 #import "common.h"
@@ -42,7 +41,6 @@
 #import "Foundation/NSURLHandle.h"
 #import "Foundation/NSValue.h"
 #import "GNUstepBase/GSMime.h"
-#import "GNUstepBase/GSLock.h"
 #import "GNUstepBase/GSTLS.h"
 #import "GNUstepBase/NSData+GNUstepBase.h"
 #import "GNUstepBase/NSString+GNUstepBase.h"
@@ -575,8 +573,8 @@ debugWrite(GSHTTPURLHandle *handle, NSData *data)
 	    }
 
 	  auth = [authentication authorizationForAuthentication: nil
-							 method: method
-							   path: [u fullPath]];
+	    method: method
+	    path: [u pathWithEscapes]];
 	  /* If authentication is nil then auth will also be nil
 	   */
 	  if (auth != nil)
@@ -895,7 +893,7 @@ debugWrite(GSHTTPURLHandle *handle, NSData *data)
 
 		  auth = [authentication authorizationForAuthentication: ac
 		    method: method
-		    path: [url fullPath]];
+		    path: [url pathWithEscapes]];
 		  if (auth != nil)
 		    {
 		      [self writeProperty: auth forKey: @"Authorization"];
@@ -1103,8 +1101,7 @@ debugWrite(GSHTTPURLHandle *handle, NSData *data)
   /*
    * Set up request - differs for proxy version unless tunneling via ssl.
    */
-  path = [[[u fullPath] stringByTrimmingSpaces]
-    stringByAddingPercentEscapesUsingEncoding: NSUTF8StringEncoding];
+  path = [[u pathWithEscapes] stringByTrimmingSpaces];
   if ([path length] == 0)
     {
       path = @"/";
@@ -1175,12 +1172,12 @@ debugWrite(GSHTTPURLHandle *handle, NSData *data)
       return;
     }
 
-  in = [[NSString alloc] initWithFormat: @"(%@:%@ <-- %@:%@)",
+  ASSIGN(in, ([NSString stringWithFormat: @"(%@:%@ <-- %@:%@)",
     [sock socketLocalAddress], [sock socketLocalService],
-    [sock socketAddress], [sock socketService]];
-  out = [[NSString alloc] initWithFormat: @"(%@:%@ --> %@:%@)",
+    [sock socketAddress], [sock socketService]]));
+  ASSIGN(out, ([NSString stringWithFormat: @"(%@:%@ --> %@:%@)",
     [sock socketLocalAddress], [sock socketLocalService],
-    [sock socketAddress], [sock socketService]];
+    [sock socketAddress], [sock socketService]]));
 
   if (debug)
     {
@@ -1925,8 +1922,7 @@ debugWrite(GSHTTPURLHandle *handle, NSData *data)
 	      method = @"GET";
 	    }
 	}
-      path = [[[u fullPath] stringByTrimmingSpaces]
-        stringByAddingPercentEscapesUsingEncoding: NSUTF8StringEncoding];
+      path = [[u pathWithEscapes] stringByTrimmingSpaces];
       if ([path length] == 0)
 	{
 	  path = @"/";
