@@ -19,11 +19,12 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 */
 #import "common.h"
+#import "GSPrivate.h"
 #import "GNUstepBase/GSLocale.h"
+#import "Foundation/NSAutoreleasePool.h"
 #import "Foundation/NSDictionary.h"
 #import "Foundation/NSArray.h"
 #import "Foundation/NSLock.h"
@@ -109,7 +110,7 @@ GSDomainFromDefaultLocale(void)
   /* Protect locale access with locks to prevent multiple threads using
    * it and interfering with the buffer.
    */
-  [gnustep_global_lock lock];
+  [GSPrivateGlobalLock() lock];
 
   /**
    * Set the current locale to the system default, and backup
@@ -222,7 +223,7 @@ GSDomainFromDefaultLocale(void)
    */
   if (saved == nil)
     {
-      saved = [NSObject leak: dict];
+      [NSObject keep: AUTORELEASE([dict copy]) at: &saved];
     }
 
   /**
@@ -231,7 +232,7 @@ GSDomainFromDefaultLocale(void)
    */
   privateSetLocale(LC_ALL, backupLocale);
 
-  [gnustep_global_lock unlock];
+  [GSPrivateGlobalLock() unlock];
   return saved;
 }
 
@@ -261,6 +262,7 @@ GSLanguageFromLocale(NSString *locale)
       || [locale length] < 2)
     return @"English";
 
+  ENTER_POOL
   gbundle = [NSBundle bundleForLibrary: @"gnustep-base"];
   aliases = [gbundle pathForResource: @"Locale"
 		              ofType: @"aliases"
@@ -270,22 +272,27 @@ GSLanguageFromLocale(NSString *locale)
       NSDictionary	*dict;
 
       dict = [NSDictionary dictionaryWithContentsOfFile: aliases];
-      language = [dict objectForKey: locale];
+      language = [[dict objectForKey: locale] copy];
       if (language == nil && [locale pathExtension] != nil)
 	{
 	  locale = [locale stringByDeletingPathExtension];
           if ([locale isEqual: @"C"] || [locale isEqual: @"POSIX"])
-            return @"English";
-	  language = [dict objectForKey: locale];
+	    {
+              language = @"English";
+	    }
+	  else
+	    {
+	      language = [[dict objectForKey: locale] copy];
+	    }
 	}
       if (language == nil)
 	{
 	  locale = [locale substringWithRange: NSMakeRange(0, 2)];
-	  language = [dict objectForKey: locale];
+	  language = [[dict objectForKey: locale] copy];
 	}
     }
-
-  return language;
+  LEAVE_POOL
+  return AUTORELEASE(language);
 }
 
 NSArray *
@@ -329,14 +336,14 @@ NSString *GSDefaultLanguageLocale()
 #ifdef LC_MESSAGES
   NSString *backup;
 
-  [gnustep_global_lock lock];
+  [GSPrivateGlobalLock() lock];
 
   backup = privateSetLocale(LC_ALL, nil);
   privateSetLocale(LC_ALL, @"");
   locale = privateSetLocale(LC_MESSAGES, nil);  
   privateSetLocale(LC_ALL, backup);
 
-  [gnustep_global_lock unlock];
+  [GSPrivateGlobalLock() unlock];
 #endif
 #endif
 
