@@ -18,8 +18,7 @@
    
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
    */ 
 
 #import "common.h"
@@ -32,9 +31,6 @@
 #import "Foundation/NSRunLoop.h"
 #import "Foundation/NSTimer.h"
 #import "Foundation/NSValue.h"
-#if defined(_REENTRANT)
-#import "GNUstepBase/GSLock.h"
-#endif
 
 #import <dns_sd.h>		// Apple's DNS Service Discovery
 
