@@ -24,8 +24,7 @@
    
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
    */ 
 
 #include <sys/types.h>
@@ -40,12 +39,8 @@
 
 #if     defined(_WIN32)
 
-#if     defined(__WIN64__)
 #include <winsock2.h>
-#endif
-
 #include <io.h>
-#include <winsock2.h>
 #include <ws2tcpip.h>
 #include <wininet.h>
 #if     !defined(EAFNOSUPPORT)
@@ -53,7 +48,7 @@
 #endif
 #define BADSOCKET(X)    ((X) == INVALID_SOCKET)
 #define GSNETERROR      WSAGetLastError() 
-#define GSWOULDBLOCK    (GSNETERROR == WSAEWOULDBLOCK || GSNETERROR == WSAEINPROGRESS)
+#define GSWOULDBLOCK(X) (WSAEWOULDBLOCK == (X) || WSAEINPROGRESS == (X))
 
 #else
 
@@ -74,10 +69,10 @@
 #define INVALID_SOCKET  -1
 #define BADSOCKET(X)    ((X) < 0)
 #define GSNETERROR      errno
-#define GSWOULDBLOCK    (EINPROGRESS == errno\
- || EALREADY == errno\
- || EINTR == errno\
- || EAGAIN == errno)
+#define GSWOULDBLOCK(X) (EINPROGRESS == (X)\
+ || EALREADY == (X)\
+ || EINTR == (X)\
+ || EAGAIN == (X))
 
 #endif  /* _WIN32 */
 
