@@ -17,8 +17,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 
    */
 
@@ -141,7 +140,6 @@ static RunLoopEventType typeForStream(NSStream *aStream)
 
 + (void) initialize
 {
-  GSMakeWeakPointer(self, "delegate");
 }
 
 - (void) close
@@ -165,7 +163,7 @@ static RunLoopEventType typeForStream(NSStream *aStream)
     {
       [self close];
     }
-  GSAssignZeroingWeakPointer((void**)&_delegate, (void*)0);
+  _delegate = nil;
 }
 
 - (void) dealloc
@@ -287,26 +285,11 @@ static RunLoopEventType typeForStream(NSStream *aStream)
     || [self streamStatus] == NSStreamStatusError)
     {
       _delegateValid = NO;
-      GSAssignZeroingWeakPointer((void**)&_delegate, (void*)0);
+      _delegate = nil;
     }
   else
     {
-      if (delegate == nil)
-	{
-	  _delegate = self;
-	}
-      if (delegate == self)
-	{
-	  if (_delegate != nil && _delegate != self)
-	    {
-              GSAssignZeroingWeakPointer((void**)&_delegate, (void*)0);
-	    }
-	  _delegate = delegate;
-	}
-      else
-	{
-          GSAssignZeroingWeakPointer((void**)&_delegate, (void*)delegate);
-	}
+      _delegate = (nil == delegate) ? self : delegate;
       /* We don't want to send any events the the delegate after the
        * stream has been closed.
        */
@@ -787,7 +770,6 @@ static RunLoopEventType typeForStream(NSStream *aStream)
   if (self == [GSInputStream class])
     {
       GSObjCAddClassBehavior(self, [GSStream class]);
-      GSMakeWeakPointer(self, "delegate");
     }
 }
 
@@ -820,7 +802,6 @@ static RunLoopEventType typeForStream(NSStream *aStream)
   if (self == [GSOutputStream class])
     {
       GSObjCAddClassBehavior(self, [GSStream class]);
-      GSMakeWeakPointer(self, "delegate");
     }
 }
 
@@ -1091,7 +1072,6 @@ static RunLoopEventType typeForStream(NSStream *aStream)
 
 + (void) initialize
 {
-  GSMakeWeakPointer(self, "delegate");
 }
 
 + (id) serverStreamToAddr: (NSString*)addr port: (NSInteger)port
