@@ -16,8 +16,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 
    */
 
@@ -189,9 +188,22 @@ static gnutls_anon_client_credentials_t anoncred;
  */
 @implementation GSTLSObject
 
+static NSLock                   *certificateListLock = nil;
+static NSMutableDictionary      *certificateListCache = nil;
+
+static NSLock                   *credentialsLock = nil;
+static NSMutableDictionary      *credentialsCache = nil;
+
 static NSLock                   *fileLock = nil;
 static NSMutableDictionary      *fileMap = nil;
 
+static NSLock                   *paramsLock = nil;
+static NSMutableDictionary      *paramsCache = nil;
+
+static NSLock                   *privateKeyLock = nil;
+static NSMutableDictionary      *privateKeyCache0 = nil;
+static NSMutableDictionary      *privateKeyCache1 = nil;
+
 + (void) _defaultsChanged: (NSNotification*)n
 {
   NSBundle              *bundle;
@@ -290,6 +302,24 @@ static NSMutableDictionary      *fileMap = nil;
   gnutls_global_set_log_level(globalDebug);
 }
 
++ (void) atExit
+{
+  if ([NSObject shouldCleanUp])
+    {
+      DESTROY(certificateListLock);
+      DESTROY(certificateListCache);
+      DESTROY(credentialsLock);
+      DESTROY(credentialsCache);
+      DESTROY(fileLock);
+      DESTROY(fileMap);
+      DESTROY(paramsLock);
+      DESTROY(paramsCache);
+      DESTROY(privateKeyLock);
+      DESTROY(privateKeyCache0);
+      DESTROY(privateKeyCache1);
+    }
+}
+
 + (NSData*) dataForTLSFile: (NSString*)fileName
 {
   NSData        *result;
@@ -329,6 +359,8 @@ static NSMutableDictionary      *fileMap = nil;
         {
           beenHere = YES;
 
+	  [self registerAtExit];
+
           fileLock = [NSLock new];
           fileMap = [NSMutableDictionary new];
 
@@ -398,8 +430,6 @@ static NSMutableDictionary      *fileMap = nil;
 @end
 
 @implementation GSTLSDHParams
-static NSLock                   *paramsLock = nil;
-static NSMutableDictionary      *paramsCache = nil;
 static NSTimeInterval           paramsWhen = 0.0;
 static BOOL                     paramsGenerating = NO;
 static GSTLSDHParams            *paramsCurrent = nil;
@@ -483,7 +513,7 @@ static GSTLSDHParams            *paramsCurrent = nil;
    * thread since it's likely to be rather slow.
    */
   if (nil != paramsCurrent && NO == paramsGenerating
-    && (now = paramsWhen) > 24.0 * 60.0 * 60.0)
+    && paramsWhen > 24.0 * 60.0 * 60.0)
     {
       [NSThread detachNewThreadSelector: @selector(generate)
                                toTarget: self
@@ -497,10 +527,8 @@ static GSTLSDHParams            *paramsCurrent = nil;
   if (nil == paramsLock)
     {
       paramsLock = [NSLock new];
-      [[NSObject leakAt: &paramsLock] release];
       paramsWhen = [NSDate timeIntervalSinceReferenceDate];
       paramsCache = [NSMutableDictionary new];
-      [[NSObject leakAt: &paramsCache] release];
       [[NSNotificationCenter defaultCenter] addObserver: self
 	selector: @selector(housekeeping:)
 	name: @"GSHousekeeping" object: nil];
@@ -571,8 +599,6 @@ static GSTLSDHParams            *paramsCurrent = nil;
 
 @implementation GSTLSCertificateList
 
-static NSLock                   *certificateListLock = nil;
-static NSMutableDictionary      *certificateListCache = nil;
 
 + (void) certInfo: (gnutls_x509_crt_t)cert to: (NSMutableString*)str
 {
@@ -701,9 +727,7 @@ static NSMutableDictionary      *certificateListCache = nil;
   if (nil == certificateListLock)
     {
       certificateListLock = [NSLock new];
-      [[NSObject leakAt: &certificateListLock] release];
       certificateListCache = [NSMutableDictionary new];
-      [[NSObject leakAt: &certificateListCache] release];
       [[NSNotificationCenter defaultCenter] addObserver: self
 	selector: @selector(housekeeping:)
 	name: @"GSHousekeeping" object: nil];
@@ -758,7 +782,7 @@ static NSMutableDictionary      *certificateListCache = nil;
       if (count > 0)
         {
           time_t        now = (time_t)[[NSDate date] timeIntervalSince1970];
-          unsigned int  i = count;
+          unsigned int  i;
 
           l->crts = malloc(sizeof(gnutls_x509_crt_t) * count);
           memcpy(l->crts, crts, sizeof(gnutls_x509_crt_t) * count);
@@ -885,9 +909,6 @@ static NSMutableDictionary      *certificateListCache = nil;
 
 @implementation GSTLSPrivateKey
 
-static NSLock                   *privateKeyLock = nil;
-static NSMutableDictionary      *privateKeyCache0 = nil;
-static NSMutableDictionary      *privateKeyCache1 = nil;
 
 /* Method to purge older keys from cache.
  */
@@ -941,11 +962,8 @@ static NSMutableDictionary      *privateKeyCache1 = nil;
   if (nil == privateKeyLock)
     {
       privateKeyLock = [NSLock new];
-      [[NSObject leakAt: &privateKeyLock] release];
       privateKeyCache0 = [NSMutableDictionary new];
-      [[NSObject leakAt: &privateKeyCache0] release];
       privateKeyCache1 = [NSMutableDictionary new];
-      [[NSObject leakAt: &privateKeyCache1] release];
 
       [[NSNotificationCenter defaultCenter] addObserver: self
 	selector: @selector(housekeeping:)
@@ -1075,8 +1093,6 @@ static NSMutableDictionary      *privateKeyCache1 = nil;
 
 @implementation GSTLSCredentials
 
-static NSLock                   *credentialsLock = nil;
-static NSMutableDictionary      *credentialsCache = nil;
 
 /* Method to purge older credentials from cache.
  */
@@ -1107,9 +1123,7 @@ static NSMutableDictionary      *credentialsCache = nil;
   if (nil == credentialsLock)
     {
       credentialsLock = [NSLock new];
-      [[NSObject leakAt: &credentialsLock] release];
       credentialsCache = [NSMutableDictionary new];
-      [[NSObject leakAt: &credentialsCache] release];
 
       [[NSNotificationCenter defaultCenter] addObserver: self
 	selector: @selector(housekeeping:)
