@@ -23,8 +23,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 
    <title>NSArray class reference</title>
    $Date$ $Revision$
@@ -86,7 +85,7 @@ static Class GSPlaceholderArrayClass;
 
 static GSPlaceholderArray	*defaultPlaceholderArray;
 static NSMapTable		*placeholderMap;
-static pthread_mutex_t          placeholderLock = PTHREAD_MUTEX_INITIALIZER;
+static gs_mutex_t		placeholderLock = GS_MUTEX_INIT_STATIC;
 
 
 /**
@@ -106,8 +105,35 @@ static SEL	rlSel;
 
 + (void) atExit
 {
-  DESTROY(defaultPlaceholderArray);
-  DESTROY(placeholderMap);
+  if ([NSObject shouldCleanUp])
+    {
+      id	o;
+
+      /* The default placeholder array overrides -dealloc so we must get rid of
+       * it directly.
+       */
+      o = defaultPlaceholderArray;
+      defaultPlaceholderArray = nil;
+      NSDeallocateObject(o);
+
+      /* Deallocate all the placeholders in the map before destroying it.
+       */
+      GS_MUTEX_LOCK(placeholderLock);
+      if (placeholderMap)
+	{
+	  NSMapEnumerator   mEnum = NSEnumerateMapTable(placeholderMap);
+	  Class             c;
+	  id                o;
+
+	  while (NSNextMapEnumeratorPair(&mEnum, (void *)&c, (void *)&o))
+	    {
+	      NSDeallocateObject(o);
+	    }
+	  NSEndMapTableEnumeration(&mEnum);
+	  DESTROY(placeholderMap);
+	}
+      GS_MUTEX_UNLOCK(placeholderLock);
+    }
 }
 
 + (void) initialize
@@ -167,9 +193,9 @@ static SEL	rlSel;
 	   * locate the correct placeholder in the (lock protected)
 	   * table of placeholders.
 	   */
-	  (void)pthread_mutex_lock(&placeholderLock);
+	  GS_MUTEX_LOCK(placeholderLock);
 	  obj = (id)NSMapGet(placeholderMap, (void*)z);
-	  if (obj == nil)
+	  if (obj == nil && NO == [NSObject isExiting])
 	    {
 	      /*
 	       * There is no placeholder object for this zone, so we
@@ -178,7 +204,7 @@ static SEL	rlSel;
 	      obj = (id)NSAllocateObject(GSPlaceholderArrayClass, 0, z);
 	      NSMapInsert(placeholderMap, (void*)z, (void*)obj);
 	    }
-	  (void)pthread_mutex_unlock(&placeholderLock);
+	  GS_MUTEX_UNLOCK(placeholderLock);
 	  return obj;
 	}
     }
@@ -1146,7 +1172,7 @@ compare(id elem1, id elem2, void* context)
     }
   if (range.length == 1)
     {
-      switch (CALL_BLOCK(comparator, key, [self objectAtIndex: range.location]))
+      switch (CALL_NON_NULL_BLOCK(comparator, key, [self objectAtIndex: range.location]))
         {
           case NSOrderedSame:
             return range.location;
@@ -1212,7 +1238,7 @@ compare(id elem1, id elem2, void* context)
        * For a search from the left, we'd have the correct index anyways. Check
        * whether it's equal to the key and return NSNotFound otherwise
        */
-      return (NSOrderedSame == CALL_BLOCK(comparator,
+      return (NSOrderedSame == CALL_NON_NULL_BLOCK(comparator,
         key, [self objectAtIndex: index]) ? index : NSNotFound);
     }
   // Never reached
@@ -1858,10 +1884,10 @@ compare(id elem1, id elem2, void* context)
         }
       else // call block directly
 #     endif
-      if (CALL_BLOCK(predicate, obj, count, &shouldStop))
+      if (CALL_NON_NULL_BLOCK(predicate, obj, count, &shouldStop))
         {
           /* TODO: It would be more efficient to collect an NSRange and only
-           * pass it to the index set when CALL_BLOCK returned NO. */
+           * pass it to the index set when CALL_NON_NULL_BLOCK returned NO. */
           [set addIndex: count];
         }
       if (shouldStop)
@@ -1959,7 +1985,7 @@ compare(id elem1, id elem2, void* context)
         }
       else // call block directly
 #     endif
-      if (CALL_BLOCK(predicate, obj, count, &shouldStop))
+      if (CALL_NON_NULL_BLOCK(predicate, obj, count, &shouldStop))
         {
           index = count;
           shouldStop = YES;
@@ -2187,16 +2213,14 @@ compare(id elem1, id elem2, void* context)
 }
 
 /**
- * Removes the last object in the array.  Raises an exception if the array
- * is already empty.
+ * Removes the last object in the array if one exists (otherwise it just returns).
  */
 - (void) removeLastObject
 {
   NSUInteger	count = [self count];
 
   if (count == 0)
-    [NSException raise: NSRangeException
-		 format: @"Trying to remove from an empty array."];
+    return;
   [self removeObjectAtIndex: count-1];
 }
 
@@ -2658,7 +2682,7 @@ compare(id elem1, id elem2, void* context)
   if (self != nil)
     {
       array = anArray;
-      IF_NO_GC(RETAIN(array));
+      IF_NO_ARC(RETAIN(array);)
       pos = 0;
       get = [array methodForSelector: oaiSel];
       cnt = (NSUInteger (*)(NSArray*, SEL))[array methodForSelector: countSel];
