@@ -27,8 +27,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 
    <title>NSAttributedString class reference</title>
    $Date$ $Revision$
@@ -445,6 +444,7 @@ appendUIntData(NSMutableData *d, NSUInteger i)
 		     effectiveRange: (NSRange*)aRange
 {
   [self subclassResponsibility: _cmd];/* Primitive method! */
+  *aRange = NSMakeRange(NSNotFound, 0);
   return nil;
 }
 
