@@ -23,8 +23,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 
 
    <title>NSBundle class reference</title>
@@ -56,19 +55,25 @@
 
 #import "GSPrivate.h"
 
+/* Store the working directory at startup */
+static NSString		*_launchDirectory = nil;
+
 static NSFileManager	*
 manager()
 {
   static NSFileManager	*mgr = nil;
 
-  if (mgr == nil)
+  if (nil == mgr)
     {
       mgr = RETAIN([NSFileManager defaultManager]);
+      ASSIGN(_launchDirectory, [mgr currentDirectoryPath]);
       [[NSObject leakAt: &mgr] release];
     }
   return mgr;
 }
 
+#define	NOT_LOCALIZED	@""
+
 static NSDictionary     *langAliases = nil;
 static NSDictionary     *langCanonical = nil;
 
@@ -185,9 +190,6 @@ static NSMapTable	*_bundles = NULL;
 static NSMapTable	*_byClass = NULL;
 static NSMapTable	*_byIdentifier = NULL;
 
-/* Store the working directory at startup */
-static NSString		*_launchDirectory = nil;
-
 static NSString		*_base_version
   = OBJC_STRINGIFY(GNUSTEP_BASE_MAJOR_VERSION.GNUSTEP_BASE_MINOR_VERSION);
 
@@ -196,6 +198,8 @@ static NSString		*_base_version
  */
 static NSDictionary	*_emptyTable = nil;
 
+static NSString		*_mainBundlePath = nil;
+
 /* When we are linking in an object file, GSPrivateLoadModule calls our
    callback routine for every Class and Category loaded.  The following
    variable stores the bundle that is currently doing the loading so we know
@@ -253,14 +257,23 @@ static AAssetManager *_assetManager = NULL;
 static NSString*
 AbsolutePathOfExecutable(NSString *path, BOOL atLaunch)
 {
+  NSString	*tmp;
+
+  if (0 == [path length])
+    {
+      fprintf(stderr, "AbsolutePathOfExecutable() empty path.\n");
+      return nil;
+    }
   if (NO == [path isAbsolutePath])
     {
       NSFileManager	*mgr = manager();
       NSDictionary	*env;
       NSString		*pathlist;
-      NSString		*prefix;
-      id		patharr;
+      NSString		*prefix = nil;
+      id		pathArray;
+      NSEnumerator	*enumerator;
       NSString		*result = nil;
+      NSUInteger	index;
 
       env = [[NSProcessInfo processInfo] environment];
       pathlist = [env objectForKey: @"PATH"];
@@ -271,30 +284,53 @@ AbsolutePathOfExecutable(NSString *path, BOOL atLaunch)
 	  pathlist = [env objectForKey: @"Path"];
 	}
 #if defined(_WIN32)
-      patharr = [pathlist componentsSeparatedByString: @";"];
+      pathArray = [pathlist componentsSeparatedByString: @";"];
 #else
-      patharr = [pathlist componentsSeparatedByString: @":"];
+      pathArray = [pathlist componentsSeparatedByString: @":"];
 #endif
-      /* Add . if not already in path */
-      if ([patharr indexOfObject: @"."] == NSNotFound)
+      pathArray = AUTORELEASE([pathArray mutableCopy]);
+
+      /* The directory value '.' can be replaced by either the
+       * path to the current directory or the launch directory
+       * if it is known.  Duplicates can be removed.
+       */
+      if (atLaunch == YES)
 	{
-	  patharr = AUTORELEASE([patharr mutableCopy]);
-	  [patharr addObject: @"."];
-	}
-      patharr = [patharr objectEnumerator];
-      while (nil != (prefix = [patharr nextObject]))
-	{
-	  if ([prefix isEqual: @"."])
+	  prefix = _launchDirectory;
+	  if ([prefix length] == 0)
 	    {
-	      if (atLaunch == YES)
-		{
-		  prefix = _launchDirectory;
-		}
-	      else
-		{
-		  prefix = [mgr currentDirectoryPath];
-		}
+	      fprintf(stderr, "AbsolutePathOfExecutable() failed to get"
+		" launch directory for '%s'.\n", [path UTF8String]);
+	      return nil;
 	    }
+	}
+      if (nil == prefix)
+	{
+	  prefix = [mgr currentDirectoryPath];
+	  if ([prefix length] == 0)
+	    {
+	      fprintf(stderr, "AbsolutePathOfExecutable() failed to get"
+		" current directory for '%s'.\n", [path UTF8String]);
+	      return nil;
+	    }
+	}
+      index = [pathArray indexOfObject: @"."];
+      if (NSNotFound == index)
+	{
+	  [pathArray addObject: prefix];
+	}
+      else
+	{
+	  [pathArray replaceObjectAtIndex: index withObject: prefix];
+	}
+      while ((index = [pathArray indexOfObject: @"."]) != NSNotFound)
+	{
+	  [pathArray removeObjectAtIndex: index];
+	}
+
+      enumerator = [pathArray objectEnumerator];
+      while (nil != (prefix = [enumerator nextObject]))
+	{
 	  prefix = [prefix stringByAppendingPathComponent: path];
 	  if ([mgr isExecutableFileAtPath: prefix])
 	    {
@@ -334,10 +370,31 @@ AbsolutePathOfExecutable(NSString *path, BOOL atLaunch)
 	    }
 #endif
 	}
+      if (nil == result)
+	{
+	  fprintf(stderr, "AbsolutePathOfExecutable() unable to find '%s'"
+	    " in any of %s.\n", [path UTF8String],
+	    [[pathArray description] UTF8String]);
+	  return nil;
+	}
       path = result;
     }
+  tmp = path;
   path = [path stringByResolvingSymlinksInPath];
+  if ([path length] == 0)
+    {
+      fprintf(stderr, "AbsolutePathOfExecutable() resolving symlinks failed"
+	" for '%s'.\n", [tmp UTF8String]);
+      return nil;
+    }
+  tmp = path;
   path = [path stringByStandardizingPath];
+  if ([path length] == 0)
+    {
+      fprintf(stderr, "AbsolutePathOfExecutable() standardizing path failed"
+	" for '%s'.\n", [tmp UTF8String]);
+      return nil;
+    }
   return path;
 }
 
@@ -373,20 +430,17 @@ GSPrivateExecutablePath()
 	      executablePath = nil;
 	    }
 #endif
-	  if (executablePath == nil || [executablePath length] == 0)
+	  if ([executablePath length] == 0)
 	    {
 	      executablePath
 		= [[[NSProcessInfo processInfo] arguments] objectAtIndex: 0];
+	      if ([executablePath length] == 0)
+		{
+		  fprintf(stderr,
+		    "Unable to get executable path from NSProcessInfo.\n");
+		}
 	    }
-	  if (NO == [executablePath isAbsolutePath])
-	    {
-	      executablePath = AbsolutePathOfExecutable(executablePath, YES);
-	    }
-	  else
-	    {
-	      executablePath = [executablePath stringByResolvingSymlinksInPath];
-	      executablePath = [executablePath stringByStandardizingPath];
-	    }
+	  executablePath = AbsolutePathOfExecutable(executablePath, YES);
 	  IF_NO_ARC([executablePath retain];)
 	  beenHere = YES;
 	}
@@ -396,6 +450,213 @@ GSPrivateExecutablePath()
   return executablePath;
 }
 
+/* Try to locate resources for tool name (which is this tool) in
+ * standard places like xxx/Library/Tools/Resources/name */
+/* This could be converted into a public +bundleForTool:
+ * method.  At the moment it's only used privately
+ * to locate the main bundle for this tool.
+ */
+static inline NSString *
+_find_main_bundle_for_tool(NSString *toolName)
+{
+  NSArray 	*paths;
+  NSEnumerator 	*enumerator;
+  NSString 	*path;
+  NSString 	*tail;
+  NSFileManager *fm = manager();
+
+  /*
+   * Eliminate any base path or extensions.
+   */
+  toolName = [toolName lastPathComponent];
+  do
+    {
+      toolName = [toolName stringByDeletingPathExtension];
+    }
+  while ([[toolName pathExtension] length] > 0);
+
+  if ([toolName length] == 0)
+    {
+      return nil;
+    }
+
+  tail = [@"Tools" stringByAppendingPathComponent:
+    [@"Resources" stringByAppendingPathComponent: toolName]];
+
+  paths = NSSearchPathForDirectoriesInDomains(NSLibraryDirectory,
+    NSAllDomainsMask, YES);
+
+  enumerator = [paths objectEnumerator];
+  while ((path = [enumerator nextObject]))
+    {
+      BOOL isDir;
+      path = [path stringByAppendingPathComponent: tail];
+
+      if ([fm fileExistsAtPath: path  isDirectory: &isDir]  &&  isDir)
+	{
+	  return path;
+	}
+    }
+
+  return nil;
+}
+
+static NSString *
+_find_main_bundle_path()
+{
+  if (nil == _mainBundlePath)
+    {
+      /* We figure out the main bundle directory by examining the location
+	 of the executable on disk.  */
+      NSString *path, *s;
+
+      /* We don't know at the beginning if it's a tool or an application.  */
+      BOOL isApplication = YES;
+
+      /* Sometimes we detect that this is a non-installed tool.  That is
+       * special because we want to lookup local resources before installed
+       * ones.  Keep track of this special case in this variable.
+       */
+      BOOL isNonInstalledTool = NO;
+
+      /* If it's a tool, we will need the tool name.  Since we don't
+         know yet if it's a tool or an application, we always store
+         the executable name here - just in case it turns out it's a
+         tool.  */
+      NSString *toolName = [GSPrivateExecutablePath() lastPathComponent];
+
+#if defined(_WIN32) || defined(__CYGWIN__)
+      toolName = [toolName stringByDeletingPathExtension];
+#endif
+
+      /* Strip off the name of the executable */
+      path = [GSPrivateExecutablePath() stringByDeletingLastPathComponent];
+
+      /* We now need to chop off the extra subdirectories, the library
+	 combo and the target directory if they exist.  The executable
+	 and this library should match so that is why we can use the
+	 compiled-in settings. */
+      /* library combo */
+      s = [path lastPathComponent];
+      if ([s isEqual: library_combo])
+	{
+	  path = [path stringByDeletingLastPathComponent];
+	}
+      /* target dir */
+      s = [path lastPathComponent];
+      if ([s isEqual: gnustep_target_dir])
+	{
+	  path = [path stringByDeletingLastPathComponent];
+	}
+      /* object dir */
+      s = [path lastPathComponent];
+      if ([s hasSuffix: @"obj"])
+	{
+	  path = [path stringByDeletingLastPathComponent];
+	  /* if it has an object dir it can only be a
+             non-yet-installed tool.  */
+	  isApplication = NO;
+	  isNonInstalledTool = YES;
+	}
+
+#ifndef __ANDROID__ /* don't check suffix on Android's fake executable path */
+      if (isApplication == YES)
+	{
+	  s = [path lastPathComponent];
+
+	  if ([s hasSuffix: @".app"] == NO
+	    && [s hasSuffix: @".debug"] == NO
+	    && [s hasSuffix: @".profile"] == NO
+	    && [s hasSuffix: @".gswa"] == NO	// GNUstep Web
+	    && [s hasSuffix: @".woa"] == NO	// GNUstep Web
+	    )
+	    {
+	      NSFileManager	*mgr = manager();
+	      BOOL		f;
+
+	      /* Not one of the common extensions, but
+	       * might be an app wrapper with another extension...
+	       * Look for Info-gnustep.plist or Info.plist in a
+	       * Resources subdirectory.
+	       */
+	      s = [path stringByAppendingPathComponent: @"Resources"];
+	      if ([mgr fileExistsAtPath: s isDirectory: &f] == NO || f == NO)
+		{
+		  isApplication = NO;
+		}
+	      else
+		{
+		  NSString	*i;
+
+		  i = [s stringByAppendingPathComponent: @"Info-gnustep.plist"];
+		  if ([mgr isReadableFileAtPath: i] == NO)
+		    {
+		      i = [s stringByAppendingPathComponent: @"Info.plist"];
+		      if ([mgr isReadableFileAtPath: i] == NO)
+			{
+			  isApplication = NO;
+			}
+		    }
+		}
+	    }
+	}
+#endif /* !__ANDROID__ */
+
+      if (isApplication == NO)
+	{
+	  NSString *maybePath = nil;
+
+	  if (isNonInstalledTool)
+	    {
+	      /* We're pretty confident about this case.  'path' is
+	       * obtained by {tool location on disk} and walking up
+	       * until we got out of the obj directory.  So we're
+	       * now in GNUSTEP_BUILD_DIR.  Resources will be in
+	       * Resources/{toolName}.
+	       */
+	      path = [path stringByAppendingPathComponent: @"Resources"];
+	      maybePath = [path stringByAppendingPathComponent: toolName];
+
+	      /* PS: We could check here if we found the resources,
+	       * and if not, keep going with the other attempts at
+	       * locating them.  But if we know that this is an
+	       * uninstalled tool, really we don't want to use
+	       * installed resources - we prefer resource lookup to
+	       * fail so the developer will fix whatever issue they
+	       * have with their building.
+	       */
+	    }
+	  else
+	    {
+	      if (maybePath == nil)
+		{
+		  /* This is for gnustep-make version 2, where tool resources
+		   * are in GNUSTEP_*_LIBRARY/Tools/Resources/{toolName}.
+		   */
+		  maybePath = _find_main_bundle_for_tool(toolName);
+		}
+
+	      /* If that didn't work, maybe the tool was created with
+	       * gnustep-make version 1.  So we try {tool location on
+	       * disk after walking up the non-flattened
+	       * dirs}/Resources/{toolName}, which is where
+	       * gnustep-make version 1 would put resources.
+	       */
+	      if (maybePath == nil)
+		{
+		  path = [path stringByAppendingPathComponent: @"Resources"];
+		  maybePath = [path stringByAppendingPathComponent: toolName];
+		}
+	    }
+
+	  path = maybePath;
+	}
+
+      ASSIGN(_mainBundlePath, path);
+    }
+  return _mainBundlePath;
+}
+
 static NSArray *
 bundle_directory_readable(NSString *path)
 {
@@ -509,7 +770,7 @@ addBundlePath(NSMutableArray *list, NSArray *contents,
 	    }	  
 	}
     }
-  if (nil == lang)
+  if ([lang length] == 0)
     {
       [list addObject: path];
     }
@@ -566,60 +827,133 @@ _find_framework(NSString *name)
   return nil;
 }
 
-
-/* Try to locate resources for tool name (which is this tool) in
- * standard places like xxx/Library/Tools/Resources/name */
-/* This could be converted into a public +bundleForTool:
- * method.  At the moment it's only used privately
- * to locate the main bundle for this tool.
+/* This method is the backbone of the resource searching for NSBundle. It
+ * constructs an array of paths, where each path is a possible location
+ * for a resource in the bundle.  The current algorithm for searching goes:
+ *
+ *   <rootPath>/Resources/<subPath>
+ *   <rootPath>/Resources/<subPath>/<language.lproj>
+ *   <rootPath>/<subPath>
+ *   <rootPath>/<subPath>/<language.lproj>
+ *
+ * NB. If localization is nil we ask NSUserDefaults for the preferred
+ * languages list (NSLanguages) and add language specific subdirectories
+ * for each language.  It is more efficient to search for a specific
+ * language by providing a non-nil value, or to provide an empty
+ * string if no language specific lookup is needed.
  */
-static inline NSString *
-_find_main_bundle_for_tool(NSString *toolName)
+static NSArray *
+_find_paths(NSString *rootPath, NSString *subPath, NSString *localization)
 {
-  NSArray *paths;
-  NSEnumerator *enumerator;
-  NSString *path;
-  NSString *tail;
-  NSFileManager *fm = manager();
+  NSString		*primary;
+  NSString		*language;
+  NSArray		*languages;
+  NSArray		*contents;
+  NSMutableArray	*array;
+  NSEnumerator		*enumerate;
 
-  /*
-   * Eliminate any base path or extensions.
+  array = [NSMutableArray arrayWithCapacity: 8];
+  languages = localization ? nil : [[NSUserDefaults standardUserDefaults]
+    stringArrayForKey: @"NSLanguages"];
+
+  primary = [rootPath stringByAppendingPathComponent: @"Resources"];
+  contents = bundle_directory_readable(primary);
+  addBundlePath(array, contents, primary, subPath, nil);
+  /* If we have been asked for a specific localization, we add it.
    */
-  toolName = [toolName lastPathComponent];
-  do
+  if (localization != nil)
     {
-      toolName = [toolName stringByDeletingPathExtension];
+      addBundlePath(array, contents, primary, subPath, localization);
     }
-  while ([[toolName pathExtension] length] > 0);
-
-  if ([toolName length] == 0)
+  else
     {
-      return nil;
-    }
-
-  tail = [@"Tools" stringByAppendingPathComponent:
-	     [@"Resources" stringByAppendingPathComponent:
-		 toolName]];
-
-  paths = NSSearchPathForDirectoriesInDomains (NSLibraryDirectory,
-					       NSAllDomainsMask, YES);
-
-  enumerator = [paths objectEnumerator];
-  while ((path = [enumerator nextObject]))
-    {
-      BOOL isDir;
-      path = [path stringByAppendingPathComponent: tail];
-
-      if ([fm fileExistsAtPath: path  isDirectory: &isDir]  &&  isDir)
+      /* This matches OS X behavior, which only searches languages that
+       * are in the user's preference. Don't use -preferredLocalizations -
+       * that would cause a recursive loop.
+       */
+      enumerate = [languages objectEnumerator];
+      while ((language = [enumerate nextObject]))
 	{
-	  return path;
+	  addBundlePath(array, contents, primary, subPath, language);
 	}
     }
+  
+#ifdef __ANDROID__
+  /* Android: check subdir and localization directly, as AAssetDir and thereby
+   * NSDirectoryEnumerator doesn't list directories
+   */
+  NSString *originalPrimary = primary;
+  if (subPath)
+    {
+      primary = [originalPrimary stringByAppendingPathComponent: subPath];
+      contents = bundle_directory_readable(primary);
+      addBundlePath(array, contents, primary, nil, nil);
+      
+      if (localization)
+	{
+	  if ([localization length])
+	    {
+	      primary = [primary stringByAppendingPathComponent:
+		[localization stringByAppendingPathExtension: @"lproj"]];
+	    }
+	  contents = bundle_directory_readable(primary);
+	  addBundlePath(array, contents, primary, nil, nil);
+	}
+      else
+	{
+	  NSString *subPathPrimary = primary;
 
-  return nil;
+	  enumerate = [languages objectEnumerator];
+	  while ((language = [enumerate nextObject]))
+	    {
+	      primary = [subPathPrimary stringByAppendingPathComponent:
+		[language stringByAppendingPathExtension: @"lproj"]];
+	      contents = bundle_directory_readable(primary);
+	      addBundlePath(array, contents, primary, nil, nil);
+	    }
+	}
+    }
+  if (localization)
+    {
+      if ([localization length])
+	{
+	  primary = [originalPrimary stringByAppendingPathComponent:
+	    [localization stringByAppendingPathExtension: @"lproj"]];
+	}
+      contents = bundle_directory_readable(primary);
+      addBundlePath(array, contents, primary, nil, nil);
+    }
+  else
+    {
+      enumerate = [languages objectEnumerator];
+      while ((language = [enumerate nextObject]))
+	{
+	  primary = [originalPrimary stringByAppendingPathComponent:
+	    [language stringByAppendingPathExtension: @"lproj"]];
+	  contents = bundle_directory_readable(primary);
+	  addBundlePath(array, contents, primary, nil, nil);
+	}
+    }
+#endif /* __ANDROID__ */
+  
+  primary = rootPath;
+  contents = bundle_directory_readable(primary);
+  addBundlePath(array, contents, primary, subPath, nil);
+  if (localization != nil)
+    {
+      addBundlePath(array, contents, primary, subPath, localization);
+    }
+  else
+    {
+      enumerate = [languages objectEnumerator];
+      while ((language = [enumerate nextObject]))
+	{
+	  addBundlePath(array, contents, primary, subPath, language);
+	}
+    }
+  return array;
 }
 
-
 @implementation NSBundle (Private)
 
 + (NSString *) _absolutePathOfExecutable: (NSString *)path
@@ -1145,6 +1479,99 @@ _bundle_load_callback(Class theClass, struct objc_category *theCategory)
     [NSValue valueWithPointer: (void*)theClass]];
 }
 
+/* Look up a resource in a bundle, outpinally with a sub-path and with
+ * a specific language preference.
+ * If the path of the bundle is nil, search the main bundle.
+ */
+NSString*
+GSPrivateResourcePath(NSString *name, NSString *extension, NSString *rootPath,
+  NSString *subPath, NSString *localization)
+{
+  NSFileManager	*mgr = manager();
+  NSString	*path;
+  NSString	*file;
+  NSEnumerator	*pathlist;
+
+  if (nil == rootPath)
+    {
+      rootPath = _find_main_bundle_path();
+    }
+  if (name == nil)
+    {
+      name = @"";
+    }
+  if ([extension length] == 0)
+    {
+      file = name;
+    }
+  else
+    {
+      file = [name stringByAppendingPathExtension: extension];
+    }
+
+  pathlist = [_find_paths(rootPath, subPath, localization) objectEnumerator];
+  while ((path = [pathlist nextObject]) != nil)
+    {
+      NSArray	*paths = bundle_directory_readable(path);
+
+      if (YES == [paths containsObject: file])
+	{
+	  path = [path stringByAppendingPathComponent: file];
+	  if (YES == [mgr isReadableFileAtPath: path])
+	    {
+	      return path;
+	    }
+	}
+    }
+
+#ifdef __ANDROID__
+  /* Android: check for directory resources by passing file path as subpath,
+   * as AAssetDir and thereby NSDirectoryEnumerator doesn't list directories
+   */
+  subPath = subPath ? [subPath stringByAppendingPathComponent: file] : file;
+  pathlist = [_find_paths(rootPath, subPath, localization) objectEnumerator];
+  while ((path = [pathlist nextObject]) != nil)
+    {
+      NSString *lastPathComponent = [path lastPathComponent];
+
+      if ([lastPathComponent isEqualToString:file]
+        && [mgr isReadableFileAtPath: path])
+        {
+          return path;
+        }
+    }
+#endif /* __ANDROID__ */
+
+  return nil;
+}
+
+NSDictionary*
+GSPrivateInfoDictionary(NSString *rootPath)
+{
+  NSString	*path;
+  NSDictionary	*info;
+
+  path = GSPrivateResourcePath(@"Info-gnustep", @"plist", rootPath,
+    nil, NOT_LOCALIZED);
+  if (path)
+    {
+      info = [NSDictionary dictionaryWithContentsOfFile: path];
+    }
+  else
+    {
+      path = GSPrivateResourcePath(@"Info", @"plist", rootPath,
+	nil, NOT_LOCALIZED);
+      if (path)
+	{
+	  info = [NSDictionary dictionaryWithContentsOfFile: path];
+	}
+      else
+	{
+	  info = [NSDictionary dictionary];
+	}
+    }
+  return info;
+}
 
 @implementation NSBundle
 
@@ -1152,6 +1579,7 @@ _bundle_load_callback(Class theClass, struct objc_category *theCategory)
 {
   if ([NSObject shouldCleanUp])
     {
+      DESTROY(_mainBundlePath);
       DESTROY(_emptyTable);
       DESTROY(langAliases);
       DESTROY(langCanonical);
@@ -1292,8 +1720,6 @@ _bundle_load_callback(Class theClass, struct objc_category *theCategory)
       if ((str = [env objectForKey: @"LIBRARY_COMBO"]) != nil)
 	library_combo = RETAIN(str);
 
-      _launchDirectory = RETAIN([manager() currentDirectoryPath]);
-
       _gnustep_bundle = RETAIN([self bundleForLibrary: @"gnustep-base"
 					      version: _base_version]);
 
@@ -1436,168 +1862,33 @@ _bundle_load_callback(Class theClass, struct objc_category *theCategory)
  */
 + (NSBundle *) mainBundle
 {
-  [load_lock lock];
   if (!_mainBundle)
     {
-      /* We figure out the main bundle directory by examining the location
-	 of the executable on disk.  */
-      NSString *path, *s;
+      BOOL	firstTime = NO;
 
-      /* We don't know at the beginning if it's a tool or an application.  */
-      BOOL isApplication = YES;
-
-      /* Sometimes we detect that this is a non-installed tool.  That is
-       * special because we want to lookup local resources before installed
-       * ones.  Keep track of this special case in this variable.
-       */
-      BOOL isNonInstalledTool = NO;
-
-      /* If it's a tool, we will need the tool name.  Since we don't
-         know yet if it's a tool or an application, we always store
-         the executable name here - just in case it turns out it's a
-         tool.  */
-      NSString *toolName = [GSPrivateExecutablePath() lastPathComponent];
-#if defined(_WIN32) || defined(__CYGWIN__)
-      toolName = [toolName stringByDeletingPathExtension];
-#endif
-
-      /* Strip off the name of the program */
-      path = [GSPrivateExecutablePath() stringByDeletingLastPathComponent];
-
-      /* We now need to chop off the extra subdirectories, the library
-	 combo and the target directory if they exist.  The executable
-	 and this library should match so that is why we can use the
-	 compiled-in settings. */
-      /* library combo */
-      s = [path lastPathComponent];
-      if ([s isEqual: library_combo])
+      [load_lock lock];
+      if (!_mainBundle)
 	{
-	  path = [path stringByDeletingLastPathComponent];
+	  NSString	*path = _find_main_bundle_path();
+
+	  /* We do alloc and init separately so initWithPath: knows we are
+	      the _mainBundle.  Please note that we do *not* autorelease
+	      mainBundle, because we don't want it to be ever released.  */
+	  _mainBundle = [self alloc];
+	  /* Please note that _mainBundle should *not* be nil.  */
+	  _mainBundle = [_mainBundle initWithPath: path];
+	  firstTime = YES;
 	}
-      /* target dir */
-      s = [path lastPathComponent];
-      if ([s isEqual: gnustep_target_dir])
-	{
-	  path = [path stringByDeletingLastPathComponent];
-	}
-      /* object dir */
-      s = [path lastPathComponent];
-      if ([s hasSuffix: @"obj"])
-	{
-	  path = [path stringByDeletingLastPathComponent];
-	  /* if it has an object dir it can only be a
-             non-yet-installed tool.  */
-	  isApplication = NO;
-	  isNonInstalledTool = YES;
-	}
-
-#ifndef __ANDROID__ /* don't check suffix on Android's fake executable path */
-      if (isApplication == YES)
-	{
-	  s = [path lastPathComponent];
-
-	  if ([s hasSuffix: @".app"] == NO
-	    && [s hasSuffix: @".debug"] == NO
-	    && [s hasSuffix: @".profile"] == NO
-	    && [s hasSuffix: @".gswa"] == NO	// GNUstep Web
-	    && [s hasSuffix: @".woa"] == NO	// GNUstep Web
-	    )
-	    {
-	      NSFileManager	*mgr = manager();
-	      BOOL		f;
-
-	      /* Not one of the common extensions, but
-	       * might be an app wrapper with another extension...
-	       * Look for Info-gnustep.plist or Info.plist in a
-	       * Resources subdirectory.
-	       */
-	      s = [path stringByAppendingPathComponent: @"Resources"];
-	      if ([mgr fileExistsAtPath: s isDirectory: &f] == NO || f == NO)
-		{
-		  isApplication = NO;
-		}
-	      else
-		{
-		  NSString	*i;
-
-		  i = [s stringByAppendingPathComponent: @"Info-gnustep.plist"];
-		  if ([mgr isReadableFileAtPath: i] == NO)
-		    {
-		      i = [s stringByAppendingPathComponent: @"Info.plist"];
-		      if ([mgr isReadableFileAtPath: i] == NO)
-			{
-			  isApplication = NO;
-			}
-		    }
-		}
-	    }
-	}
-#endif /* !__ANDROID__ */
-
-      if (isApplication == NO)
-	{
-	  NSString *maybePath = nil;
-
-	  if (isNonInstalledTool)
-	    {
-	      /* We're pretty confident about this case.  'path' is
-	       * obtained by {tool location on disk} and walking up
-	       * until we got out of the obj directory.  So we're
-	       * now in GNUSTEP_BUILD_DIR.  Resources will be in
-	       * Resources/{toolName}.
-	       */
-	      path = [path stringByAppendingPathComponent: @"Resources"];
-	      maybePath = [path stringByAppendingPathComponent: toolName];
-
-	      /* PS: We could check here if we found the resources,
-	       * and if not, keep going with the other attempts at
-	       * locating them.  But if we know that this is an
-	       * uninstalled tool, really we don't want to use
-	       * installed resources - we prefer resource lookup to
-	       * fail so the developer will fix whatever issue they
-	       * have with their building.
-	       */
-	    }
-	  else
-	    {
-	      if (maybePath == nil)
-		{
-		  /* This is for gnustep-make version 2, where tool resources
-		   * are in GNUSTEP_*_LIBRARY/Tools/Resources/{toolName}.
-		   */
-		  maybePath = _find_main_bundle_for_tool (toolName);
-		}
-
-	      /* If that didn't work, maybe the tool was created with
-	       * gnustep-make version 1.  So we try {tool location on
-	       * disk after walking up the non-flattened
-	       * dirs}/Resources/{toolName}, which is where
-	       * gnustep-make version 1 would put resources.
-	       */
-	      if (maybePath == nil)
-		{
-		  path = [path stringByAppendingPathComponent: @"Resources"];
-		  maybePath = [path stringByAppendingPathComponent: toolName];
-		}
-	    }
-
-	  path = maybePath;
-	}
-
-      NSDebugMLLog(@"NSBundle", @"Found main in %@\n", path);
-      /* We do alloc and init separately so initWithPath: knows we are
-          the _mainBundle.  Please note that we do *not* autorelease
-          mainBundle, because we don't want it to be ever released.  */
-      _mainBundle = [self alloc];
-      /* Please note that _mainBundle should *not* be nil.  */
-      _mainBundle = [_mainBundle initWithPath: path];
+      [load_lock unlock];
       NSAssert(_mainBundle != nil, NSInternalInconsistencyException);
+      if (firstTime)
+	{
+	  NSDebugMLLog(@"NSBundle", @"Main bundle path is %@\n",
+	    [_mainBundle bundlePath]);
+	}
     }
-
-  [load_lock unlock];
   return _mainBundle;
 }
-
 /**
  * Returns the bundle whose code contains the specified class.<br />
  * NB: We will not find a class if the bundle has not been loaded yet!
@@ -2199,202 +2490,20 @@ IF_NO_ARC(
   [load_lock unlock];
 }
 
-/* This method is the backbone of the resource searching for NSBundle. It
-   constructs an array of paths, where each path is a possible location
-   for a resource in the bundle.  The current algorithm for searching goes:
-
-     <rootPath>/Resources/<bundlePath>
-     <rootPath>/Resources/<bundlePath>/<language.lproj>
-     <rootPath>/<bundlePath>
-     <rootPath>/<bundlePath>/<language.lproj>
-*/
-+ (NSArray *) _bundleResourcePathsWithRootPath: (NSString*)rootPath
-				       subPath: (NSString*)subPath
-				  localization: (NSString*)localization
-{
-  NSString		*primary;
-  NSString		*language;
-  NSArray		*languages;
-  NSArray		*contents;
-  NSMutableArray	*array;
-  NSEnumerator		*enumerate;
-
-  array = [NSMutableArray arrayWithCapacity: 8];
-  languages = [[NSUserDefaults standardUserDefaults]
-    stringArrayForKey: @"NSLanguages"];
-
-  primary = [rootPath stringByAppendingPathComponent: @"Resources"];
-  contents = bundle_directory_readable(primary);
-  addBundlePath(array, contents, primary, subPath, nil);
-  /* If we have been asked for a specific localization, we add it.
-   */
-  if (localization != nil)
-    {
-      addBundlePath(array, contents, primary, subPath, localization);
-    }
-  else
-    {
-      /* This matches OS X behavior, which only searches languages that
-       * are in the user's preference. Don't use -preferredLocalizations -
-       * that would cause a recursive loop.
-       */
-      enumerate = [languages objectEnumerator];
-      while ((language = [enumerate nextObject]))
-	{
-	  addBundlePath(array, contents, primary, subPath, language);
-	}
-    }
-  
-#ifdef __ANDROID__
-  /* Android: check subdir and localization directly, as AAssetDir and thereby
-   * NSDirectoryEnumerator doesn't list directories
-   */
-  NSString *originalPrimary = primary;
-  if (subPath)
-    {
-      primary = [originalPrimary stringByAppendingPathComponent: subPath];
-      contents = bundle_directory_readable(primary);
-      addBundlePath(array, contents, primary, nil, nil);
-      
-      if (localization)
-	{
-	  primary = [primary stringByAppendingPathComponent:
-	    [localization stringByAppendingPathExtension: @"lproj"]];
-	  contents = bundle_directory_readable(primary);
-	  addBundlePath(array, contents, primary, nil, nil);
-	}
-      else
-	{
-	  NSString *subPathPrimary = primary;
-
-	  enumerate = [languages objectEnumerator];
-	  while ((language = [enumerate nextObject]))
-	    {
-	      primary = [subPathPrimary stringByAppendingPathComponent:
-		[language stringByAppendingPathExtension: @"lproj"]];
-	      contents = bundle_directory_readable(primary);
-	      addBundlePath(array, contents, primary, nil, nil);
-	    }
-	}
-    }
-  if (localization)
-    {
-      primary = [originalPrimary stringByAppendingPathComponent:
-	[localization stringByAppendingPathExtension: @"lproj"]];
-      contents = bundle_directory_readable(primary);
-      addBundlePath(array, contents, primary, nil, nil);
-    }
-  else
-    {
-      enumerate = [languages objectEnumerator];
-      while ((language = [enumerate nextObject]))
-	{
-	  primary = [originalPrimary stringByAppendingPathComponent:
-	    [language stringByAppendingPathExtension: @"lproj"]];
-	  contents = bundle_directory_readable(primary);
-	  addBundlePath(array, contents, primary, nil, nil);
-	}
-    }
-#endif /* __ANDROID__ */
-  
-  primary = rootPath;
-  contents = bundle_directory_readable(primary);
-  addBundlePath(array, contents, primary, subPath, nil);
-  if (localization != nil)
-    {
-      addBundlePath(array, contents, primary, subPath, localization);
-    }
-  else
-    {
-      enumerate = [languages objectEnumerator];
-      while ((language = [enumerate nextObject]))
-	{
-	  addBundlePath(array, contents, primary, subPath, language);
-	}
-    }
-  return array;
-}
-
-+ (NSString *) _pathForResource: (NSString *)name
-			 ofType: (NSString *)extension
-		     inRootPath: (NSString *)rootPath
-		    inDirectory: (NSString *)subPath
-{
-  NSFileManager	*mgr = manager();
-  NSString	*path;
-  NSString	*file;
-  NSEnumerator	*pathlist;
-
-  if (name == nil)
-    {
-      name = @"";
-    }
-  if ([extension length] == 0)
-    {
-      file = name;
-    }
-  else
-    {
-      file = [name stringByAppendingPathExtension: extension];
-    }
-
-  pathlist = [[self _bundleResourcePathsWithRootPath: rootPath
-    subPath: subPath localization: nil] objectEnumerator];
-  while ((path = [pathlist nextObject]) != nil)
-    {
-      NSArray	*paths = bundle_directory_readable(path);
-
-      if (YES == [paths containsObject: file])
-	{
-	  path = [path stringByAppendingPathComponent: file];
-	  if (YES == [mgr isReadableFileAtPath: path])
-	    {
-	      return path;
-	    }
-	}
-    }
-
-#ifdef __ANDROID__
-  /* Android: check for directory resources by passing file path as subpath,
-   * as AAssetDir and thereby NSDirectoryEnumerator doesn't list directories
-   */
-  subPath = subPath ? [subPath stringByAppendingPathComponent: file] : file;
-  pathlist = [[self _bundleResourcePathsWithRootPath: rootPath
-    subPath: subPath localization: nil] objectEnumerator];
-  while ((path = [pathlist nextObject]) != nil)
-    {
-      NSString *lastPathComponent = [path lastPathComponent];
-      if ([lastPathComponent isEqualToString:file]
-        && [mgr isReadableFileAtPath: path])
-        {
-          return path;
-        }
-    }
-#endif /* __ANDROID__ */
-
-  return nil;
-}
-
 
 + (NSString *) pathForResource: (NSString *)name
 			ofType: (NSString *)extension
 		   inDirectory: (NSString *)bundlePath
 		   withVersion: (int)version
 {
-  return [self _pathForResource: name
-			 ofType: extension
-		     inRootPath: bundlePath
-		    inDirectory: nil];
+  return GSPrivateResourcePath(name, extension, bundlePath, nil, nil);
 }
 
 + (NSString *) pathForResource: (NSString *)name
 			ofType: (NSString *)extension
 		   inDirectory: (NSString *)bundlePath
 {
-  return [self _pathForResource: name
-			 ofType: extension
-		     inRootPath: bundlePath
-		    inDirectory: nil];
+  return GSPrivateResourcePath(name, extension, bundlePath, nil, nil);
 }
 
 + (NSURL*) URLForResource: (NSString*)name
@@ -2431,10 +2540,7 @@ IF_NO_ARC(
 #endif
     rootPath = [self bundlePath];
 
-  return [NSBundle _pathForResource: name
-			     ofType: extension
-			 inRootPath: rootPath
-		        inDirectory: subPath];
+  return GSPrivateResourcePath(name, extension, rootPath, subPath, nil);
 }
 
 - (NSURL *) URLForResource: (NSString *)name
@@ -2484,8 +2590,7 @@ IF_NO_ARC(
   NSMutableArray *resources;
   NSEnumerator *pathlist;
 
-  pathlist = [[NSBundle _bundleResourcePathsWithRootPath: bundlePath
-    subPath: subPath localization: localization] objectEnumerator];
+  pathlist = [_find_paths(bundlePath, subPath, localization) objectEnumerator];
   resources = [NSMutableArray arrayWithCapacity: 2];
   allfiles = (extension == nil || [extension length] == 0);
 
@@ -3003,27 +3108,9 @@ IF_NO_ARC(
 
 - (NSDictionary *) infoDictionary
 {
-  NSString* path;
-
-  if (_infoDict)
-    return _infoDict;
-
-  path = [self pathForResource: @"Info-gnustep" ofType: @"plist"];
-  if (path)
+  if (nil == _infoDict)
     {
-      _infoDict = [[NSDictionary alloc] initWithContentsOfFile: path];
-    }
-  else
-    {
-      path = [self pathForResource: @"Info" ofType: @"plist"];
-      if (path)
-	{
-	  _infoDict = [[NSDictionary alloc] initWithContentsOfFile: path];
-	}
-      else
-	{
-	  _infoDict = RETAIN([NSDictionary dictionary]);
-	}
+      _infoDict = [GSPrivateInfoDictionary(_path) copy];
     }
   return _infoDict;
 }
@@ -3333,7 +3420,7 @@ IF_NO_ARC(
 
 #ifdef __ANDROID__
 
-+ (AAssetManager *)assetManager
++ (AAssetManager *) assetManager
 {
   return _assetManager;
 }
@@ -3365,7 +3452,7 @@ IF_NO_ARC(
     {
       NSString *resourcePath = [_mainBundle resourcePath];
 
-      if ([path hasPrefix: resourcePath]
+      if (resourcePath && [path hasPrefix: resourcePath]
 	&& [path length] > [resourcePath length])
 	{
 	  NSString *assetPath;
