@@ -18,8 +18,7 @@
    
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 
    */ 
 
@@ -29,26 +28,18 @@
 #  include <objc/capabilities.h>
 #endif
 
+#import	"GNUstepBase/GSObjCRuntime.h"
+
+#define WEAK_READ(x) objc_loadWeak((id*)x)
+#define WEAK_WRITE(addr, x) objc_storeWeak((id*)addr, (id)x)
+
 #if defined(OBJC_CAP_ARC)
-#    include <objc/objc-arc.h>
-#    define ARC_WEAK_READ(x) objc_loadWeak((id*)x)
-#    define ARC_WEAK_WRITE(addr, x) objc_storeWeak((id*)addr, (id)x)
-#    define WEAK_READ(x) (*x)
-#    define WEAK_WRITE(addr, x) (*(addr) =  x)
-#    define STRONG_WRITE(addr, x) objc_storeStrong((id*)addr, (id)x)
-#    define STRONG_ACQUIRE(x) objc_retain(x)
+#  define STRONG_WRITE(addr, x) objc_storeStrong((id*)addr, (id)x)
+#  define STRONG_ACQUIRE(x) objc_retain(x)
 #else
-#  define WEAK_READ(x) (*x)
-#  define WEAK_WRITE(addr, x) (*(addr) =  x)
 #  define STRONG_WRITE(addr, x) ASSIGN(*((id*)addr), ((id)x))
 #  define STRONG_ACQUIRE(x) RETAIN(((id)x))
 #endif
-#ifndef ARC_WEAK_WRITE
-#  define ARC_WEAK_WRITE(addr, x) WEAK_WRITE(addr, x)
-#endif
-#ifndef ARC_WEAK_READ
-#  define ARC_WEAK_READ(x) WEAK_READ(x)
-#endif
 
 
 /* Declare a structure type to copy pointer functions information 
@@ -106,10 +97,6 @@ inline static BOOL personalityType(int options, int flag)
 static inline void *pointerFunctionsRead(PFInfo *PF, void **addr)
 {
   if (memoryType(PF->options, NSPointerFunctionsWeakMemory))
-    {
-      return ARC_WEAK_READ((id*)addr);
-    }
-  if (memoryType(PF->options, NSPointerFunctionsZeroingWeakMemory))
     {
       return WEAK_READ((id*)addr);
     }
@@ -122,10 +109,6 @@ static inline void *pointerFunctionsRead(PFInfo *PF, void **addr)
 static inline void pointerFunctionsAssign(PFInfo *PF, void **addr, void *value)
 {
   if (memoryType(PF->options, NSPointerFunctionsWeakMemory))
-    {
-      ARC_WEAK_WRITE(addr, value);
-    }
-  else if (memoryType(PF->options, NSPointerFunctionsZeroingWeakMemory))
     {
       WEAK_WRITE(addr, value);
     }
@@ -139,23 +122,17 @@ static inline void pointerFunctionsAssign(PFInfo *PF, void **addr, void *value)
     }
 }
 
-/* Acquire the pointer value to store for the specified item.
- */
 static inline void *
-pointerFunctionsAcquire(PFInfo *PF, void **dst, void *src)
+pointerFunctionsAcquire(PFInfo *PF, void *src)
 {
   if (PF->acquireFunction != 0)
-    src = (*PF->acquireFunction)(src, PF->sizeFunction,
-    PF->options & NSPointerFunctionsCopyIn ? YES : NO);
-  // FIXME: This shouldn't be here.  Acquire and assign are separate
-  // operations.  Acquire is for copy-in operations (i.e. retain / copy),
-  // assign is for move operations of already-owned pointers.  Combining them
-  // like this is Just Plain Wrong™
-  pointerFunctionsAssign(PF, dst, src);
+    {
+      src = (*PF->acquireFunction)(src, PF->sizeFunction,
+	PF->options & NSPointerFunctionsCopyIn ? YES : NO);
+    }
   return src;
 }
 
-
 /**
  * Moves a pointer from location to another.
  */
@@ -205,13 +182,10 @@ pointerFunctionsEqual(PFInfo *PF, void *item1, void *item2)
 static inline void
 pointerFunctionsRelinquish(PFInfo *PF, void **itemptr)
 {
-  
   if (PF->relinquishFunction != 0)
     (*PF->relinquishFunction)(*itemptr, PF->sizeFunction);
   if (memoryType(PF->options, NSPointerFunctionsWeakMemory))
-    ARC_WEAK_WRITE(itemptr, 0);
-  else if (memoryType(PF->options, NSPointerFunctionsZeroingWeakMemory))
-    WEAK_WRITE(itemptr, (void*)0);
+    WEAK_WRITE(itemptr, 0);
   else
     *itemptr = 0;
 }
@@ -228,9 +202,7 @@ pointerFunctionsReplace(PFInfo *PF, void **dst, void *src)
       if (PF->relinquishFunction != 0)
 	(*PF->relinquishFunction)(*dst, PF->sizeFunction);
       if (memoryType(PF->options, NSPointerFunctionsWeakMemory))
-        ARC_WEAK_WRITE(dst, 0);
-      else if (memoryType(PF->options, NSPointerFunctionsZeroingWeakMemory))
-        WEAK_WRITE(dst, (void*)0);
+        WEAK_WRITE(dst, src);
       else
 	*dst = src;
     }
