@@ -19,8 +19,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 
    <title>NSDictionary class reference</title>
    $Date$ $Revision$
@@ -470,6 +469,7 @@ static SEL	appSel;
 
   if (objectCount != [keys count])
     {
+      RELEASE(self);
       [NSException raise: NSInvalidArgumentException
 		  format: @"init with obj and key arrays of different sizes"];
     }
@@ -879,9 +879,7 @@ static SEL	appSel;
  */
 - (NSArray*) allKeysForObject: (id)anObject
 {
-  unsigned	c;
-
-  if (anObject == nil || (c = [self count]) == 0)
+  if (anObject == nil || [self count] == 0)
     {
       return nil;
     }
@@ -891,12 +889,12 @@ static SEL	appSel;
       IMP		nxtObj = [e methodForSelector: nxtSel];
       IMP		myObj = [self methodForSelector: objSel];
       BOOL		(*eqObj)(id, SEL, id);
+      unsigned		c = 0;
       id		k;
       id		result;
       GS_BEGINIDBUF(a, [self count]);
 
       eqObj = (BOOL (*)(id, SEL, id))[anObject methodForSelector: eqSel];
-      c = 0;
       while ((k = (*nxtObj)(e, nxtSel)) != nil)
 	{
 	  id	o = (*myObj)(self, objSel, k);
