@@ -18,8 +18,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 
    $Date$ $Revision$
 */
@@ -379,17 +378,17 @@ GSPrivateBaseAddress(void *addr, void **base)
       DESTROY(self);
       return nil;
     }
-  _symbols = malloc (neededSpace);
+  _symbols = (asymbol**)malloc(neededSpace);
   if (!_symbols)
     {
-      //NSLog (@"GSBinaryFileInfo: Can't allocate buffer");
+      //NSLog(@"GSBinaryFileInfo: Can't allocate buffer");
       DESTROY(self);
       return nil;
     }
-  _symbolCount = bfd_canonicalize_symtab (_abfd, _symbols);
+  _symbolCount = bfd_canonicalize_symtab(_abfd, _symbols);
   if (_symbolCount < 0)
     {
-      //NSLog (@"GSBinaryFileInfo: BFD error while reading symbols");
+      //NSLog(@"GSBinaryFileInfo: BFD error while reading symbols");
       DESTROY(self);
       return nil;
     }
@@ -454,7 +453,7 @@ static void find_address(bfd *abfd, asection *section,
     }
 
 
-  if (bfd_find_nearest_line (abfd, section, info->symbols,
+  if (bfd_find_nearest_line(abfd, section, info->symbols,
     address - vma, &fileName, &functionName, &line))
     {
       GSFunctionInfo	*fi;
@@ -594,16 +593,21 @@ struct GSBacktraceState
 static _Unwind_Reason_Code
 GSUnwindCallback(struct _Unwind_Context* context, void* arg)
 {
-    struct GSBacktraceState *state = (struct GSBacktraceState*)arg;
-    uintptr_t pc = _Unwind_GetIP(context);
-    if (pc) {
-      if (state->current == state->end) {
-        return _URC_END_OF_STACK;
-      } else {
-        *state->current++ = (void*)pc;
-      }
+  struct GSBacktraceState	*state = (struct GSBacktraceState*)arg;
+  uintptr_t 			pc = _Unwind_GetIP(context);
+
+  if (pc)
+    {
+      if (state->current == state->end)
+	{
+	  return _URC_END_OF_STACK;
+	}
+      else
+	{
+	  *state->current++ = (void*)pc;
+	}
     }
-    return 0; //_URC_OK/_URC_NO_REASON
+  return 0; //_URC_OK/_URC_NO_REASON
 }
 
 #endif	/* WITH_UNWIND && !HAVE_BACKTRACE */
@@ -988,20 +992,20 @@ GSPrivateReturnAddresses(NSUInteger **returns)
   numReturns = (capture)(0, MAXFRAMES, (void**)addr, NULL);
   if (numReturns > 0)
     {
-      *returns = malloc(numReturns * sizeof(void*));
-      memcpy(*returns, addr, numReturns * sizeof(void*));
+      *returns = (NSUInteger*)malloc(numReturns * sizeof(NSUInteger));
+      memcpy(*returns, addr, numReturns * sizeof(NSUInteger));
     }
   
   GS_MUTEX_UNLOCK(traceLock);
 
 #elif	defined(HAVE_BACKTRACE)
-  void          *addr[MAXFRAMES*sizeof(void*)];
+  void          *addr[MAXFRAMES];
 
   numReturns = backtrace(addr, MAXFRAMES);
   if (numReturns > 0)
     {
-      *returns = malloc(numReturns * sizeof(void*));
-      memcpy(*returns, addr, numReturns * sizeof(void*));
+      *returns = (NSUInteger*)malloc(numReturns * sizeof(NSUInteger));
+      memcpy(*returns, addr, numReturns * sizeof(NSUInteger));
     }
 #elif defined(WITH_UNWIND)
   void          *addr[MAXFRAMES];
@@ -1012,8 +1016,8 @@ GSPrivateReturnAddresses(NSUInteger **returns)
   numReturns = state.current - addr;
   if (numReturns > 0)
     {
-      *returns = malloc(numReturns * sizeof(void*));
-      memcpy(*returns, addr, numReturns * sizeof(void*));
+      *returns = (NSUInteger*)malloc(numReturns * sizeof(NSUInteger));
+      memcpy(*returns, addr, numReturns * sizeof(NSUInteger));
     }
 #else
   int   n;
@@ -1034,7 +1038,7 @@ GSPrivateReturnAddresses(NSUInteger **returns)
     {
       jbuf_type *env;
 
-      *returns = malloc(numReturns * sizeof(void*));
+      *returns = (NSUInteger*)malloc(numReturns * sizeof(NSUInteger));
 
       env = jbuf();
       if (sigsetjmp(env->buf, 1) == 0)
@@ -1088,7 +1092,7 @@ GSPrivateReturnAddresses(NSUInteger **returns)
                 {
                   break;
                 }
-              memcpy(&(*returns)[i], env->addr, sizeof(void*));
+              memcpy(&(*returns)[i], env->addr, sizeof(NSUInteger));
             }
           signal(SIGSEGV, env->segv);
           signal(SIGBUS, env->bus);
