@@ -18,23 +18,22 @@
    
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 */
 
-#include <Foundation/NSHFSFileTypes.h>
+#import "Foundation/NSHFSFileTypes.h"
 
-NSString *NSFileTypeForHFSTypeCode(NSUInteger hfsFileTypeCode)
+GS_DECLARE NSString *NSFileTypeForHFSTypeCode(NSUInteger hfsFileTypeCode)
 {
   return nil;
 }
 
-NSUInteger NSHFSTypeCodeFromFileType(NSString *fileTypeString)
+GS_DECLARE NSUInteger NSHFSTypeCodeFromFileType(NSString *fileTypeString)
 {
   return 0;
 }
 
-NSString *NSHFSTypeOfFile(NSString *fullFilePath)
+GS_DECLARE NSString *NSHFSTypeOfFile(NSString *fullFilePath)
 {
   return nil;
 }
