@@ -13,12 +13,11 @@
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02111 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 
    $Date: 2007-06-08 04: 04: 14 -0400 (Fri, 08 Jun 2007) $ $Revision: 25230 $
    */
@@ -31,14 +30,15 @@
 @interface NSKeyValueMutableArray : NSMutableArray
 {
   @protected
-  id object;
-  NSString *key;
+  id		object;
+  NSString 	*key;
   NSMutableArray *array;
   BOOL otherChangeInProgress;
   BOOL notifiesObservers;
 }
 
-+ (NSKeyValueMutableArray *) arrayForKey: (NSString *)aKey ofObject: (id)anObject;
++ (NSKeyValueMutableArray *) arrayForKey: (NSString*)aKey
+				ofObject: (id)anObject;
 - (id) initWithKey: (NSString *)aKey ofObject: (id)anObject;
 
 @end
@@ -52,10 +52,10 @@
 }
 
 + (id) arrayForKey: (NSString *)aKey ofObject: (id)anObject
-  withCapitalizedKey: (const char *)capitalized;
+withCapitalizedKey: (const char *)capitalized;
 
 - (id) initWithKey: (NSString *)aKey ofObject: (id)anObject
-  withCapitalizedKey: (const char *)capitalized;
+withCapitalizedKey: (const char *)capitalized;
 
 @end
 
@@ -66,10 +66,10 @@
 }
 
 + (id) arrayForKey: (NSString *)aKey ofObject: (id)anObject
-  withCapitalizedKey: (const char *)capitalized;
+withCapitalizedKey: (const char *)capitalized;
 
 - (id) initWithKey: (NSString *)aKey ofObject: (id)anObject
-  withCapitalizedKey: (const char *)capitalized;
+withCapitalizedKey: (const char *)capitalized;
 
 @end
 
@@ -85,6 +85,12 @@
 @end
 
 
+/* NB. For removal of objects we can remove multiple objects at the same
+ * time and the notifications are sent with an NSIndexSet specifying the
+ * indices which are being altered.  We therefore funnuel all the other
+ * removal methods through the -removeObjectsAtIndexes: method so that
+ * the subclasses only need to implement that one themselves.
+ */
 @implementation NSKeyValueMutableArray
 
 + (NSKeyValueMutableArray *) arrayForKey: (NSString *)aKey
@@ -94,6 +100,7 @@
   unsigned size = [aKey maximumLengthOfBytesUsingEncoding: 
 			  NSUTF8StringEncoding];
   char keybuf[size + 1];
+
   [aKey getCString: keybuf
          maxLength: size + 1
           encoding: NSUTF8StringEncoding];
@@ -127,12 +134,18 @@
       object = anObject;
       key = [aKey copy];
       otherChangeInProgress = NO;
-      notifiesObservers = 
-       [[anObject class] automaticallyNotifiesObserversForKey: aKey];
+      notifiesObservers
+	= [[anObject class] automaticallyNotifiesObserversForKey: aKey];
     }
   return self;
 }
 
+- (void) dealloc
+{
+  RELEASE(key);
+  DEALLOC
+}
+
 - (NSUInteger) count
 {
   if (array == nil)
@@ -156,9 +169,85 @@
   [self insertObject: anObject  atIndex: [self count]];
 }
 
+- (void) removeFirstObject
+{
+  NSUInteger count = [self count];
+
+  if (0 == count)
+    {
+      return;
+    }
+  [self removeObjectAtIndex: 0];
+}
+
 - (void) removeLastObject
 {
-  [self removeObjectAtIndex: ([self count] - 1)];
+  NSUInteger count = [self count];
+
+  if (0 == count)
+    {
+      return;
+    }
+  [self removeObjectAtIndex: (count - 1)];
+}
+
+- (void) removeObject: (id)anObject
+{
+  NSUInteger count = [self count];
+
+  if (count > 0)
+    {
+      NSMutableIndexSet	*indexes = nil;
+
+      while (count-- > 0)
+	{
+	  if ([[self objectAtIndex: count] isEqual: anObject])
+	    {
+	      if (nil == indexes)
+		{
+		  indexes = [NSMutableIndexSet indexSet];
+		}
+	      [indexes addIndex: count];
+	    }
+	}
+      if (indexes)
+	{
+	  [self removeObjectsAtIndexes: indexes];
+	}
+    }  
+}
+
+- (void) removeObjectAtIndex: (NSUInteger)index
+{
+  if (index != NSNotFound)
+    {
+      [self removeObjectsAtIndexes: [NSIndexSet indexSetWithIndex: index]];
+    }
+}
+
+- (void) removeObjectsFromIndices: (NSUInteger*)indices
+                       numIndices: (NSUInteger)count
+{
+  if (count > 0)
+    {
+      NSMutableIndexSet	*indexes = nil;
+
+      while (count-- > 0)
+	{
+	  if (indices[count] != NSNotFound)
+	    {
+	      if (nil == indexes)
+		{
+	          indexes = [NSMutableIndexSet indexSet];
+		}
+	      [indexes addIndex: indices[count]];
+	    }
+	}
+      if (indexes)
+	{
+          [self removeObjectsAtIndexes: indexes];
+	}
+    }
 }
 
 @end
@@ -166,14 +255,14 @@
 @implementation NSKeyValueFastMutableArray
 
 + (id) arrayForKey: (NSString *)aKey ofObject: (id)anObject
-       withCapitalizedKey: (const char *)capitalized
+withCapitalizedKey: (const char *)capitalized
 {
   return [[[self alloc] initWithKey: aKey ofObject: anObject
                  withCapitalizedKey: capitalized] autorelease];
 }
 
 - (id) initWithKey: (NSString *)aKey ofObject: (id)anObject
-       withCapitalizedKey: (const char *)capitalized
+withCapitalizedKey: (const char *)capitalized
 {
   SEL insert;
   SEL remove;
@@ -223,24 +312,33 @@
   [super dealloc];
 }
 
-- (void) removeObjectAtIndex: (NSUInteger)index
+- (void) removeObjectsAtIndexes: (NSIndexSet*)indexes
 {
-  NSIndexSet *indexes = nil;
+  NSUInteger	index = [indexes lastIndex];
+
+  if (nil == indexes || NSNotFound == index)
+    {
+      return;
+    }
 
   if (notifiesObservers && !otherChangeInProgress)
     {
-      indexes = [NSIndexSet indexSetWithIndex: index];
       [object willChange: NSKeyValueChangeRemoval
-        valuesAtIndexes: indexes
+         valuesAtIndexes: indexes
                   forKey: key];
     }
   [removeObjectInvocation setArgument: &index atIndex: 2];
   [removeObjectInvocation invoke];
+  while ((index = [indexes indexLessThanIndex: index]) != NSNotFound)
+    {
+      [removeObjectInvocation setArgument: &index atIndex: 2];
+      [removeObjectInvocation invoke];
+    }
   if (notifiesObservers && !otherChangeInProgress)
     {
       [object didChange: NSKeyValueChangeRemoval
         valuesAtIndexes: indexes
-                  forKey: key];
+                 forKey: key];
     }
 }
 
@@ -252,8 +350,8 @@
     {
       indexes = [NSIndexSet indexSetWithIndex: index];
       [object willChange: NSKeyValueChangeInsertion
-      valuesAtIndexes: indexes
-                forKey: key];
+	 valuesAtIndexes: indexes
+		  forKey: key];
     }
   [insertObjectInvocation setArgument: &anObject atIndex: 2];
   [insertObjectInvocation setArgument: &index atIndex: 3];
@@ -261,7 +359,7 @@
   if (notifiesObservers && !otherChangeInProgress)
     {
       [object didChange: NSKeyValueChangeInsertion
-       valuesAtIndexes: indexes
+        valuesAtIndexes: indexes
                  forKey: key];
     }
 }
@@ -275,8 +373,8 @@
       otherChangeInProgress = YES;
       indexes = [NSIndexSet indexSetWithIndex: index];
       [object willChange: NSKeyValueChangeReplacement
-        valuesAtIndexes: indexes
-                 forKey: key];
+         valuesAtIndexes: indexes
+                  forKey: key];
     }
   if (replaceObjectInvocation)
     {
@@ -292,27 +390,26 @@
   if (notifiesObservers && !otherChangeInProgress)
     {
       [object didChange: NSKeyValueChangeReplacement
-       valuesAtIndexes: indexes
+	valuesAtIndexes: indexes
                  forKey: key];
       otherChangeInProgress = NO;
     }
 }
 
-
 @end
 
 @implementation NSKeyValueSlowMutableArray
 
 + (id) arrayForKey: (NSString *)aKey ofObject: (id)anObject
-       withCapitalizedKey: (const char *)capitalized
+withCapitalizedKey: (const char *)capitalized
 {
   return [[[self alloc] initWithKey: aKey ofObject: anObject
-                withCapitalizedKey: capitalized] autorelease];
+                 withCapitalizedKey: capitalized] autorelease];
 }
 
-- (id) initWithKey: (NSString *)aKey ofObject: (id)anObject
-       withCapitalizedKey: (const char *)capitalized;
-
+- (id) initWithKey: (NSString *)aKey
+	  ofObject: (id)anObject
+withCapitalizedKey: (const char *)capitalized
 {
   SEL set = NSSelectorFromString([NSString stringWithFormat: 
     @"set%s:", capitalized]);
@@ -333,21 +430,35 @@
   return self;
 }
 
-- (void) removeObjectAtIndex: (NSUInteger)index
+- (void) dealloc
 {
-  NSIndexSet *indexes = nil;
-  NSMutableArray *temp;
+  RELEASE(setArrayInvocation);
+  DEALLOC
+}
+
+- (void) removeObjectsAtIndexes: (NSIndexSet*)indexes
+{
+  NSUInteger		index = [indexes lastIndex];
+  NSMutableArray 	*temp;
+
+  if (nil == indexes || NSNotFound == index)
+    {
+      return;
+    }
 
   if (notifiesObservers && !otherChangeInProgress)
     {
-      indexes = [NSIndexSet indexSetWithIndex: index];
       [object willChange: NSKeyValueChangeRemoval
-        valuesAtIndexes: indexes
+         valuesAtIndexes: indexes
                   forKey: key];
     }
   
   temp = [NSMutableArray arrayWithArray: [object valueForKey: key]];
   [temp removeObjectAtIndex: index];
+  while ((index = [indexes indexLessThanIndex: index]) != NSNotFound)
+    {
+      [temp removeObjectAtIndex: index];
+    }
 
   [setArrayInvocation setArgument: &temp atIndex: 2];
   [setArrayInvocation invoke];
@@ -355,21 +466,21 @@
   if (notifiesObservers && !otherChangeInProgress)
     {
       [object didChange: NSKeyValueChangeRemoval
-       valuesAtIndexes: indexes
+        valuesAtIndexes: indexes
                  forKey: key];
     }
 }
 
 - (void) insertObject: (id)anObject atIndex: (NSUInteger)index
 {
-  NSIndexSet *indexes = nil;
+  NSIndexSet	*indexes = nil;
   NSMutableArray *temp;
 
   if (notifiesObservers && !otherChangeInProgress)
     {
       indexes = [NSIndexSet indexSetWithIndex: index];
       [object willChange: NSKeyValueChangeInsertion
-        valuesAtIndexes: indexes
+         valuesAtIndexes: indexes
                   forKey: key];
     }
 
@@ -382,7 +493,7 @@
   if (notifiesObservers && !otherChangeInProgress)
     {
       [object didChange: NSKeyValueChangeInsertion
-       valuesAtIndexes: indexes
+        valuesAtIndexes: indexes
                  forKey: key];
     }
 }
@@ -396,7 +507,7 @@
     {
       indexes = [NSIndexSet indexSetWithIndex: index];
       [object willChange: NSKeyValueChangeReplacement
-        valuesAtIndexes: indexes
+         valuesAtIndexes: indexes
                   forKey: key];
     }
   
@@ -410,12 +521,11 @@
   if (notifiesObservers && !otherChangeInProgress)
     {
       [object didChange: NSKeyValueChangeReplacement
-       valuesAtIndexes: indexes
+        valuesAtIndexes: indexes
                  forKey: key];
     }
 }
 
-
 @end
 
 
@@ -443,11 +553,11 @@
              maxLength: size + 1
               encoding: NSUTF8StringEncoding];
       
-      if (!GSObjCFindVariable (anObject, cKeyPtr, &type, &size, &offset))
-        found = GSObjCFindVariable (anObject, ++cKeyPtr, &type, &size, &offset);
+      if (!GSObjCFindVariable(anObject, cKeyPtr, &type, &size, &offset))
+        found = GSObjCFindVariable(anObject, ++cKeyPtr, &type, &size, &offset);
       if (found)
         {
-          array = GSObjCGetVal (anObject, cKeyPtr, NULL, type, size, offset);
+          array = GSObjCGetVal(anObject, cKeyPtr, NULL, type, size, offset);
         }
       else
         {
@@ -478,9 +588,14 @@
     }
 }
 
-- (void) removeObjectAtIndex: (NSUInteger)index
+- (void) removeObjectsAtIndexes: (NSIndexSet*)indexes
 {
-  NSIndexSet *indexes = nil;
+  NSUInteger	index = [indexes lastIndex];
+
+  if (nil == indexes || NSNotFound == index)
+    {
+      return;
+    }
 
   if (notifiesObservers)
     {
@@ -490,6 +605,10 @@
                   forKey: key];
     }
   [array removeObjectAtIndex: index];
+  while ((index = [indexes indexLessThanIndex: index]) != NSNotFound)
+    {
+      [array removeObjectAtIndex: index];
+    }
   if (notifiesObservers)
     {
       [object didChange: NSKeyValueChangeRemoval
@@ -521,10 +640,16 @@
 - (void) removeLastObject
 {
   NSIndexSet *indexes =  nil;
+  NSUInteger count = [array count];
+
+  if (0 == count)
+    {
+      return;
+    }
 
   if (notifiesObservers)
     {
-      indexes = [NSIndexSet indexSetWithIndex: [array count] - 1];
+      indexes = [NSIndexSet indexSetWithIndex: count - 1];
       [object willChange: NSKeyValueChangeRemoval
          valuesAtIndexes: indexes
                   forKey: key];
