@@ -18,8 +18,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 
    $Date$ $Revision$
 */
@@ -41,8 +40,8 @@
 #import "Foundation/NSValue.h"
 #import "GNUstepBase/GSObjCRuntime.h"
 #import "GNUstepBase/Unicode.h"
-#import "GNUstepBase/GSLock.h"
 #import "GSInvocation.h"
+#import "GSPThread.h"
 
 #if defined(USE_LIBFFI)
 #import "cifframe.h"
@@ -78,9 +77,11 @@ static id               null;
 static inline void
 setup()
 {
+  static gs_mutex_t     setupLock = GS_MUTEX_INIT_STATIC;
+
   if (nil == kvoLock)
     {
-      [gnustep_global_lock lock];
+      GS_MUTEX_LOCK(setupLock);
       if (nil == kvoLock)
 	{
 	  kvoLock = [NSRecursiveLock new];
@@ -88,12 +89,12 @@ setup()
 	  classTable = NSCreateMapTable(NSNonOwnedPointerMapKeyCallBacks,
 	    NSNonOwnedPointerMapValueCallBacks, 128);
 	  infoTable = NSCreateMapTable(NSNonOwnedPointerMapKeyCallBacks,
-	    NSNonOwnedPointerMapValueCallBacks, 1024);
+	    NSObjectMapValueCallBacks, 1024);
 	  dependentKeyTable = NSCreateMapTable(NSNonOwnedPointerMapKeyCallBacks,
 	      NSOwnedPointerMapValueCallBacks, 128);
 	  baseClass = NSClassFromString(@"GSKVOBase");
 	}
-      [gnustep_global_lock unlock];
+      GS_MUTEX_UNLOCK(setupLock);
     }
 }
 /*
@@ -110,9 +111,9 @@ setup()
  */
 @interface	GSKVOReplacement : NSObject
 {
-  Class         original;       /* The original class */
-  Class         replacement;    /* The replacement class */
-  NSMutableSet  *keys;          /* The observed setter keys */
+  Class         	original;       /* The original class */
+  Class         	replacement;    /* The replacement class */
+  NSMutableDictionary  	*keys;          /* The observed setter keys */
 }
 - (id) initWithClass: (Class)aClass;
 - (void) overrideSetterFor: (NSString*)aKey;
@@ -474,16 +475,16 @@ cifframe_callback(ffi_cif *cif, void *retp, void **args, void *user)
   replacement = NSClassFromString(name);
   GSObjCAddClassBehavior(replacement, baseClass);
 
-  /* Create the set of setter methods overridden.
+  /* Create the dictionary of setter methods overridden.
    */
-  keys = [NSMutableSet new];
+  keys = [NSMutableDictionary new];
 
   return self;
 }
 
 - (void) overrideSetterFor: (NSString*)aKey
 {
-  if ([keys member: aKey] == nil)
+  if ([keys objectForKey: aKey] == nil)
     {
       NSMethodSignature	*sig;
       SEL		sel;
@@ -492,9 +493,13 @@ cifframe_callback(ffi_cif *cif, void *retp, void **args, void *user)
       NSString          *suffix;
       NSString          *a[2];
       unsigned          i;
+      
       BOOL              found = NO;
       NSString		*tmp;
-      unichar u;
+      unichar 		u;
+#if defined(USE_LIBFFI)
+      GSCodeBuffer    	*b = nil;
+#endif
 
       suffix = [aKey substringFromIndex: 1];
       u = uni_toupper([aKey characterAtIndex: 0]);
@@ -609,10 +614,7 @@ cifframe_callback(ffi_cif *cif, void *retp, void **args, void *user)
                 else
                   {
 #if defined(USE_LIBFFI)
-                    GSCodeBuffer    *b;
-
                     b = cifframe_closure(sig, cifframe_callback);
-                    [b retain];
                     imp = [b executable];
 #else
                     imp = 0;
@@ -639,7 +641,16 @@ cifframe_callback(ffi_cif *cif, void *retp, void **args, void *user)
         }
       if (found == YES)
         {
-          [keys addObject: aKey];
+	  id	info = nil;
+
+#if defined(USE_LIBFFI)
+	  info = b;	// Need to safe the code buffer
+#endif
+	  if (nil == info)
+	    {
+	      info = [NSNull null];
+	    }
+          [keys setObject: info forKey: aKey];
         }
       else
         {
@@ -647,15 +658,15 @@ cifframe_callback(ffi_cif *cif, void *retp, void **args, void *user)
 
           if (depKeys)
             {
-              NSMapEnumerator enumerator = NSEnumerateMapTable(depKeys);
-              NSString *mainKey;
-              NSHashTable *dependents;
+              NSMapEnumerator	enumerator = NSEnumerateMapTable(depKeys);
+              NSString 		*mainKey;
+              NSHashTable 	*dependents;
 
               while (NSNextMapEnumeratorPair(&enumerator, (void **)(&mainKey),
                 (void**)&dependents))
                 {
-                  NSHashEnumerator dependentKeyEnum;
-                  NSString *dependentKey;
+                  NSHashEnumerator	dependentKeyEnum;
+                  NSString 		*dependentKey;
 
                   if (!dependents) continue;
                   dependentKeyEnum = NSEnumerateHashTable(dependents);
@@ -666,7 +677,7 @@ cifframe_callback(ffi_cif *cif, void *retp, void **args, void *user)
                         {
                           [self overrideSetterFor: mainKey];
                           // Mark the key as used
-                          [keys addObject: aKey];
+                          [keys setObject: [NSNull null] forKey: aKey];
                           found = YES;
                         }
                     }
@@ -1160,8 +1171,7 @@ cifframe_callback(ffi_cif *cif, void *retp, void **args, void *user)
   if (observation == nil)
     {
       observation = [GSKVOObservation new];
-      GSAssignZeroingWeakPointer((void**)&observation->observer,
-	(void*)anObserver);
+      observation->observer = anObserver;
       observation->context = aContext;
       observation->options = options;
       [pathInfo->observations addObject: observation];
@@ -1542,6 +1552,7 @@ cifframe_callback(ffi_cif *cif, void *retp, void **args, void *user)
     {
       info = [[GSKVOInfo alloc] initWithInstance: self];
       [self setObservationInfo: info];
+      RELEASE(info);
       object_setClass(self, [r replacement]);
     }
 
@@ -1591,7 +1602,6 @@ cifframe_callback(ffi_cif *cif, void *retp, void **args, void *user)
        * turn off key-value-observing for it.
        */
       object_setClass(self, [self class]);
-      IF_NO_ARC(AUTORELEASE(info);)
       [self setObservationInfo: nil];
     }
   if ([aPath rangeOfString:@"."].location != NSNotFound)
@@ -1618,7 +1628,6 @@ cifframe_callback(ffi_cif *cif, void *retp, void **args, void *user)
        * turn off key-value-observing for it.
        */
       object_setClass(self, [self class]);
-      IF_NO_GC(AUTORELEASE(info);)
       [self setObservationInfo: nil];
     }
   [kvoLock unlock];
@@ -2071,6 +2080,60 @@ cifframe_callback(ffi_cif *cif, void *retp, void **args, void *user)
 
 @end
 
+@implementation NSObject (NSKeyValueObservingPrivate)
+
+- (void)_notifyObserversOfChangeForKey:(NSString *)aKey
+                              oldValue:(id)old
+                              newValue:(id)new
+{
+  GSKVOPathInfo *pathInfo;
+  GSKVOInfo     *info;
+
+  info = (GSKVOInfo *)[self observationInfo];
+  if (info == nil)
+    {
+      return;
+    }
+          
+  if (new == nil)
+    {
+      new = null;
+    }
+  if (old == nil)
+    {
+      old = null;
+    }
+
+  pathInfo = [info lockReturningPathInfoForKey: aKey];
+  if (pathInfo != nil)
+    {
+      if (pathInfo->recursion++ == 0)
+        {
+          [pathInfo->change setObject: old
+                               forKey: NSKeyValueChangeOldKey];
+          [pathInfo->change removeObjectForKey: NSKeyValueChangeNewKey];
+          [pathInfo->change setValue:
+            [NSNumber numberWithInt: NSKeyValueChangeSetting]
+            forKey: NSKeyValueChangeKindKey];
+          [pathInfo notifyForKey: aKey ofInstance: [info instance] prior: YES];
+
+          [pathInfo->change setValue: new
+                              forKey: NSKeyValueChangeNewKey];
+          [pathInfo notifyForKey: aKey ofInstance: [info instance] prior: NO];
+        }
+      if (pathInfo->recursion > 0)
+        {
+          pathInfo->recursion--;
+        }
+      [info unlock];
+    }
+
+  [self willChangeValueForDependentsOfKey: aKey];
+  [self didChangeValueForDependentsOfKey: aKey];
+}
+
+@end
+
 @implementation NSObject (NSKeyValueObservingCustomization)
 
 + (BOOL) automaticallyNotifiesObserversForKey: (NSString*)aKey
