@@ -18,8 +18,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 
    */
 
@@ -253,12 +252,15 @@ static NSMapTable	*globalClassMap = 0;
       if (_delegate != nil)
 	{
 	  r = [_delegate unarchiver: self didDecodeObject: o];
-	  if (r != o)
+	  /* Apple documentation says that the delegate may return nil to
+	   * indicate that the decoded objects should not be changed.
+	   */
+	  if (r != nil && r != o)
 	    {
 	      [_delegate unarchiver: self
 		  willReplaceObject: o
 			 withObject: r];
-	      o = r;
+	      o = RETAIN(r);
 	      GSIArraySetItemAtIndex(_objMap, (GSIArrayItem)o, index);
 	    }
 	}
@@ -295,8 +297,6 @@ static NSMapTable	*globalClassMap = 0;
 
 + (void) initialize
 {
-  GSMakeWeakPointer(self, "delegate");
-
   if (globalClassMap == 0)
     {
       globalClassMap =
