@@ -18,14 +18,14 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 
    <title>NSLog reference</title>
    $Date$ $Revision$
    */
 
 #import "common.h"
+#import "GSPThread.h"
 #import "Foundation/NSDate.h"
 #import "Foundation/NSCalendarDate.h"
 #import "Foundation/NSTimeZone.h"
@@ -101,14 +101,16 @@ GSLogLock()
 {
   if (myLock == nil)
     {
-      [gnustep_global_lock lock];
+      static gs_mutex_t	setupLock = GS_MUTEX_INIT_STATIC;
+
+      GS_MUTEX_LOCK(setupLock);
       if (myLock == nil)
 	{
 	  myLock = [NSRecursiveLock new];
           lockImp = [myLock methodForSelector: @selector(lock)];
           unlockImp = [myLock methodForSelector: @selector(unlock)];
 	}
-      [gnustep_global_lock unlock];
+      GS_MUTEX_UNLOCK(setupLock);
     }
   return myLock;
 }
