@@ -18,8 +18,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 
    <title>NSObject class reference</title>
    $Date$ $Revision$
@@ -84,6 +83,17 @@
 #endif
 #endif
 
+/* objc_enumerationMutation() is called whenever a collection mutates in the
+ * middle of fast enumeration.  We need to have this defined and linked into
+ * any code that uses fast enumeration, so we define it in NSObject.h
+ * This symbol is exported to take precedence over the weak symbol provided
+ * by the runtime library.
+ */
+GS_EXPORT void objc_enumerationMutation(id obj)
+{
+  [NSException raise: NSGenericException 
+    format: @"Collection %@ was mutated while being enumerated", obj];
+}
 
 /* platforms which do not support weak */
 #if defined (__WIN32)
@@ -141,7 +151,6 @@ BOOL	NSDeallocateZombies = NO;
 static Class		zombieClass = Nil;
 static NSMapTable	*zombieMap = 0;
 
-#ifndef OBJC_CAP_ARC
 static void GSMakeZombie(NSObject *o, Class c)
 {
   object_setClass(o, zombieClass);
@@ -155,9 +164,8 @@ static void GSMakeZombie(NSObject *o, Class c)
       GS_MUTEX_UNLOCK(allocationLock);
     }
 }
-#endif
 
-static void GSLogZombie(id o, SEL sel)
+extern void GSLogZombie(id o, SEL sel)
 {
   Class	c = 0;
 
@@ -172,13 +180,13 @@ static void GSLogZombie(id o, SEL sel)
     }
   if (c == 0)
     {
-      NSLog(@"*** -[??? %@]: message sent to deallocated instance %p",
-	NSStringFromSelector(sel), o);
+      fprintf(stderr, "*** -[??? %s]: message sent to deallocated instance %p",
+	sel_getName(sel), o);
     }
   else
     {
-      NSLog(@"*** -[%@ %@]: message sent to deallocated instance %p",
-	c, NSStringFromSelector(sel), o);
+      fprintf(stderr, "*** -[%s %s]: message sent to deallocated instance %p",
+	class_getName(c), sel_getName(sel), o);
     }
   if (GSPrivateEnvironmentFlag("CRASH_ON_ZOMBIE", NO) == YES)
     {
@@ -209,8 +217,7 @@ static void GSLogZombie(id o, SEL sel)
 #undef	GSATOMICREAD
 #endif
 
-#if defined(__llvm__) || (defined(USE_ATOMIC_BUILTINS) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 1)))
-/* Use the GCC atomic operations with recent GCC versions */
+#ifdef OBJC_CAP_ARC
 
 typedef intptr_t volatile *gsatomic_t;
 typedef intptr_t gsrefcount_t;
@@ -218,6 +225,15 @@ typedef intptr_t gsrefcount_t;
 #define GSAtomicIncrement(X)    __sync_add_and_fetch(X, 1)
 #define GSAtomicDecrement(X)    __sync_sub_and_fetch(X, 1)
 
+#elif (defined(USE_ATOMIC_BUILTINS) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 1)))
+/* Use the GCC atomic operations with recent GCC versions */
+
+typedef int32_t volatile *gsatomic_t;
+typedef int32_t gsrefcount_t;
+#define GSATOMICREAD(X) (*(X))
+#define GSAtomicIncrement(X)    __sync_add_and_fetch(X, 1)
+#define GSAtomicDecrement(X)    __sync_sub_and_fetch(X, 1)
+
 #elif	defined(_WIN32)
 
 /* Set up atomic read, increment and decrement for mswindows
@@ -383,7 +399,7 @@ GSAtomicDecrement(gsatomic_t X)
 
 #include <pthread.h>
 
-typedef int     gsrefcount_t;   // No atomics, use a simple integer
+typedef int32_t	gsrefcount_t;   // No atomics, use a simple integer
 
 /* Having just one allocationLock for all leads to lock contention
  * if there are lots of threads doing lots of retain/release calls.
@@ -416,12 +432,21 @@ static inline pthread_mutex_t   *GSAllocationLockForObject(id p)
 #endif
 #define alignof(type) __builtin_offsetof(struct { const char c; type member; }, member)
 
+#ifndef OBJC_CAP_ARC
+typedef struct {
+  BOOL	hadWeakReference: 1;	// set if the instance ever had a weak reference
+} gsinstinfo_t;
+#endif
+
 /*
  *	Define a structure to hold information that is held locally
  *	(before the start) in each object.
  */
 typedef struct obj_layout_unpadded {
   gsrefcount_t	retained;
+#ifndef OBJC_CAP_ARC
+  gsinstinfo_t	extra;
+#endif
 } unp;
 #define	UNP sizeof(unp)
 
@@ -442,9 +467,26 @@ struct obj_layout {
   char	padding[__BIGGEST_ALIGNMENT__ - ((UNP % __BIGGEST_ALIGNMENT__)
     ? (UNP % __BIGGEST_ALIGNMENT__) : __BIGGEST_ALIGNMENT__)];
   gsrefcount_t	retained;
+#ifndef OBJC_CAP_ARC
+  gsinstinfo_t	extra;
+#endif
 };
 typedef	struct obj_layout *obj;
 
+#ifndef OBJC_CAP_ARC
+BOOL
+GSPrivateMarkedWeak(id anObject, BOOL  mark)
+{
+  BOOL	wasMarked = ((obj)anObject)[-1].extra.hadWeakReference;
+
+  if (mark)
+    {
+      ((obj)anObject)[-1].extra.hadWeakReference = YES;
+    }
+  return wasMarked;
+}
+#endif
+
 /*
  * These symbols are provided by newer versions of the GNUstep Objective-C
  * runtime.  When linked against an older version, we will use our internal
@@ -494,9 +536,7 @@ static BOOL objc_release_fast_no_destroy_internal(id anObject)
          * have been greater than zero)
          */
         (((obj)anObject)[-1].retained) = 0;
-#  ifdef OBJC_CAP_ARC
         objc_delete_weak_refs(anObject);
-#  endif
         return YES;
       }
 #else	/* GSATOMICREAD */
@@ -505,9 +545,7 @@ static BOOL objc_release_fast_no_destroy_internal(id anObject)
     pthread_mutex_lock(theLock);
     if (((obj)anObject)[-1].retained == 0)
       {
-#  ifdef OBJC_CAP_ARC
         objc_delete_weak_refs(anObject);
-#  endif
         pthread_mutex_unlock(theLock);
         return YES;
       }
@@ -570,12 +608,12 @@ NSDecrementExtraRefCountWasZero(id anObject)
   return release_fast_no_destroy(anObject);
 }
 
-size_t object_getRetainCount_np_internal(id anObject)
+static size_t object_getRetainCount_np_internal(id anObject)
 {
   return ((obj)anObject)[-1].retained + 1;
 }
 
-size_t getRetainCount(id anObject)
+static size_t getRetainCount(id anObject)
 {
 #ifdef __GNUSTEP_RUNTIME__
   if (object_getRetainCount_np)
@@ -636,7 +674,8 @@ static id objc_retain_fast_np_internal(id anObject)
 #endif	/* GSATOMICREAD */
   if (YES == tooFar)
     {
-      static NSHashTable        *overrun = nil;
+      static NSHashTable	*overrun = nil;
+      static gs_mutex_t       	countLock = GS_MUTEX_INIT_STATIC;
 
       /* We store this instance in a hash table so that we will only raise
        * an exception for it once (and can therefore expect to log the instance
@@ -644,7 +683,7 @@ static id objc_retain_fast_np_internal(id anObject)
        * NB. The hash table does not retain the object, so the code in the
        * lock protected region below should be safe anyway.
        */
-      [gnustep_global_lock lock];
+      GS_MUTEX_LOCK(countLock);
       if (nil == overrun)
         {
           overrun = NSCreateHashTable(NSNonRetainedObjectHashCallBacks, 0);
@@ -657,7 +696,7 @@ static id objc_retain_fast_np_internal(id anObject)
         {
           tooFar = NO;
         }
-      [gnustep_global_lock lock];
+      GS_MUTEX_UNLOCK(countLock);
       if (YES == tooFar)
         {
           NSString      *base;
@@ -814,33 +853,28 @@ NSDeallocateObject(id anObject)
       (*finalize_imp)(anObject, finalize_sel);
 
       AREM(aClass, (id)anObject);
-      if (NSZombieEnabled == YES)
+      if (NSZombieEnabled)
 	{
-#ifdef OBJC_CAP_ARC
-	  if (0 != zombieMap)
-	    {
-              GS_MUTEX_LOCK(allocationLock);
-              if (0 != zombieMap)
-                {
-                  NSMapInsert(zombieMap, (void*)anObject, (void*)aClass);
-                }
-              GS_MUTEX_UNLOCK(allocationLock);
-	    }
-	  if (NSDeallocateZombies == YES)
-	    {
-	      object_dispose(anObject);
-	    }
-	  else
-	    {
-	      object_setClass(anObject, zombieClass);
-	    }
-#else
+	  /* Replace the isa pointer etc to turn the object into a zombie.
+	   */
 	  GSMakeZombie(anObject, aClass);
-	  if (NSDeallocateZombies == YES)
+
+	  if (NSDeallocateZombies)
 	    {
+#if	defined(OBJC_CAP_ARC)
+	      /* On the modern runtime object_dispose() is called to free
+	       * an instance, but that needs to look at the isa pointer
+	       * and that is now the zombie class so we cant use it.
+	       * So NSDeallocateZombies does nothing.
+	       */
+#else
+	      /* On the classic runtime it makes sense to have an option to
+	       * free memory as the isa pointer in the freed memory may let
+	       * it work as a zombie until it is overwritten.
+	       */
 	      NSZoneFree(z, o);
-	    }
 #endif
+	    }
 	}
       else
 	{
@@ -947,6 +981,8 @@ static id gs_weak_load(id obj)
 {
 #ifdef OBJC_CAP_ARC
   _objc_weak_load = gs_weak_load;
+#else
+  GSWeakInit();
 #endif
   objc_create_block_classes_as_subclasses_of(self);
 }
@@ -1042,13 +1078,6 @@ static id gs_weak_load(id obj)
       }
 #endif
 
-      /* Create the global lock.
-       * NB. Ths is one of the first things we do ... setting up a new lock
-       * must not call any other Objective-C classes and must not involve
-       * any use of the autorelease system.
-       */
-      gnustep_global_lock = [GSUntracedRecursiveLock new];
-
       /* Behavior debugging ... enable with environment variable if needed.
        */
       GSObjCBehaviorDebug(GSPrivateEnvironmentFlag("GNUSTEP_BEHAVIOR_DEBUG",
@@ -1073,16 +1102,22 @@ static id gs_weak_load(id obj)
        */
       NSConstantStringClass = [NSString constantStringClass];
 
-      /* Now that the string class (and autorelease) is set up, we can set
-       * the name of the lock to a string value safely.
-       */
-      [gnustep_global_lock setName: @"gnustep_global_lock"];
-
       /* Determine zombie management flags and set up a map to store
        * information about zombie objects.
        */
       NSZombieEnabled = GSPrivateEnvironmentFlag("NSZombieEnabled", NO);
-      NSDeallocateZombies = GSPrivateEnvironmentFlag("NSDeallocateZombies", NO);
+      if (NSZombieEnabled)
+	{
+          NSDeallocateZombies
+	    = GSPrivateEnvironmentFlag("NSDeallocateZombies", NO);
+#ifdef OBJC_CAP_ARC
+	  if (NSDeallocateZombies)
+	    {
+	      fprintf(stderr, "WARNING the NSDeallocateZombies environment"
+		" setting has no effect with this Objective-C runtime.\n");
+	    }
+#endif
+	}
       zombieMap = NSCreateMapTable(NSNonOwnedPointerMapKeyCallBacks,
 	NSNonOwnedPointerMapValueCallBacks, 0);
 
@@ -1098,12 +1133,14 @@ static id gs_weak_load(id obj)
 
 + (void) _atExit
 {
+/*
   NSMapTable	*m = nil;
   GS_MUTEX_LOCK(allocationLock);
   m = zombieMap;
   zombieMap = nil;
   GS_MUTEX_UNLOCK(allocationLock);
   DESTROY(m);
+*/
 }
 
 /**
@@ -2163,26 +2200,6 @@ static id gs_weak_load(id obj)
   return NSDefaultMallocZone();
 }
 
-/**
- * Called to encode the instance variables of the receiver to aCoder.<br />
- * Subclasses should call the superclass method at the start of their
- * own implementation.
- */
-- (void) encodeWithCoder: (NSCoder*)aCoder
-{
-  return;
-}
-
-/**
- * Called to intialise instance variables of the receiver from aDecoder.<br />
- * Subclasses should call the superclass method at the start of their
- * own implementation.
- */
-- (id) initWithCoder: (NSCoder*)aDecoder
-{
-  return self;
-}
-
 + (BOOL) resolveClassMethod: (SEL)name
 {
   return NO;
@@ -2484,13 +2501,21 @@ static id gs_weak_load(id obj)
     }
   return c;
 }
+- (NSUInteger) retainCount
+{
+  return 0;	// So that gs_weak_load() knows the object was deallocated
+}
+- (void) logZombie: (SEL)selector
+{
+  GSLogZombie(self, selector);
+}
 - (void) forwardInvocation: (NSInvocation*)anInvocation
 {
   NSUInteger	size = [[anInvocation methodSignature] methodReturnLength];
   unsigned char	v[size];
 
   memset(v, '\0', size);
-  GSLogZombie(self, [anInvocation selector]);
+  [self logZombie: [anInvocation selector]];
   [anInvocation setReturnValue: (void*)v];
   return;
 }
