@@ -18,8 +18,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 
 */
 #import "common.h"
@@ -179,30 +178,17 @@ static SEL	remSel;
 {
    if ([aCoder allowsKeyedCoding])
     {
-      if ([aCoder class] == [NSKeyedArchiver class])
-	{
-          /* HACK ... MacOS-X seems to code differently if the coder is an
-           * actual instance of NSKeyedArchiver
-           */
-          NSArray *array = [self array];
-
-	  [(NSKeyedArchiver*)aCoder _encodeArrayOfObjects: array
-						   forKey: @"NS.objects"];
-	}
-      else
-	{
-	  unsigned	i = 0;
-	  NSEnumerator	*e = [self objectEnumerator];
-	  id		o;
-
-	  while ((o = [e nextObject]) != nil)
-	    {
-	      NSString	*key;
-
-	      key = [NSString stringWithFormat: @"NS.object.%u", i++];
-	      [(NSKeyedArchiver*)aCoder encodeObject: o forKey: key];
-	    }
-	}
+      unsigned	i = 0;
+      NSEnumerator	*e = [self objectEnumerator];
+      id		o;
+    
+      while ((o = [e nextObject]) != nil)
+        {
+          NSString	*key;
+    
+          key = [NSString stringWithFormat: @"NS.object.%u", i++];
+          [(NSKeyedArchiver*)aCoder encodeObject: o forKey: key];
+        }
     }
   else
     {
@@ -212,9 +198,9 @@ static SEL	remSel;
 
       [aCoder encodeValueOfObjCType: @encode(unsigned) at: &count];
       while ((o = [e nextObject]) != nil)
-	{
-	  [aCoder encodeValueOfObjCType: @encode(id) at: &o];
-	}
+        {
+          [aCoder encodeValueOfObjCType: @encode(id) at: &o];
+        }
     }
 }
 
@@ -339,7 +325,7 @@ static SEL	remSel;
 
   if (count == 0)
     {
-      return [self init];
+      self = [self init];
     }
   else
     {
@@ -360,10 +346,8 @@ static SEL	remSel;
 	}
       self = [self initWithObjects: objs count: count];
       GS_ENDIDBUF();
-      return self;
     }
-
-  return nil;
+  return self;
 }
 
 - (instancetype) initWithArray: (NSArray *)other copyItems: (BOOL)flag
@@ -429,11 +413,11 @@ static SEL	remSel;
 	  {
 	    if (flag == YES)
 	      {
-		objs[i] = [[other objectAtIndex: i] copy];
+		objs[j] = [[other objectAtIndex: i] copy];
 	      }
 	    else
 	      {
-		objs[i] = [other objectAtIndex: i];
+		objs[j] = [other objectAtIndex: i];
 	      }
 	    j++;
 	  }
@@ -448,7 +432,7 @@ static SEL	remSel;
 
   if (flag == YES)
     {
-      while(j--)
+      while (j--)
 	{
 	  [objs[j] release];
 	}
@@ -528,23 +512,22 @@ static SEL	remSel;
                           copyItems: (BOOL)flag
 {
   unsigned	c = [other count];
-  id		o, e = [other objectEnumerator];
   unsigned	i = 0, j = 0;
   unsigned      loc = range.location;
   unsigned      len = range.length;
   GS_BEGINIDBUF(os, c);
 
-  while ((o = [e nextObject]))
+  FOR_IN(id, o, other)
     {
       if (i >= loc && j < len)
 	{
 	  if (flag)
             {
-              os[i] = [o copy];
+              os[j] = [o copy];
             }
 	  else
             {
-              os[i] = o;
+              os[j] = o;
             }
 	  j++;
 	}
@@ -555,13 +538,14 @@ static SEL	remSel;
 	  break;
 	}
     }
+  END_FOR_IN(other)
 
   self = [self initWithObjects: os count: c];
   if (flag)
     {
-      while (i--)
+      while (j--)
         {
-          [os[i] release];
+          [os[j] release];
         }
     }
   GS_ENDIDBUF();
