@@ -18,8 +18,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 
    <title>NSPort class reference</title>
    $Date$ $Revision$
@@ -70,8 +69,6 @@ static Class	NSPort_concrete_class;
     {
       NSUserDefaults	*defs;
 
-      GSMakeWeakPointer(self, "delegate");
-
       NSPort_abstract_class = self;
       NSPort_concrete_class = [NSMessagePort class];
 
