@@ -18,8 +18,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 
    */
 
@@ -689,7 +688,7 @@ static BOOL skipSpace(pldata *pld)
   return NO;
 }
 
-static inline id parseQuotedString(pldata* pld)
+static inline id parseQuotedString(pldata* pld) NS_RETURNS_RETAINED
 {
   unsigned	start = ++pld->pos;
   unsigned	escaped = 0;
@@ -892,7 +891,7 @@ static inline id parseQuotedString(pldata* pld)
   return obj;
 }
 
-static inline id parseUnquotedString(pldata *pld)
+static inline id parseUnquotedString(pldata *pld) NS_RETURNS_RETAINED
 {
   unsigned	start = pld->pos;
   unsigned	i;
@@ -932,7 +931,7 @@ static inline id parseUnquotedString(pldata *pld)
   return obj;
 }
 
-static id parsePlItem(pldata* pld)
+static id parsePlItem(pldata* pld) NS_RETURNS_RETAINED
 {
   id	result = nil;
   BOOL	start = (pld->pos == 0 ? YES : NO);
@@ -1033,7 +1032,12 @@ static id parsePlItem(pldata* pld)
 	  result = dict;
 	  if (pld->opt == NSPropertyListImmutable)
 	    {
-              result = GS_IMMUTABLE(result);
+	      if (NO == [result makeImmutable])
+		{
+		  id	tmp = result;
+		  result = [tmp copy];
+		  RELEASE(tmp);
+		}
 	    }
 	}
 	break;
@@ -1085,7 +1089,12 @@ static id parsePlItem(pldata* pld)
 	  result = array;
 	  if (pld->opt == NSPropertyListImmutable)
 	    {
-              result = GS_IMMUTABLE(result);
+	      if (NO == [result makeImmutable])
+		{
+		  id	tmp = result;
+		  result = [tmp copy];
+		  RELEASE(tmp);
+		}
 	    }
 	}
 	break;
@@ -1188,11 +1197,13 @@ static id parsePlItem(pldata* pld)
 	    if (pld->pos >= pld->end)
 	      {
 		pld->err = @"unexpected end of string when parsing data";
+		DESTROY(result);
 		return nil;
 	      }
 	    if (pld->ptr[pld->pos] != '>')
 	      {
 		pld->err = @"unexpected character (wanted '>')";
+		DESTROY(result);
 		return nil;
 	      }
 	    pld->pos++;
@@ -1346,7 +1357,7 @@ static id parsePlItem(pldata* pld)
       if (skipSpace(pld) == YES)
 	{
 	  pld->err = @"extra data after parsed string";
-	  result = nil;		// Not at end of string.
+	  DESTROY(result);	// Not at end of string.
 	}
       else
         {
@@ -2368,30 +2379,46 @@ static BOOL	classInitialized = NO;
 
 + (void) initialize
 {
-  if (classInitialized == NO)
+  NSMutableCharacterSet	*s;
+
+  if (Nil == NSStringClass) NSStringClass = [NSString class];
+  if (Nil == GSStringClass) GSStringClass = [GSString class];
+  if (Nil == NSMutableStringClass)
+    NSMutableStringClass = [NSMutableString class];
+  if (Nil == GSMutableStringClass)
+    GSMutableStringClass = [GSMutableString class];
+
+  if (Nil == NSArrayClass) NSArrayClass = [NSArray class];
+  if (Nil == plArray)
     {
-      NSMutableCharacterSet	*s;
-
-      classInitialized = YES;
-
-      NSStringClass = [NSString class];
-      NSMutableStringClass = [NSMutableString class];
-      NSDataClass = [NSData class];
-      NSDateClass = [NSDate class];
-      NSNumberClass = [NSNumber class];
-      NSArrayClass = [NSArray class];
-      NSDictionaryClass = [NSDictionary class];
-      GSStringClass = [GSString class];
-      GSMutableStringClass = [GSMutableString class];
-
       plArray = [GSMutableArray class];
       plAdd = (id (*)(id, SEL, id))
 	[plArray instanceMethodForSelector: @selector(addObject:)];
+    }
 
+  if (Nil == NSDictionaryClass) NSDictionaryClass = [NSDictionary class];
+  if (Nil == plDictionary)
+    {
       plDictionary = [GSMutableDictionary class];
       plSet = (id (*)(id, SEL, id, id))
 	[plDictionary instanceMethodForSelector: @selector(setObject:forKey:)];
+    }
 
+  if (Nil == NSDataClass) NSDataClass = [NSData class];
+  if (Nil == NSNumberClass) NSNumberClass = [NSNumber class];
+  if (nil == boolN)
+    {
+      boolN = [[NSNumber numberWithBool: NO] retain];
+      [[NSObject leakAt: &boolN] release];
+    }
+  if (nil == boolY)
+    {
+      boolY = [[NSNumber numberWithBool: YES] retain];
+      [[NSObject leakAt: &boolY] release];
+    }
+
+  if (nil == oldQuotables)
+    {
       /* The '$', '.', '/' and '_' characters used to be OK to use in
        * property lists, but OSX now quotes them, so we follow suite.
        */
@@ -2402,7 +2429,10 @@ static BOOL	classInitialized = NO;
       [s invert];
       oldQuotables = s;
       [[NSObject leakAt: &oldQuotables] release];
+    }
 
+  if (nil == xmlQuotables)
+    {
       s = [NSMutableCharacterSet new];
       [s addCharactersInString: @"&<>'\\\""];
       [s addCharactersInRange: NSMakeRange(0x0001, 0x001f)];
@@ -2412,13 +2442,17 @@ static BOOL	classInitialized = NO;
       [s addCharactersInRange: NSMakeRange(0xFFFE, 0x0002)];
       xmlQuotables = s;
       [[NSObject leakAt: &xmlQuotables] release];
-
-      boolN = [[NSNumber numberWithBool: NO] retain];
-      [[NSObject leakAt: &boolN] release];
-
-      boolY = [[NSNumber numberWithBool: YES] retain];
-      [[NSObject leakAt: &boolY] release];
     }
+
+  /* Initialize the date class last as it has external dependencies on
+   * time zone data.
+   */
+  if (Nil == NSDateClass)
+    {
+      NSDateClass = [NSDate class];
+    }
+
+  classInitialized = YES;
 }
 
 + (NSData*) dataFromPropertyList: (id)aPropertyList
@@ -2723,8 +2757,9 @@ GSPropertyListMake(id obj, NSDictionary *loc, BOOL xml,
             GSBinaryPLParser	*p = [GSBinaryPLParser alloc];
             
             p = [p initWithData: data mutability: anOption];
-            result = [p rootObject];
-            RELEASE(p);
+	    /* to avoid a leak on exception, autorelease before parse
+	     */
+            result = [AUTORELEASE(p) rootObject];
           }
           break;
           
