@@ -18,8 +18,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 
    <title>NSSet class reference</title>
    $Date$ $Revision$
@@ -450,7 +449,7 @@ static Class NSMutableSet_concrete_class;
     {
       k[i] = [e nextObject];
     }
-  return AUTORELEASE([[NSArray allocWithZone: NSDefaultMallocZone()]
+  result = AUTORELEASE([[NSArray allocWithZone: NSDefaultMallocZone()]
     initWithObjects: k count: c]);
   GS_ENDIDBUF();
   return result;
