@@ -24,8 +24,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 
    <title>NSThread class reference</title>
 */
@@ -97,6 +96,8 @@ typedef struct {
 
 #define	EXPOSE_NSThread_IVARS	1
 #define	GS_NSThread_IVARS \
+  id                    _stringCollatorCache; \
+  BOOL                  _targetIsBlock; \
   gs_thread_id_t        _pthreadID; \
   NSUInteger            _threadID; \
   GSLockInfo            _lockInfo
@@ -116,6 +117,11 @@ typedef struct {
 #  include <sys/file.h>
 #endif
 
+#if defined(__ANDROID__)
+#  include <string.h>       // For strerror
+#  include <sys/resource.h> // For getpriority and setpriority
+#endif
+
 #if	defined(HAVE_SYS_FCNTL_H)
 #  include <sys/fcntl.h>
 #elif	defined(HAVE_FCNTL_H)
@@ -163,6 +169,8 @@ GS_PRIVATE_INTERNAL(NSThread)
 #define pthreadID (internal->_pthreadID)
 #define threadID (internal->_threadID)
 #define lockInfo (internal->_lockInfo)
+#define targetIsBlock (internal->_targetIsBlock)
+#define stringCollatorCache (internal->_stringCollatorCache)
 
 
 #if defined(HAVE_PTHREAD_MAIN_NP)
@@ -378,11 +386,12 @@ GSSleepUntilIntervalSinceReferenceDate(NSTimeInterval when)
 static NSArray *
 commonModes(void)
 {
+  static gs_mutex_t     modesLock = GS_MUTEX_INIT_STATIC;
   static NSArray	*modes = nil;
 
   if (modes == nil)
     {
-      [gnustep_global_lock lock];
+      GS_MUTEX_LOCK(modesLock);
       if (modes == nil)
 	{
 	  Class	c = NSClassFromString(@"NSApplication");
@@ -398,7 +407,7 @@ commonModes(void)
 		NSDefaultRunLoopMode, NSConnectionReplyMode, nil];
 	    }
 	}
-      [gnustep_global_lock unlock];
+      GS_MUTEX_UNLOCK(modesLock);
     }
   return modes;
 }
@@ -1003,6 +1012,34 @@ unregisterActiveThread(NSThread *thread)
     return NO;
   }
   return YES;
+#elif defined(__ANDROID__)
+/* Android's pthread_setschedparam is currently broken, as it checks
+ * if the priority is in the range of the system's min and max
+ * priorities. The interval bounds are queried with `sched_get_priority_min`,
+ * and `sched_get_priority_max` which just return 0, regardless of the
+ * specified scheduling policy.
+ *
+ * The solution is to use `setpriority` to set the thread
+ * priority. This is possible because on Linux, it is not a per-process setting
+ * as specified by POSIX but a per-thread setting (See the `Bugs` section in `setpriority`).
+ *
+ * Android's internal implementation also relies on this behavior, so it
+ * is safe to use it here.
+ */
+
+  // Clamp pri into the required range.
+  if (pri > 1) { pri = 1; }
+  if (pri < 0) { pri = 0; }
+
+  // Convert [0.0, 1.0] to [-20, 19] range where -20 is the highest
+  // and 19 the lowest priority.
+  int priority = (int)(-20 + (1-pri) * 39);
+  if (setpriority(PRIO_PROCESS, 0, priority) == -1)
+  {
+    NSLog(@"Failed to set thread priority %d: %s", priority, strerror(errno));
+    return NO;
+  }
+  return YES;
 #elif defined(_POSIX_THREAD_PRIORITY_SCHEDULING) && (_POSIX_THREAD_PRIORITY_SCHEDULING > 0)
   int res;
   int	policy;
@@ -1096,6 +1133,18 @@ unregisterActiveThread(NSThread *thread)
         NSLog(@"Unknown thread priority: %d", winPri);
         break;
     }
+#elif defined(__ANDROID__)
+/* See notes in setThreadPriority
+ */
+  int priority = getpriority(PRIO_PROCESS, 0);
+  if (priority == -1)
+  {
+    NSLog(@"Failed to get thread priority: %s", strerror(errno));
+    return pri;
+  }
+
+  // Convert [-20, 19] to [0.0, 1.0] range
+  pri = 1 - (priority + 20) / 39.0;
 #elif defined(_POSIX_THREAD_PRIORITY_SCHEDULING) && (_POSIX_THREAD_PRIORITY_SCHEDULING > 0)
   int res;
   int policy;
@@ -1125,8 +1174,6 @@ unregisterActiveThread(NSThread *thread)
   return pri;
 }
 
-
-
 /*
  * Thread instance methods.
  */
@@ -1150,6 +1197,7 @@ unregisterActiveThread(NSThread *thread)
   DESTROY(_target);
   DESTROY(_arg);
   DESTROY(_name);
+  DESTROY(stringCollatorCache);
   if (_autorelease_vars.pool_cache != 0)
     {
       [NSAutoreleasePool _endThread: self];
@@ -1292,7 +1340,15 @@ unregisterActiveThread(NSThread *thread)
         NSStringFromSelector(_cmd)];
     }
 
-  [_target performSelector: _selector withObject: _arg];
+  if (targetIsBlock)
+    {
+      GSThreadBlock block = (GSThreadBlock)_target;
+      CALL_BLOCK_NO_ARGS(block);
+    }
+  else
+    {
+      [_target performSelector: _selector withObject: _arg];
+    }
 }
 
 - (NSString*) name
@@ -1347,6 +1403,8 @@ unregisterActiveThread(NSThread *thread)
               */
               if (i > 15)
                 {
+                  NSWarnLog(@"Truncating thread name '%s' to 15 characters"
+		    @" due to platform limitations", buf);
                   i = 15;
                 }
               else
@@ -1414,6 +1472,7 @@ nsthreadLauncher(void *thread)
 
   setThreadForCurrentThread(t);
 
+  ENTER_POOL
   /*
    * Let observers know a new thread is starting.
    */
@@ -1426,6 +1485,7 @@ nsthreadLauncher(void *thread)
 		  userInfo: nil];
 
   [t _setName: [t name]];
+  LEAVE_POOL
 
   [t main];
 
@@ -1525,9 +1585,18 @@ nsthreadLauncher(void *thread)
   return _thread_dictionary;
 }
 
+- (id) _stringCollatorCache
+{
+  return (id)stringCollatorCache;
+}
+- (void) _setStringCollatorCache: (id) cache
+{
+  ASSIGN(stringCollatorCache, cache);
+}
+
 @end
 
-
+
 
 @implementation NSThread (GSLockInfo)
 
@@ -1711,7 +1780,6 @@ lockInfoErr(NSString *str)
               if (YES == th->_active && nil != info->wait)
                 {
                   BOOL          wasLocked;
-                  GSStackTrace  *stck;
 
                   if (th == self
                     || NULL != NSHashGet(_activeBlocked, (const void*)th))
@@ -1727,7 +1795,7 @@ lockInfoErr(NSString *str)
                       wasLocked = NO;
                     }
                   if (nil != info->wait
-                    && nil != (stck = NSHashGet(info->held, (const void*)want)))
+                    && nil != (id)NSHashGet(info->held, (const void*)want))
                     {
                       /* This thread holds the lock we are interested in and
                        * is waiting for another lock.
@@ -2094,12 +2162,14 @@ GSRunLoopInfoForThread(NSThread *aThread)
     }
   if (aThread->_runLoopInfo == nil)
     {
-      [gnustep_global_lock lock];
+      static gs_mutex_t	infoLock = GS_MUTEX_INIT_STATIC;
+
+      GS_MUTEX_LOCK(infoLock);
       if (aThread->_runLoopInfo == nil)
         {
           aThread->_runLoopInfo = [GSRunLoopThreadInfo new];
 	}
-      [gnustep_global_lock unlock];
+      GS_MUTEX_UNLOCK(infoLock);
     }
   info = aThread->_runLoopInfo;
   return info;
@@ -2358,6 +2428,31 @@ GSRunLoopInfoForThread(NSThread *aThread)
 
 @end
 
+@implementation NSThread (BlockAdditions)
+
++ (void) detachNewThreadWithBlock: (GSThreadBlock)block
+{
+  NSThread	*thread;
+
+  thread = [[NSThread alloc] initWithBlock: block];
+  [thread start];
+
+  RELEASE(thread);
+}
+
+- (instancetype) initWithBlock: (GSThreadBlock)block
+{
+  if (nil != (self = [self init]))
+    {
+      targetIsBlock = YES;
+      /* Copy block to heap */
+      _target = _Block_copy(block);
+    } 
+  return self;
+}
+
+@end
+
 /**
  * <p>
  *   This function is provided to let threads started by some other
