@@ -18,12 +18,13 @@
    
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
    */ 
 
 #import "common.h"
 
+#import "GSPThread.h"
+
 #if	GS_HAVE_NSURLSESSION
 #import <Foundation/NSURLSession.h>
 #endif
@@ -46,6 +47,7 @@ typedef struct {
 
 
 static NSURLCache	*shared = nil;
+static gs_mutex_t       cacheLock = GS_MUTEX_INIT_STATIC;
 
 @implementation	NSURLCache
 
@@ -62,9 +64,9 @@ static NSURLCache	*shared = nil;
 
 + (void) setSharedURLCache: (NSURLCache *)cache
 {
-  [gnustep_global_lock lock];
+  GS_MUTEX_LOCK(cacheLock);
   ASSIGN(shared, cache);
-  [gnustep_global_lock unlock];
+  GS_MUTEX_UNLOCK(cacheLock);
 }
 
 - (void) dealloc
@@ -82,7 +84,7 @@ static NSURLCache	*shared = nil;
 {
   NSURLCache	*c;
 
-  [gnustep_global_lock lock];
+  GS_MUTEX_LOCK(cacheLock);
   if (shared == nil)
     {
       NSString	*path = nil;
@@ -95,7 +97,7 @@ static NSURLCache	*shared = nil;
       
     }
   c = RETAIN(shared);
-  [gnustep_global_lock unlock];
+  GS_MUTEX_UNLOCK(cacheLock);
   return AUTORELEASE(c);
 }
 
