@@ -18,8 +18,7 @@
    
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
    */ 
 
 #import "common.h"
@@ -39,6 +38,7 @@ typedef struct {
   NSMutableDictionary		*headers;
   BOOL				shouldHandleCookies;
   BOOL                          debug;
+  BOOL        assumesHTTP3Capable;
   id<GSLogDelegate>             ioDelegate;
   NSURL				*URL;
   NSURL				*mainDocumentURL;
@@ -56,6 +56,9 @@ typedef struct {
 @interface	_GSMutableInsensitiveDictionary : NSMutableDictionary
 @end
 
+@interface	_GSInsensitiveDictionary : NSMutableDictionary
+@end
+
 @implementation	NSURLRequest
 
 + (id) allocWithZone: (NSZone*)z
@@ -116,6 +119,7 @@ typedef struct {
 	  ASSIGN(inst->bodyStream, this->bodyStream);
 	  ASSIGN(inst->method, this->method);
 	  inst->shouldHandleCookies = this->shouldHandleCookies;
+    inst->assumesHTTP3Capable = this->assumesHTTP3Capable;
 	  inst->debug = this->debug;
 	  inst->ioDelegate = this->ioDelegate;
 	  inst->headers = [this->headers mutableCopy];
@@ -375,6 +379,11 @@ typedef struct {
   return [this->headers objectForKey: field];
 }
 
+- (BOOL) assumesHTTP3Capable
+{
+  return this->assumesHTTP3Capable;
+}
+
 @end
 
 
@@ -454,6 +463,11 @@ typedef struct {
     }
 }
 
+- (void)setAssumesHTTP3Capable:(BOOL)capable
+{
+  this->assumesHTTP3Capable = capable;
+}
+
 @end
 
 @implementation	NSURLRequest (Private)
@@ -468,6 +482,11 @@ typedef struct {
   return this->ioDelegate;
 }
 
+- (NSDictionary *) _insensitiveHeaders
+{
+  return [this->headers copy];
+}
+
 - (id) _propertyForKey: (NSString*)key
 {
   return [this->properties objectForKey: key];
