@@ -20,8 +20,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 
    <title>NSUserDefaults class reference</title>
    $Date$ $Revision$
@@ -52,11 +51,16 @@
 #import "Foundation/NSTimer.h"
 #import "Foundation/NSValue.h"
 #import "GNUstepBase/GSLocale.h"
-#import "GNUstepBase/GSLock.h"
 #import "GNUstepBase/NSProcessInfo+GNUstepBase.h"
 #import "GNUstepBase/NSString+GNUstepBase.h"
 
+#import "NSKVOInternal.h"
+
 #if	defined(_WIN32)
+#import "win32/NSString+Win32Additions.h"
+
+#include <winnls.h>
+
 /* Fake interface to avoid compiler warnings
  */
 @interface	NSUserDefaultsWin32 : NSUserDefaults
@@ -92,9 +96,11 @@ static NSString		*defaultsFile = @".GNUstepDefaults";
 
 static NSUserDefaults	*sharedDefaults = nil;
 static NSDictionary     *argumentsDictionary = nil;
+static NSString	        *bundleIdentifier = nil;
 static NSString	        *processName = nil;
 static NSRecursiveLock	*classLock = nil;
 static NSLock	        *syncLock = nil;
+static BOOL		initializing = NO;
 
 /* Flag to say whether the sharedDefaults variable has been set up by a
  * call to the +standardUserDefaults method.  If this is YES but the variable
@@ -356,23 +362,119 @@ systemLanguages()
 {
   NSMutableArray *names = [NSMutableArray arrayWithCapacity: 10];
 
-  // Add the languages listed in the LANGUAGE environment variable
-  // (a non-POSIX GNU extension)
+  #if defined(_WIN32)
+  NSEnumerator	*enumerator;
+  NSArray	*languages;
+  NSString	*locale;
+  BOOL 		ret;
+  unsigned long numberOfLanguages = 0;
+  unsigned long length = 7;
+  unsigned long factor = sizeof(wchar_t);
+  wchar_t	*buffer = malloc(length * factor);
+
+  if (!buffer)
+    {
+      return names;
+    }
+
+  /* Returns a wchar_t list of languages in the form ll-CC, where ll is the
+   * two-letter language code, and CC is the two-letter country code.
+   */
+  ret = GetUserPreferredUILanguages(MUI_LANGUAGE_NAME, &numberOfLanguages,
+    buffer, &length);
+  if (!ret)
+    {
+      length = 0;
+      if (GetLastError() == ERROR_INSUFFICIENT_BUFFER
+	&& GetUserPreferredUILanguages(MUI_LANGUAGE_NAME, &numberOfLanguages,
+	  NULL, &length))
+	{
+	  wchar_t *oldBuffer = buffer; 
+
+	  buffer = realloc(buffer, length * factor);
+	  if (!buffer)
+	    {
+	      free(oldBuffer);
+	      return names;
+	    }
+
+	  ret = GetUserPreferredUILanguages(MUI_LANGUAGE_NAME,
+	    &numberOfLanguages, buffer, &length);
+	  if (!ret)
+	    {
+	      free(buffer);
+	      return names;
+	    }
+	}
+    }
+
+  languages = [NSString arrayFromWCharList: buffer length: length];
+  enumerator = [languages objectEnumerator];
+  free(buffer);
+
+  while (nil != (locale = [enumerator nextObject]))
+    {
+      /* Replace "-" Separator with "_" */
+      locale = [locale stringByReplacingOccurrencesOfString: @"-"
+						 withString: @"_"];
+      [names addObjectsFromArray: GSLanguagesFromLocale(locale)];
+    }
+  #elif defined(__ANDROID__)
+  /* When running on Android, the process must be correctly initialized
+   * with GSInitializeProcessAndroid (See NSProcessInfo).
+   *
+   * If the minimum API level is 24 or higher, the user-prefered locales
+   * are retrieved from the Android system and passed as GSAndroidLocaleList
+   * process argument
+   */
+  NSArray	*args = [[NSProcessInfo processInfo] arguments];
+  NSEnumerator	*enumerator = [args objectEnumerator];
+  NSString	*key = nil;
+  NSString	*localeList = nil;
+
+  [enumerator nextObject];	// Skip process name.
+  while (nil != (key = [enumerator nextObject]))
+    {
+      if ([key isEqualToString: @"-GSAndroidLocaleList"])
+        {
+          localeList = [enumerator nextObject];
+          break;
+        }
+    }
+
+  // The locale list is a comma-separated list of locales of form ll-CC
+  if (localeList != nil)
+    {
+      NSString	*locale;
+      NSArray	*locales = [localeList componentsSeparatedByString: @","];
+
+      enumerator = [locales objectEnumerator];
+      while (nil != (locale = [enumerator nextObject]))
+        {
+          [names addObjectsFromArray: GSLanguagesFromLocale(locale)];
+        }
+    }
+  #else
+  /* Add the languages listed in the LANGUAGE environment variable
+   * (a non-POSIX GNU extension)
+   */
   {
-    NSString	*env = [[[NSProcessInfo processInfo] environment]
-				 objectForKey: @"LANGUAGE"];
+    NSString	*env;
+
+    env = [[[NSProcessInfo processInfo] environment] objectForKey: @"LANGUAGE"];
     if (env != nil && [env length] > 0)
       {
-	NSArray *array = [env componentsSeparatedByString: @":"];
-	NSEnumerator *enumerator = [array objectEnumerator];
-	NSString *locale;
+	NSArray		*array = [env componentsSeparatedByString: @":"];
+	NSEnumerator	*enumerator = [array objectEnumerator];
+	NSString	*locale;
+
 	while (nil != (locale = [enumerator nextObject]))
 	  {
 	    [names addObjectsFromArray: GSLanguagesFromLocale(locale)];
 	  }
       }
   }	
-  
+
   // If LANGUAGES did not yield any languages, try LC_MESSAGES
 
   if ([names count] == 0)
@@ -384,6 +486,7 @@ systemLanguages()
 	  [names addObjectsFromArray: GSLanguagesFromLocale(locale)];
 	}
     }
+  #endif
 
   return names;
 }
@@ -559,21 +662,43 @@ newLanguages(NSArray *oldNames)
 + (void) atExit
 {
   DESTROY(sharedDefaults);
+  DESTROY(bundleIdentifier);
   DESTROY(processName);
   DESTROY(argumentsDictionary);
   DESTROY(classLock);
   DESTROY(syncLock);
 }
 
+/* Opt-out off automatic willChange/didChange notifications 
+ * as the KVO behaviour for NSUserDefaults is slightly different.
+ *
+ * We do not notify observers of changes that do not actually
+ * change the value of a key (the value is equal to the old value).
+ *
+ * https://developer.apple.com/documentation/foundation/nsuserdefaults#2926902
+ * "You can use key-value observing to be notified of any updates to
+ * a particular default value. You can also register as an observer for
+ * NSUserDefaultsDidChangeNotification on the defaultCenter notification center
+ * in order to be notified of all updates to a local defaults database."
+ */
++ (BOOL) automaticallyNotifiesObserversForKey: (NSString*)key
+{
+  return NO;
+}
+
 + (void) initialize
 {
-  if (self == [NSUserDefaults class])
+  static BOOL	beenHere = NO;
+
+  if (NO == beenHere && self == [NSUserDefaults class])
     {
       ENTER_POOL
       NSEnumerator      *enumerator;
       NSArray           *args;
       NSString          *key;
 
+      beenHere = YES;
+      initializing = YES;
       nextObjectSel = @selector(nextObject);
       objectForKeySel = @selector(objectForKey:);
       addSel = @selector(addEntriesFromDictionary:);
@@ -591,6 +716,20 @@ newLanguages(NSArray *oldNames)
       [self registerAtExit];
 
       processName = [[[NSProcessInfo processInfo] processName] copy];
+      key = [GSPrivateInfoDictionary(nil) objectForKey: @"CFBundleIdentifier"];
+      if (NO == [key isKindOfClass: [NSString class]])
+	{
+	  /* No bundle identifier found: use the process name instead.
+	   */
+	  key = processName;
+          if (NO == [key isKindOfClass: [NSString class]])
+	    {
+	      fprintf(stderr, "+[NSUserDefaults initialize] unable to"
+		" determine bundle identifier or process name.\n");
+	      key = nil;
+	    }
+	}
+      bundleIdentifier = [key copy];
 
       /* Initialise the defaults flags to take values from the
        * process arguments.  These are otherwise set in updateCache()
@@ -599,7 +738,7 @@ newLanguages(NSArray *oldNames)
        * the +_createArgumentDictionary: method.
        */
       args = [[NSProcessInfo processInfo] arguments];
-      enumerator = [[[NSProcessInfo processInfo] arguments] objectEnumerator];
+      enumerator = [args objectEnumerator];
       [enumerator nextObject];	// Skip process name.
       while (nil != (key = [enumerator nextObject]))
         {
@@ -659,6 +798,7 @@ newLanguages(NSArray *oldNames)
 
       [self _createArgumentDictionary: args];
       LEAVE_POOL
+      initializing = NO;
     }
 }
 
@@ -918,8 +1058,11 @@ newLanguages(NSArray *oldNames)
        */
       [defs->_searchList addObject: GSPrimaryDomain];
       [defs->_searchList addObject: NSArgumentDomain];
-      [defs->_searchList addObject: processName];
-      [defs persistentDomainForName: processName];
+      if (bundleIdentifier)
+	{
+	  [defs->_searchList addObject: bundleIdentifier];
+	  [defs persistentDomainForName: bundleIdentifier];
+	}
       [defs->_searchList addObject: NSGlobalDomain];
       [defs persistentDomainForName: NSGlobalDomain];
       [defs->_searchList addObject: GSConfigDomain];
@@ -1219,6 +1362,7 @@ newLanguages(NSArray *oldNames)
   RELEASE(_tempDomains);
   RELEASE(_changedDomains);
   RELEASE(_dictionaryRep);
+  RELEASE(_defaultsDatabase);
   RELEASE(_fileLock);
   RELEASE(_lock);
   [super dealloc];
@@ -1260,7 +1404,7 @@ newLanguages(NSArray *oldNames)
     {
       DESTROY(_dictionaryRep);
       [_searchList removeObject: aName];
-      index = [_searchList indexOfObject: processName];
+      index = [_searchList indexOfObject: bundleIdentifier];
       index = (index == NSNotFound) ? 0 : (index + 1);
       aName = [aName copy];
       [_searchList insertObject: aName atIndex: index];
@@ -1402,13 +1546,39 @@ newLanguages(NSArray *oldNames)
   [_lock lock];
   NS_DURING
     {
-      GSPersistentDomain	*pd = [_persDomains objectForKey: processName];
+      GSPersistentDomain	*pd;
 
+      pd = [_persDomains objectForKey: bundleIdentifier];
       if (nil != pd)
 	{
-          if ([pd setObject: nil forKey: defaultName])
+	  id	old = [self objectForKey: defaultName];
+
+	  if ([pd setObject: nil forKey: defaultName])
 	    {
-	      [self _changePersistentDomain: processName];
+	      id 	new;
+
+	      [self _changePersistentDomain: bundleIdentifier];
+	      new = [self objectForKey: defaultName];
+	      /* Emit only a KVO notification when the value has actually
+	       * changed, meaning -objectForKey: would return a different
+	       * value than before.
+	       */
+	      if ([new hash] != [old hash])
+		{
+		  [self _notifyObserversOfChangeForKey: defaultName
+					      oldValue: old
+					      newValue: new];
+		}
+	    }
+	  else
+	    {
+	      /* We always notify observers of a change, even if the value
+	       * itself is unchanged.
+	       */
+	      [[NSNotificationCenter defaultCenter]
+		postNotificationName: NSUserDefaultsDidChangeNotification
+		  object: self];
+
 	    }
 	}
       [_lock unlock];
@@ -1528,18 +1698,45 @@ static BOOL isPlistObject(id o)
   NS_DURING
     {
       GSPersistentDomain	*pd;
+      id old;
 
-      pd = [_persDomains objectForKey: processName];
+      pd = [_persDomains objectForKey: bundleIdentifier];
       if (nil == pd)
 	{
-	  pd = [[GSPersistentDomain alloc] initWithName: processName
+	  pd = [[GSPersistentDomain alloc] initWithName: bundleIdentifier
 						  owner: self];
-          [_persDomains setObject: pd forKey: processName];
+          [_persDomains setObject: pd forKey: bundleIdentifier];
 	  RELEASE(pd);
 	}
+      // Make sure to search all domains and not only the process domain
+      old = [self objectForKey: defaultName];
       if ([pd setObject: value forKey: defaultName])
         {
-          [self _changePersistentDomain: processName];
+          id new;
+          
+          /* New value must be fetched from all domains, as there might be
+           * a registered default if value is nil, or the value is
+           * superseded by GSPrimary or NSArgumentDomain
+	   */
+          new = [self objectForKey: defaultName];
+          [self _changePersistentDomain: bundleIdentifier];
+	  
+	  // Emit only a KVO notification when the value has actually changed
+	  if ([new hash] != [old hash])
+	    {
+              [self _notifyObserversOfChangeForKey: defaultName
+					  oldValue: old
+					  newValue: new];
+	    }
+        }
+      else
+        {
+          /* We always notify observers of a change, even if the value
+           * itself is unchanged.
+	   */
+          [[NSNotificationCenter defaultCenter]
+            postNotificationName: NSUserDefaultsDidChangeNotification
+			  object: self];
         }
       [_lock unlock];
     }
@@ -1754,17 +1951,17 @@ static BOOL isPlistObject(id o)
 
 - (BOOL) wantToReadDefaultsSince: (NSDate*)lastSyncDate
 {
-  NSFileManager *mgr;
-  NSDictionary	*attr;
-
-  mgr = [NSFileManager defaultManager];
-  attr = [mgr fileAttributesAtPath: _defaultsDatabase traverseLink: YES];
-  if (lastSyncDate == nil)
+  if (nil == lastSyncDate)
     {
       return YES;
     }
   else
     {
+      NSFileManager	*mgr;
+      NSDictionary	*attr;
+
+      mgr = [NSFileManager defaultManager];
+      attr = [mgr fileAttributesAtPath: _defaultsDatabase traverseLink: YES];
       if (attr == nil)
 	{
 	  return YES;
@@ -1902,15 +2099,15 @@ static BOOL isPlistObject(id o)
       _lastSync = saved;
     }
   // Check and if not existent add the Application and the Global domains
-  if ([_persDomains objectForKey: processName] == nil)
+  if (bundleIdentifier && [_persDomains objectForKey: bundleIdentifier] == nil)
     {
       GSPersistentDomain	*pd;
 
-      pd = [[GSPersistentDomain alloc] initWithName: processName
+      pd = [[GSPersistentDomain alloc] initWithName: bundleIdentifier
 					      owner: self];
-      [_persDomains setObject: pd forKey: processName];
+      [_persDomains setObject: pd forKey: bundleIdentifier];
       RELEASE(pd);
-      [self _changePersistentDomain: processName];
+      [self _changePersistentDomain: bundleIdentifier];
     }
   if ([_persDomains objectForKey: NSGlobalDomain] == nil)
     {
@@ -1923,7 +2120,7 @@ static BOOL isPlistObject(id o)
       [self _changePersistentDomain: NSGlobalDomain];
     }
   [_lock unlock];
-  if (YES == haveChange)
+  if (haveChange)
     {
       [[NSNotificationCenter defaultCenter]
 	postNotificationName: NSUserDefaultsDidChangeNotification
@@ -2149,16 +2346,25 @@ GSPrivateDefaultsFlag(GSUserDefaultFlagType type)
 {
   if (nil == classLock)
     {
-      /* The order of +initialise of NSUserDefaults is such that our
-       * flags[] array is set up directly from the process arguments
-       * before classLock is created, so once * that variable exists
-       * this function may be used safely.
-       */
-      [NSUserDefaults class];
-      if (NO == hasSharedDefaults)
-        {
-          [NSUserDefaults standardUserDefaults];
-        }
+      if (initializing)
+	{
+	  fprintf(stderr, "GSPrivateDefaultsFlag(%d) called within"
+	    " +[NSUserDefaults initialize] always returns NO.\n",
+	    (int)type);
+	}
+      else
+	{
+	  /* The order of +initialise of NSUserDefaults is such that our
+	   * flags[] array is set up directly from the process arguments
+	   * before classLock is created, so once that variable exists
+	   * this function may be used safely.
+	   */
+	  [NSUserDefaults class];
+	  if (NO == hasSharedDefaults)
+	    {
+	      [NSUserDefaults standardUserDefaults];
+	    }
+	}
     }
   return flags[type];
 }
@@ -2305,6 +2511,8 @@ NSDictionary *GSPrivateDefaultLocale()
 
 - (void) _changePersistentDomain: (NSString*)domainName
 {
+  BOOL	haveChange = NO;
+
   NSAssert(nil != domainName, NSInvalidArgumentException);
   [_lock lock];
   NS_DURING
@@ -2323,9 +2531,7 @@ NSDictionary *GSPrivateDefaultLocale()
         {
           updateCache(self);
         }
-      [[NSNotificationCenter defaultCenter]
-	postNotificationName: NSUserDefaultsDidChangeNotification
-		      object: self];
+      haveChange = YES;
       [_lock unlock];
     }
   NS_HANDLER
@@ -2334,6 +2540,12 @@ NSDictionary *GSPrivateDefaultLocale()
       [localException raise];
     }
   NS_ENDHANDLER
+  if (haveChange)
+    {
+      [[NSNotificationCenter defaultCenter]
+	postNotificationName: NSUserDefaultsDidChangeNotification
+		      object: self];
+    }
 }
 
 - (NSString*) _directory
@@ -2403,11 +2615,35 @@ static BOOL isLocked = NO;
 
 - (BOOL) _readDefaults
 {
-  NSEnumerator		*enumerator;
-  NSString		*domainName;
-  BOOL			haveChange = NO;
+  NSFileManager	*mgr = [NSFileManager defaultManager];
+  NSEnumerator	*enumerator;
+  NSString	*domainName;
+  BOOL		haveChange = NO;
+  static BOOL	beenHere = NO;
 
-  enumerator = [[[NSFileManager defaultManager]
+  if (NO == beenHere)
+    {
+      NSString	*npath = _defaultsDatabase;
+      NSString	*opath = _defaultsDatabase;
+
+      beenHere = YES;
+      /* The default domain name for a program changed from being the name
+       * of the executable to being the bundle identifier (if available).
+       * If the domain file does not exist for the new name but does exist
+       * for the old name, we move the file to the modern location.
+       */
+      opath = [npath stringByAppendingPathComponent: processName];
+      opath = [opath stringByAppendingPathExtension: @"plist"];
+      npath = [npath stringByAppendingPathComponent: bundleIdentifier];
+      npath = [npath stringByAppendingPathExtension: @"plist"];
+      if (NO == [mgr isReadableFileAtPath: npath]
+	&& YES == [mgr isReadableFileAtPath: opath])
+	{
+	  [mgr movePath: opath toPath: npath handler: nil];
+	}
+    }
+
+  enumerator = [[mgr
     directoryContentsAtPath: _defaultsDatabase] objectEnumerator];
   while (nil != (domainName = [enumerator nextObject]))
     {
@@ -2420,7 +2656,7 @@ static BOOL isLocked = NO;
 	}
       domainName = [domainName stringByDeletingPathExtension];
 
-      /* We may what to know what domains are being loaded.
+      /* We may want to know what domains are being loaded.
        */
       NSDebugMLog(@"domain name: %@", domainName);
 
@@ -2666,7 +2902,7 @@ static BOOL isLocked = NO;
 
       mgr = [NSFileManager defaultManager];
       disk = nil;
-      if (YES == [mgr isReadableFileAtPath: path])
+      if ([mgr isReadableFileAtPath: path])
         {
           NSData	*data;
 
