@@ -20,8 +20,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 
    */
 
@@ -1012,7 +1011,7 @@ NSLog(@"_processTag <%@%@ %@>", flag?@"/": @"", tag, attributes);
     }
   else
     {
-      entity = @"";             // not resolved
+      ASSIGN(entity, @"");	// not resolved
     }
   return entity;
 }
@@ -1861,8 +1860,6 @@ NSLog(@"_processTag <%@%@ %@>", flag?@"/": @"", tag, attributes);
 
 + (void) initialize
 {
-  GSMakeWeakPointer(self, "_delegate");
-  GSMakeWeakPointer(self, "_owner");
 }
 
 - (void) dealloc
