@@ -18,14 +18,14 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 */
 
 #ifndef	_INCLUDED_NSXMLPRIVATE_H
 #define	_INCLUDED_NSXMLPRIVATE_H
 
 #import "common.h"
+#import "GSPrivate.h"
 
 #ifdef	HAVE_LIBXML
 
@@ -66,6 +66,9 @@
  */
 #define	XMLSTRING(X)	((const unsigned char*)[X UTF8String])
 
+void cleanup_namespaces(xmlNodePtr node, xmlNsPtr ns) GS_ATTRIB_PRIVATE;
+BOOL ensure_oldNs(xmlNodePtr node) GS_ATTRIB_PRIVATE;
+
 inline static unsigned char *XMLStringCopy(NSString *source)
 {
   char *xmlstr;
@@ -114,10 +117,10 @@ StringFromXMLString(const unsigned char *bytes, unsigned length)
 #define	GS_XMLNODETYPE \
 union\
 {\
-	xmlDoc *doc;\
-	xmlDtd *dtd;\
-	xmlEntity *entity;\
-	xmlNode *node;\
+  xmlDoc	*doc;\
+  xmlDtd	*dtd;\
+  xmlEntity	*entity;\
+  xmlNode	*node;\
 }
 
 #else
@@ -136,7 +139,7 @@ union\
  * The 'subNodes' array is used to retain the objects pointed to by subnodes.
  *
  * When we create an Objective-C object for a node we also create objects
- * for all parents up to the root object. (Reusing existing once as we find 
+ * for all parents up to the root object. (Reusing existing ones as we find 
  * them in the _private points of parent nodes)
  * This adds the object to the subnode array of the parent object, retaining 
  * the object in this process.
@@ -155,11 +158,12 @@ union\
  * URI is probably not needed at all ... I'm not sure
  */
 #define GS_NSXMLNode_IVARS \
-  NSUInteger	  kind; \
-  GS_XMLNODETYPE node;  \
-  NSUInteger      options; \
-  id              objectValue; \
-  NSMutableArray *subNodes;
+  NSUInteger	  	kind; \
+  GS_XMLNODETYPE	node;  \
+  NSUInteger      	options; \
+  id              	objectValue; \
+  NSMutableArray 	*subNodes; \
+  xmlDoc		*detached; \
 
 
 /* When using the non-fragile ABI, the instance variables are exposed to the
