@@ -9,6 +9,16 @@
 
 #include "config.h"
 
+/* Branch prediction macros
+ */
+#if !defined(likely)
+#define likely(x)      __builtin_expect(!!(x), 1)
+#endif
+
+#if !defined(unlikely)
+#define unlikely(x)    __builtin_expect(!!(x), 0)
+#endif
+
 #if	defined(HAVE_STRING_H)
 /* For POSIX strerror_r() and others
  */
