@@ -14,7 +14,7 @@ PURPOSE. See the GNU Lesser General Public License for more details.
         
 You should have received a copy of the GNU Lesser General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
+Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 
 */
 
