@@ -15,7 +15,7 @@ PURPOSE. See the GNU Lesser General Public License for more details.
         
 You should have received a copy of the GNU Lesser General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
+Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 
 */
 
@@ -59,14 +59,14 @@ __objc_dynamic_find_symbol(dl_handle_t handle, const char* symbol)
 static dl_symbol_t 
 __objc_dynamic_find_symbol(dl_handle_t handle, const char* symbol)
 {
-    return NULL;
+  return NULL;
 }
 
 /* remove the code from memory associated with the module 'handle' */
 static int 
 __objc_dynamic_unlink(dl_handle_t handle)
 {
-    return 0;
+  return 0;
 }
 
 static char *
@@ -88,13 +88,13 @@ __objc_dynamic_error(FILE *error_stream, const char *error_string)
 static int 
 __objc_dynamic_undefined_symbol_count(void)
 {
-    return 0;
+  return 0;
 }
 
 static char** 
 __objc_dynamic_list_undefined_symbols(void)
 {
-    return NULL;
+  return NULL;
 }
 
 #endif /* __sunos_load_h_INCLUDE */
