@@ -18,13 +18,11 @@
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc.,
-   51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
-   */
+   31 Milk Street #960789 Boston, MA 02196 USA.   */
 
 #include "common.h"
 #define	EXPOSE_NSPort_IVARS	1
 #define	EXPOSE_NSMessagePort_IVARS	1
-#include "GNUstepBase/GSLock.h"
 #include "Foundation/NSArray.h"
 #include "Foundation/NSNotification.h"
 #include "Foundation/NSError.h"
@@ -188,7 +186,7 @@ static Class		messagePortClass = 0;
       ports = NSCreateMapTable(NSNonRetainedObjectMapKeyCallBacks,
 	NSNonOwnedPointerMapValueCallBacks, 0);
       [[NSObject leakAt: &ports] release];
-      messagePortLock = [GSLazyRecursiveLock new];
+      messagePortLock = [NSRecursiveLock new];
       [[NSObject leakAt: &messagePortLock] release];
       security.nLength = sizeof(SECURITY_ATTRIBUTES);
       security.lpSecurityDescriptor = 0;	// Default
@@ -311,7 +309,7 @@ static Class		messagePortClass = 0;
   this->name = [[NSString alloc] initWithFormat: @"%08x%08x",
     ((unsigned)ident), sequence++];
 
-  this->lock = [GSLazyRecursiveLock new];
+  this->lock = [NSRecursiveLock new];
   this->wHandle = INVALID_HANDLE_VALUE;
   this->wEvent = INVALID_HANDLE_VALUE;
 
@@ -373,7 +371,7 @@ static Class		messagePortClass = 0;
       self->_is_valid = YES;
       this->name = [name copy];
 
-      this->lock = [GSLazyRecursiveLock new];
+      this->lock = [NSRecursiveLock new];
 
       this->rState = RS_NONE;
 
@@ -951,16 +949,12 @@ again:
    * and try to use it while it is being deallocated.
    */
   M_LOCK(messagePortLock);
-  if (NSDecrementExtraRefCountWasZero(self))
+  if (1 == [self retainCount])
     {
       NSMapRemove(ports, (void*)[self name]);
-      M_UNLOCK(messagePortLock);
-      [self dealloc];
-    }
-  else
-    {
-      M_UNLOCK(messagePortLock);
     }
+  M_UNLOCK(messagePortLock);
+  [super release];
 }
 
 - (BOOL) sendBeforeDate: (NSDate*)when
