@@ -17,8 +17,7 @@
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the
    Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
-
+   Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
    <title>NSMessagePortNameServer class reference</title>
    */
 
