@@ -2,6 +2,7 @@
 #import <Foundation/NSAutoreleasePool.h>
 #import <Foundation/NSException.h>
 #import <Foundation/NSDebug.h>
+#import <Foundation/NSDictionary.h>
 #import <Foundation/NSObject.h>
 #import <Foundation/NSString.h>
 #import <Foundation/NSObjCRuntime.h>
@@ -79,7 +80,7 @@ static int      c1load = 0;
 int
 main(int argc, char *argv[])
 {
-  id            obj;
+  ENTER_POOL
   Class         cls;
   Class         meta;
   SEL           sel;
@@ -94,6 +95,8 @@ main(int argc, char *argv[])
   const char    *t0;
   const char    *t1;
   const char    *n;
+  unsigned	u;
+  int		i;
 
   t0 = "1@1:@";
   t1 = NSGetSizeAndAlignment(t0, &s, &a);
@@ -118,17 +121,22 @@ main(int argc, char *argv[])
   PASS(0 == class_getVersion(Nil), 
     "class_getVersion() for Nil is 0");
 
-  obj = [NSObject new];
   cls = [SubClass1 class];
 
   PASS(c1initialize != 0, "+initialize was called");
   PASS(c1load != 0, "+load was called");
   PASS(c1initialize > c1load, "+load occurs before +initialize");
   PASS(strcmp(class_getName(cls), "SubClass1") == 0, "class name works");
+#ifdef _WIN32
+  testHopeful = YES; // apparently this is not supported on MinGW/clang
+#endif
   PASS(YES == class_respondsToSelector(cls, @selector(sel2)),
     "class_respondsToSelector() works for class method");
   PASS(YES == class_respondsToSelector(cls, @selector(sel1)),
     "class_respondsToSelector() works for superclass method");
+#ifdef _WIN32
+  testHopeful = NO;
+#endif
   PASS(NO == class_respondsToSelector(cls, @selector(rangeOfString:)),
     "class_respondsToSelector() returns NO for unknown method");
   PASS(NO == class_respondsToSelector(cls, 0),
@@ -151,6 +159,55 @@ main(int argc, char *argv[])
   ivar = class_getInstanceVariable(cls, "ivar1obj");
   PASS(ivar != 0, "class_getInstanceVariable() works for superclass obj ivar");
 
+
+  i = objc_getClassList(NULL, 0);
+  PASS(i > 2, "class list contains a reasonable number of classes");
+  if (i > 2)
+    {
+      int	classCount = i;
+      Class	buf[classCount];
+      BOOL	foundClass = NO;
+      BOOL	foundSubClass = NO;
+
+      i = objc_getClassList(buf, classCount);
+      PASS(i == classCount, "retrieved all classes")
+      for (i = 0; i < classCount; i++)
+	{
+	  n = class_getName(buf[i]);
+	  if (n)
+	    {
+	      if (strcmp(n, "Class1") == 0)
+		{
+		  foundClass = YES;
+		}
+	      else if (strcmp(n, "SubClass1") == 0)
+		{
+		  foundSubClass = YES;
+		}
+	    }
+	}
+      PASS(foundClass && foundSubClass, "found classes in list")
+    }
+
+  u = 0;
+  protocols = objc_copyProtocolList(&u);
+  PASS(protocols && u, "we copied some protocols")
+  if (protocols)
+    {
+      BOOL	found = NO;
+
+      for (i = 0; i < u; i++)
+	{
+	  n = protocol_getName(protocols[i]);
+	  if (strcmp(n, "SubProto") == 0)
+	    {
+	      found = YES;
+	    }
+	}
+      free(protocols);
+      PASS(found, "we found our protocol in list")
+    }
+
   methods = class_copyMethodList(cls, &count);
   PASS(count == 3, "SubClass1 has three methods");
   PASS(methods[count] == 0, "method list is terminated");
@@ -188,6 +245,7 @@ main(int argc, char *argv[])
       PASS(sel_isEqual(sel, sel_getUid("catMethod")),
         "method 1 has expected name");
     }
+  free(methods);
 
   ivars = class_copyIvarList(cls, &count);
   PASS(count == 1, "SubClass1 has one ivar");
@@ -196,12 +254,14 @@ main(int argc, char *argv[])
     "ivar has correct name");
   PASS(strcmp(ivar_getTypeEncoding(ivars[0]), @encode(int)) == 0,
     "ivar has correct type");
+  free(ivars);
 
   protocols = class_copyProtocolList(cls, &count);
   PASS(count == 1, "SubClass1 has one protocol");
   PASS(protocols[count] == 0, "protocol list is terminated");
   PASS(strcmp(protocol_getName(protocols[0]), "SubProto") == 0,
     "protocol has correct name");
+  free(protocols);
 
   cls = objc_allocateClassPair([NSString class], "runtime generated", 0);
   PASS(cls != Nil, "can allocate a class pair");
@@ -213,7 +273,7 @@ main(int argc, char *argv[])
     "able to add iVar 'iv3'");
   PASS(class_addIvar(cls, "iv4", 1, 3, "c") == YES,
     "able to add iVar 'iv4'");
-  objc_registerClassPair (cls);
+  objc_registerClassPair(cls);
   ivar = class_getInstanceVariable(cls, "iv1");
   PASS(ivar != 0, "iv1 exists");
   PASS(ivar_getOffset(ivar) == 64, "iv1 offset is 64");
@@ -240,6 +300,40 @@ main(int argc, char *argv[])
   PASS([NSStringFromSelector(sel) isEqual: @"xxxyyy_odd_name_xxxyyy"],
     "NSStringFromSelector() works for existing selector");
 
+  LEAVE_POOL
+
+  START_SET("weakref")
+  Class	c = [NSObject class];
+  id	obj;
+  id	got;
+  id	ref;
+  int	rc;
+
+  ref = nil;
+
+  objc_storeWeak(&ref, nil);
+  PASS(ref == nil, "nil is stored unchanged")
+
+  objc_storeWeak(&ref, @"hello");
+  PASS(ref == @"hello", "literal string is stored unchanged")
+
+  objc_storeWeak(&ref, (id)c);
+  PASS(ref == (id)c, "a class is stored unchanged")
+
+  obj = [NSObject new];
+  objc_storeWeak(&ref, obj);
+  PASS(ref != obj, "object is stored as weak reference")
+  rc = [obj retainCount];
+  got = objc_loadWeakRetained(&ref);
+  PASS(got == obj && [obj retainCount] == rc + 1,
+    "objc_loadWeakRetained() returns original retained")
+  RELEASE(got);
+  RELEASE(obj);
+  got = objc_loadWeakRetained(&ref);
+  PASS(got == nil, "load of deallocated object returns nil")
+
+  END_SET("weakref")
+
   return 0;
 }
 
