@@ -81,9 +81,9 @@ static void _testBundle(NSBundle* bundle, NSString* path, NSString* className)
   PASS([arr containsObject: localPath],
     "Returned array for 'German' contains localized resource");
   Class clz = [bundle classNamed: className];
-  PASS(clz, "Class can be loaded from bundle");
+  PASS(clz != nil, "Class can be loaded from bundle");
   id obj = [clz new];
-  PASS(obj, "Objects from bundle-loaded classes can be instantiated");
+  PASS(obj != nil, "Objects from bundle-loaded classes can be instantiated");
   PASS_EQUAL([obj test], @"Something", "Correct method called");
   [obj release];
 }
@@ -103,6 +103,9 @@ int main()
   END_SET("Bundle")
 
   START_SET("Framework")
+#if defined(_WIN32)
+  SKIP("Adding a run-time search path is not supported on Windows.")
+#else
   /* This method call is required to ensure that the linker does not decide to
    * elide the framework linkage.
    */
@@ -111,6 +114,7 @@ int main()
   path = [bundle bundlePath];
   _testBundle(bundle, path, @"TestFramework");
   PASS(0 == [bundle bundleVersion], "bundleVersion is zero");
+#endif
   END_SET("Framework");
 
   [arp release]; arp = nil;
