@@ -7,22 +7,24 @@
 
 int main()
 {
-  NSAutoreleasePool   *arp = [NSAutoreleasePool new];
-  NSMutableArray *testObjs = [NSMutableArray new];
-  NSDictionary *obj;
+  NSAutoreleasePool   	*arp = [NSAutoreleasePool new];
+  NSMutableArray 	*testObjs = [NSMutableArray array];
+  NSDictionary 		*obj;
+
   test_alloc(@"NSDictionary");
-  obj = [NSDictionary new];
-  [testObjs addObject:obj];
-  PASS(obj != nil && 
-       [obj isKindOfClass:[NSDictionary class]] &&
-       [obj count] == 0,
-       "can create an empty dictionary");
-  obj = [NSDictionary dictionaryWithObject:@"Hello" forKey:@"Key"];
-  [testObjs addObject:obj];
-  PASS(obj != nil && 
-       [obj isKindOfClass:[NSDictionary class]] &&
-       [obj count] == 1, 
-       "can create a dictionary with one element");
+  obj = [NSDictionary dictionary];
+  [testObjs addObject: obj];
+  PASS(obj != nil
+    && [obj isKindOfClass: [NSDictionary class]]
+    && [obj count] == 0,
+    "can create an empty dictionary")
+
+  obj = [NSDictionary dictionaryWithObject: @"Hello" forKey: @"Key"];
+  [testObjs addObject: obj];
+  PASS(obj != nil
+    && [obj isKindOfClass: [NSDictionary class]]
+    && [obj count] == 1, 
+    "can create a dictionary with one element")
   
   test_NSObject(@"NSDictionary", testObjs);
   test_NSCoding(testObjs);
@@ -36,7 +38,7 @@ int main()
 #if __has_feature(objc_subscripting)
   NSDictionary *dictionary = @{@123 : @123.4 ,
                                @"date" : @"today" };
-  PASS([dictionary[@123] isEqual: @123.4], "Dictionary subscripting works");
+  PASS([dictionary[@123] isEqual: @123.4], "Dictionary subscripting works")
 # else
    SKIP("No dictionary subscripting support in the compiler.")
 # endif
