@@ -5,7 +5,7 @@ id a,b,c;
 
 int main(void)
 {
-  NSAutoreleasePool     *p = [NSAutoreleasePool new];
+  ENTER_POOL
   NSFileHandle  *a;
   NSFileHandle  *b;
   NSFileHandle  *c;
@@ -15,16 +15,10 @@ int main(void)
   b = [NSFileHandle fileHandleWithStandardOutput];
   c = [NSFileHandle fileHandleWithStandardError];
   END_SET("handle creation")
-  PASS([a retainCount]> 0, "stdin persists");
-  PASS([b retainCount]> 0, "stdout persists");
-  PASS([c retainCount]> 0, "strerr persists");
   PASS_EXCEPTION([a release], NSGenericException, "Cannot dealloc stdin");
   PASS_EXCEPTION([b release], NSGenericException, "Cannot dealloc stdout");
   PASS_EXCEPTION([c release], NSGenericException, "Cannot dealloc stderr");
-  PASS([a retainCount]> 0, "stdin persists");
-  PASS([b retainCount]> 0, "stdout persists");
-  PASS([c retainCount]> 0, "strerr persists");
 
-  [p drain];
+  LEAVE_POOL
   return 0;
 }
