@@ -11,25 +11,28 @@ int main()
   NSHost *tmp; 
 
   current = [NSHost currentHost];
-  PASS(current != nil && [current isKindOfClass:[NSHost class]],
+  PASS(current != nil && [current isKindOfClass: [NSHost class]],
        "NSHost understands +currentHost");
  
 #if	defined(GNUSTEP_BASE_LIBRARY)
   localh = [NSHost localHost];
-  PASS(localh != nil && [localh isKindOfClass:[NSHost class]],
+  PASS(localh != nil && [localh isKindOfClass: [NSHost class]],
        "NSHost understands +localHost");
 #else
   localh = current;
 #endif
 
-  tmp = [NSHost hostWithName:[current name]];
-  PASS([tmp isEqualToHost:current], "NSHost understands +hostWithName:");
+  tmp = [NSHost hostWithName: @"::1"];
+  PASS([[tmp address] isEqual: @"::1"], "+hostWithName: works for IPV6 addr");
+
+  tmp = [NSHost hostWithName: [current name]];
+  PASS([tmp isEqualToHost: current], "NSHost understands +hostWithName:");
   
-  tmp = [NSHost hostWithAddress:[current address]];
-  PASS([tmp isEqualToHost:current], "NSHost understands +hostWithAddress:");
+  tmp = [NSHost hostWithAddress: [current address]];
+  PASS([tmp isEqualToHost: current], "NSHost understands +hostWithAddress:");
   
-  tmp = [NSHost hostWithName:@"127.0.0.1"];
-  PASS(tmp != nil && [tmp isEqualToHost:localh], 
+  tmp = [NSHost hostWithName: @"127.0.0.1"];
+  PASS(tmp != nil && [tmp isEqualToHost: localh], 
        "NSHost understands [+hostWithName: 127.0.0.1]");
   
   [arp release]; arp = nil;
