@@ -27,8 +27,9 @@ int main()
   u = [NSURL URLWithString: @"http://www.w3.org/"];
   ms = [NSMutableSet set];
   [ms addObject: u];
-  data2 = [NSMutableData new];
-  archiver = [[NSKeyedArchiver alloc] initForWritingWithMutableData: data2];
+  data2 = [NSMutableData data];
+  archiver = AUTORELEASE([[NSKeyedArchiver alloc]
+    initForWritingWithMutableData: data2]);
   [archiver setOutputFormat: NSPropertyListXMLFormat_v1_0];
   [archiver encodeObject: ms forKey: @"root"];
   [archiver finishEncoding];
@@ -38,28 +39,25 @@ int main()
   PASS([[[ms anyObject] absoluteString] isEqual: @"http://www.w3.org/"],
     "Can archive and restore a URL");
   
-  [archiver release];
-  [data2 release];
-
-
-  PASS_RUNS(val1 = [NSString stringWithCString:"Archiver.dat"];
-		 val2 = [NSString stringWithCString:"A Goodbye"];
-		 val3 = [NSString stringWithCString:"Testing all strings"];
-		 val4 = [NSNumber numberWithUnsignedInt: 100];
-		 vals1 = [[[NSArray arrayWithObject:val1] 
-			    arrayByAddingObject:val2] 
-			   arrayByAddingObject: val4];		 
-		 vals2 = [vals1 arrayByAddingObject: val2];,
-		 "We can build basic strings and arrays for tests");
+  PASS_RUNS(
+    val1 = [NSString stringWithCString:"Archiver.dat"];
+    val2 = [NSString stringWithCString:"A Goodbye"];
+    val3 = [NSString stringWithCString:"Testing all strings"];
+    val4 = [NSNumber numberWithUnsignedInt: 100];
+    vals1 = [[[NSArray arrayWithObject: val1] 
+		    arrayByAddingObject: val2] 
+		    arrayByAddingObject: val4];		 
+    vals2 = [vals1 arrayByAddingObject: val2];,
+    "We can build basic strings and arrays for tests")
   
-  PASS([NSKeyedArchiver archiveRootObject:vals2 toFile:val1],
+  PASS([NSKeyedArchiver archiveRootObject: vals2 toFile: val1],
     "archiveRootObject:toFile: seems ok"); 
   
-  data1 = [NSKeyedArchiver archivedDataWithRootObject:vals2];
+  data1 = [NSKeyedArchiver archivedDataWithRootObject: vals2];
   PASS((data1 != nil && [data1 length] != 0),
     "archivedDataWithRootObject: seems ok");
   
-  a = [NSKeyedUnarchiver unarchiveObjectWithData:data1];
+  a = [NSKeyedUnarchiver unarchiveObjectWithData: data1];
   NSLog(@"From data: original array %@, decoded array %@",vals2, a);
   PASS((a != nil && [a isKindOfClass:[NSArray class]] && [a isEqual:vals2]),
        "unarchiveObjectWithData: seems ok");
@@ -70,17 +68,18 @@ int main()
        "unarchiveObjectWithFile: seems ok");
 
   // encode
-  data2 = [[NSMutableData alloc] initWithCapacity: 10240];
-  archiver = [[NSKeyedArchiver alloc] initForWritingWithMutableData: data2];
+  data2 = [NSMutableData dataWithCapacity: 10240];
+  archiver = AUTORELEASE([[NSKeyedArchiver alloc]
+    initForWritingWithMutableData: data2]);
   [archiver encodeObject: val3 forKey: @"string"];
   [archiver finishEncoding];
 
   // decode...
-  unarchiver = [[NSKeyedUnarchiver alloc] initForReadingWithData: data2];
-  s = [[unarchiver decodeObjectForKey: @"string"] retain];
+  unarchiver = AUTORELEASE([[NSKeyedUnarchiver alloc]
+    initForReadingWithData: data2]);
+  s = [unarchiver decodeObjectForKey: @"string"];
   PASS((s != nil && [s isKindOfClass:[NSString class]] && [s isEqual: val3]),
     "encodeObject:forKey: seems okay");
-  [data2 release];
 
   NSLog(@"Original string: %@, unarchived string: %@",val3, s);
 
