@@ -5,8 +5,10 @@
 
 int main()
 {
+  ENTER_POOL
   Class theClass = NSClassFromString(@"NSObject"); 
   PASS(theClass == [NSObject class],
        "'NSObject' %s","uses +class to return self");
+  LEAVE_POOL
   return 0;
 }
