@@ -8,6 +8,7 @@ copyright 2004 Alexander Malmberg <alexander@malmberg.org>
 
 int main(int argc, char **argv)
 {
+  START_SET("zero hash")
   NSString *s = @"!)9\" ;";
   unsigned int h;
 
@@ -31,10 +32,10 @@ int main(int argc, char **argv)
   h = [s hash];
   PASS(h != 0, "[NSConstantString hash] does not return 0");
 
-  s = [[NSString alloc] initWithString: s];
+  s = [NSString stringWithString: s];
   h = [s hash];
   PASS(h != 0, "[NSString hash] does not return 0");
-
+  END_SET("zero hash")
   return 0;
 }
 
