@@ -11,12 +11,13 @@ constants.
 
 int main(int argc, char **argv)
 {
+  START_SET("nuls in strings")
   NSString *constantString=@"a\0b";
   NSString *normalString;
   unichar characters[3]={'a',0,'b'};
   NSRange r;
 
-  normalString = [[NSString alloc] initWithCharacters: characters length: 3];
+  normalString = AUTORELEASE([[NSString alloc] initWithCharacters: characters length: 3]);
 
   PASS([constantString length] == 3, "nuls in constant strings");
   PASS([normalString length] == 3, "nuls in non-constant strings");
@@ -26,6 +27,7 @@ int main(int argc, char **argv)
   r = [normalString rangeOfString: @"\0"];
   PASS(1 == r.length && 1 == r.location, "find nul in string");
 
+  END_SET("nuls in strings")
   return 0;
 }
 
