@@ -9,16 +9,16 @@ int main()
   unichar theUniChar[1] = {0xe5};
   theString = [NSString stringWithCharacters:theUniChar length:1];
 
-  PASS([theString isEqual:[[NSString alloc] initWithCString: [theString cStringUsingEncoding: NSISOLatin1StringEncoding] encoding: NSISOLatin1StringEncoding]],"foo");
+  PASS([theString isEqual: AUTORELEASE([[NSString alloc] initWithCString: [theString cStringUsingEncoding: NSISOLatin1StringEncoding] encoding: NSISOLatin1StringEncoding])],"foo");
 
   NS_DURING
-    PASS([theString isEqual:[[NSString alloc] initWithCString: [theString cString]]],"foo");
+    PASS([theString isEqual: AUTORELEASE([[NSString alloc] initWithCString: [theString cString]])],"foo");
   NS_HANDLER
     PASS(1,"bar");
   NS_ENDHANDLER
   
   NS_DURING
-    PASS([theString isEqual:[[NSMutableString alloc] initWithCString: [theString cString]]],"foo2");
+    PASS([theString isEqual: AUTORELEASE([[NSMutableString alloc] initWithCString: [theString cString]])],"foo2");
   NS_HANDLER
     PASS(1,"bar2");
   NS_ENDHANDLER
