@@ -81,6 +81,11 @@
   [serverStream scheduleInRunLoop: runLoop forMode: NSDefaultRunLoopMode];
   [serverStream open];
 
+  /* Tell main test program we are ready to handle a request
+   */
+  [[NSFileHandle fileHandleWithStandardOutput] writeData:
+    [@"Ready" dataUsingEncoding: NSASCIIStringEncoding]];
+
   // only run for a fixed time anyway
   [runLoop runUntilDate: [NSDate dateWithTimeIntervalSinceNow: lifetime]];
   
@@ -220,8 +225,9 @@
 
     case NSStreamEventErrorOccurred:
       {
-	int code = [[theStream streamError] code];
-	NSLog(@"Received error %d on stream %p", code, theStream);
+	NSError *err = [theStream streamError];
+	int code = [err code];
+	NSLog(@"Received error %@ (%d) on stream %p", err, code, theStream);
 	[theStream close];
 	[theStream removeFromRunLoop: runLoop forMode: NSDefaultRunLoopMode];
 	if (theStream == inStream) inStream = nil;
