@@ -10,24 +10,17 @@
 
 int main(int argc, char **argv, char **env)
 {
+  
+#if !defined(HAVE_GNUTLS)
+testHopeful = YES;
+#endif
+
   CREATE_AUTORELEASE_POOL(arp);
   NSFileManager *fm;
   NSBundle *bundle;
   BOOL loaded;
   NSString *helperPath;
   
-  /* The following test cases depend on the GSInetServerStream
-   * class which is completely broken on Windows.
-   *
-   * See: https://github.com/gnustep/libs-base/issues/266
-   *
-   * We will mark the test cases as hopeful on Windows.
-   */
-#if defined(_WIN32)
-  NSLog(@"Marking local web server tests as hopeful because GSInetServerStream is broken on Windows");
-  testHopeful = YES;
-#endif
-
   // load the test suite's classes
   fm = [NSFileManager defaultManager];
   helperPath = [[fm currentDirectoryPath]
@@ -42,7 +35,7 @@ int main(int argc, char **argv, char **env)
       NSDictionary *refs;
       TestWebServer *server;
       NSURLConnectionTest *testCase;
-      BOOL debug = NO;
+      BOOL debug = GSDebugSet(@"dflt");
 
       testClass = [bundle principalClass]; // NSURLConnectionTest
 
@@ -53,11 +46,11 @@ int main(int argc, char **argv, char **env)
       // create a shared TestWebServer instance for performance
       server = [[[testClass testWebServerClass] alloc]
         initWithAddress: @"localhost"
-                   port: @"1234"
+                   port: @"1233"
                    mode: NO
                   extra: d];
       [server setDebug: debug];
-      [server start: d]; // localhost:1234 HTTPS
+      [server start: d]; // localhost:1233 HTTPS
 
       /*
        *  Simple GET via HTTPS with empty response's body and
@@ -71,7 +64,7 @@ int main(int argc, char **argv, char **env)
         nil];
       [testCase setUpTest: d];
       [testCase startTest: d];
-      PASS([testCase isSuccess], "HTTPS... GET https://localhost:1234/");
+      PASS([testCase isSuccess], "HTTPS... GET https://localhost:1233/");
       [testCase tearDownTest: d];
       DESTROY(testCase);
 
@@ -90,7 +83,7 @@ int main(int argc, char **argv, char **env)
         nil];
       [testCase setUpTest: d];
       [testCase startTest: d];
-      PASS([testCase isSuccess], "HTTPS... response 400 .... GET https://localhost:1234/400");
+      PASS([testCase isSuccess], "HTTPS... response 400 .... GET https://localhost:1233/400");
       [testCase tearDownTest: d];
       DESTROY(testCase);
 
@@ -111,7 +104,7 @@ int main(int argc, char **argv, char **env)
         nil];
       [testCase setUpTest: d];
       [testCase startTest: d];
-      PASS([testCase isSuccess], "HTTPS... payload... response 400 .... POST https://localhost:1234/400");
+      PASS([testCase isSuccess], "HTTPS... payload... response 400 .... POST https://localhost:1233/400");
       [testCase tearDownTest: d];
       DESTROY(testCase);
 
@@ -133,11 +126,12 @@ int main(int argc, char **argv, char **env)
         @"/301", @"Path",      // request the handler responding with a redirect
         @"/", @"RedirectPath", // the URL's path of redirecting
         @"YES", @"IsAuxilliary", // start an auxilliary TestWebServer instance
+        @"1236", @"AuxPort",   // the port of the auxilliary instance			  
         refs, @"ReferenceFlags", // the expected reference set difference
         nil];      
       [testCase setUpTest: d];
       [testCase startTest: d];
-      PASS([testCase isSuccess], "HTTPS... redirecting... GET https://localhost:1234/301");
+      PASS([testCase isSuccess], "HTTPS... redirecting... GET https://localhost:1233/301");
       [testCase tearDownTest: d];
       DESTROY(testCase);
 
@@ -154,9 +148,9 @@ int main(int argc, char **argv, char **env)
 
   DESTROY(arp);
 
-#if defined(_WIN32)
-  testHopeful = NO;
+#if !defined(HAVE_GNUTLS)
+testHopeful = NO;
 #endif
-  
+
   return 0;
 }
