@@ -16,7 +16,7 @@
    You should have received a copy of the GNU General Public
    License along with this program; see the file COPYINGv3.
    If not, write to the Free Software Foundation,
-   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+   31 Milk Street #960789 Boston, MA 02196 USA.
 
    */
 
@@ -24,6 +24,7 @@
 
 #import "Foundation/NSArray.h"
 #import "Foundation/NSAutoreleasePool.h"
+#import "Foundation/NSBundle.h"
 #import "Foundation/NSCharacterSet.h"
 #import "Foundation/NSData.h"
 #import "Foundation/NSDictionary.h"
@@ -301,6 +302,7 @@ static BOOL snuggleStart(NSString *t)
  */
 - (NSArray*) output: (NSMutableDictionary*)d
 {
+  NSFileManager		*mgr = [NSFileManager defaultManager];
   NSMutableString	*str = [NSMutableString stringWithCapacity: 10240];
   NSDictionary		*classes;
   NSDictionary		*categories;
@@ -312,6 +314,7 @@ static BOOL snuggleStart(NSString *t)
   NSDictionary		*macros;
   NSMutableArray	*files;
   NSArray		*authors;
+  NSString		*style = @"default-styles.css";
   NSString		*base;
   NSString		*tmp;
   NSString		*file;
@@ -328,9 +331,25 @@ static BOOL snuggleStart(NSString *t)
       file = [file stringByAppendingPathExtension: @"gsdoc"];
     }
   dest = [info objectForKey: @"directory"];
-  if ([dest length] > 0 && [file isAbsolutePath] == NO)
+  if ([dest length] > 0)
     {
-      file = [dest stringByAppendingPathComponent: file];
+      style = [dest stringByAppendingPathComponent: style];
+      if ([file isAbsolutePath] == NO)
+	{
+	  file = [dest stringByAppendingPathComponent: file];
+	}
+    }
+
+  /* When there is no local default stylesheet present, we copy the
+   * stylesheet from the main bundle.
+   */
+  if ([mgr isReadableFileAtPath: style] == NO)
+    {
+      NSBundle	*bundle = [NSBundle mainBundle];
+      NSString	*path;
+
+      path = [bundle pathForResource: @"default-styles" ofType: @"css"];
+      [mgr copyPath: path toPath: style handler: nil];
     }
 
   classes = [info objectForKey: @"Classes"];
@@ -355,6 +374,16 @@ static BOOL snuggleStart(NSString *t)
       [str appendString: @"\""];
     }
 
+  tmp = [info objectForKey: @"stylesheeturl"];
+  if (nil == tmp)
+    {
+      tmp = [[NSUserDefaults standardUserDefaults]
+	stringForKey: @"StylesheetURL"];
+    }
+  [str appendString: @" stylesheeturl=\""];
+  [str appendString: tmp];
+  [str appendString: @"\""];
+
   tmp = [info objectForKey: @"up"];
   if (tmp != nil)
     {
@@ -1498,7 +1527,8 @@ static BOOL snuggleStart(NSString *t)
    * or end tag, or one of the whitespace separated words.
    * What about str?
    */
-  data = [[NSMutableData alloc] initWithLength: l * sizeof(unichar)];
+  data = AUTORELEASE([[NSMutableData alloc]
+    initWithLength: l * sizeof(unichar)]);
   ptr = buf = [data mutableBytes];
   [str getCharacters: buf];
   end = buf + l;
@@ -1602,7 +1632,7 @@ static BOOL snuggleStart(NSString *t)
 		  unsigned	len = optr - buf;
 		  unichar	c[len + 1];
 
-		  memcpy(c, buf, (len+1)*sizeof(unichar));
+		  memcpy(c, buf, len*sizeof(unichar));
 		  c[len-2] = ' ';
 		  c[len-1] = '/';
 		  c[len] = '>';
@@ -1977,7 +2007,7 @@ static BOOL snuggleStart(NSString *t)
 		  pos++;
 		  if (pos < ePos
 		    && [identStart characterIsMember:
-		      (c = [tmp characterAtIndex: pos])] == YES)
+		      [tmp characterAtIndex: pos]] == YES)
 		    {
 		      while (pos < ePos)
 			{
@@ -2341,7 +2371,7 @@ static BOOL snuggleStart(NSString *t)
 	   * No pre-existing file, and no blank template available ...
 	   * Generate a standard template.
 	   */
-	  str = [[NSMutableString alloc] initWithCapacity: 1024];
+	  str = [NSMutableString stringWithCapacity: 1024];
 
 	  [str appendString: @"<?xml version=\"1.0\"?>\n"];
 	  [str appendString: @"<!DOCTYPE gsdoc PUBLIC "];
@@ -2350,6 +2380,16 @@ static BOOL snuggleStart(NSString *t)
 
 	  [str appendString: @"<gsdoc base=\""];
 	  [str appendString: [name lastPathComponent]];
+	  [str appendString: @"\""];
+	  if (nil == tmp)
+	    {
+	      tmp = [[NSUserDefaults standardUserDefaults]
+		stringForKey: @"StylesheetURL"];
+	    }
+	  [str appendString: @" stylesheeturl=\""];
+	  [str appendString: tmp];
+	  [str appendString: @"\""];
+
 	  /*
 	   * If a -Up default has been set, create an up link in this
 	   * template file... as long as the specified up link is not
@@ -2357,10 +2397,11 @@ static BOOL snuggleStart(NSString *t)
 	   */
 	  if (up != nil && [up isEqual: [name lastPathComponent]] == NO)
 	    {
-	      [str appendString: @"\" up=\""];
+	      [str appendString: @" up=\""];
 	      [str appendString: up];
+	      [str appendString: @"\""];
 	    }
-	  [str appendString: @"\">\n"];
+	  [str appendString: @">\n"];
 	  [str appendString: @"  <head>\n"];
 	  [str appendString: @"    <title>"];
 	  [str appendString: kind];
