@@ -15,7 +15,7 @@
    You should have received a copy of the GNU General Public
    License along with this program; see the file COPYINGv3.
    If not, write to the Free Software Foundation,
-   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+   31 Milk Street #960789 Boston, MA 02196 USA.
 
 */
 
@@ -185,7 +185,7 @@ static NSString *parse_string(NSString **ptr)
 		pos = [l rangeOfString: @"*/"].location;
                 if (pos == NSNotFound)
                   {
-                    fprintf(stderr,"parse error in '%s', missing '*\'\n",
+                    fprintf(stderr,"parse error in '%s', missing '*/'\n",
                       [filename cString]);
                     pos = [l length];
                   }
