@@ -15,7 +15,7 @@
    You should have received a copy of the GNU General Public
    License along with this program; see the file COPYINGv3.
    If not, write to the Free Software Foundation, 
-   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+   31 Milk Street #960789 Boston, MA 02196 USA.
 
 */
 
@@ -33,6 +33,10 @@
 #include "SourceEntry.h"
 #include "StringsEntry.h"
 
+#ifdef _MSC_VER
+#define strdup _strdup
+#endif
+
 
 int verbose, aggressive_import, aggressive_match, aggressive_remove;
 
@@ -596,7 +600,7 @@ int main(int argc, char **argv)
       return 1;
     }
 
-  source_entries = [[NSMutableDictionary alloc] init];
+  source_entries = [NSMutableDictionary dictionary];
   error = 0;
   {
     int i;
