@@ -14,7 +14,7 @@
    You should have received a copy of the GNU General Public
    License along with this program; see the file COPYINGv3.
    If not, write to the Free Software Foundation,
-   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+   31 Milk Street #960789 Boston, MA 02196 USA.
 
    */
 
@@ -27,11 +27,13 @@
 #import	"Foundation/NSUserDefaults.h"
 #import	"Foundation/NSFileHandle.h"
 #import	"Foundation/NSAutoreleasePool.h"
+#import	"Foundation/NSPropertyList.h"
 
 
-/** <p>This tool converts a binary serialised property list to a text
-    representation.
-</p> */
+/** <p>This tool converts a serialised property list to a text
+ *  representation.
+ * </p>
+ */
 int
 main(int argc, char** argv, char **env)
 {
@@ -69,15 +71,22 @@ main(int argc, char** argv, char **env)
 
 	  NS_DURING
 	    {
-	      NSData	*myData;
-	      NSString	*myString;
-	      id	result;
+	      NSData	                *myData;
+	      NSString	                *myString;
+	      id	                result;
+              NSPropertyListFormat      aFormat;
+              NSError                   *anError;
 
 	      myData = [NSData dataWithContentsOfFile: file];
-	      result = [NSDeserializer deserializePropertyListFromData: myData
-						     mutableContainers: NO];
+	      result = [NSPropertyListSerialization
+                propertyListWithData: myData
+                             options: NSPropertyListImmutable
+                              format: &aFormat
+                               error: &anError];
 	      if (result == nil)
-		GSPrintf(stderr, @"Loading '%@' - nil property list\n", file);
+                {
+                  GSPrintf(stderr, @"Loading '%@' - %@\n", file, anError);
+                }
 	      else
 		{
 		  NSFileHandle	*out;
